unit fmod;

{ See http://www.drbob42.com/delphi/headconv.htm for C to Pascal conversion }
interface

{$I fmod.inc.pas}

(*
    FMOD version number.  Check this against FMOD::System::getVersion.
    0xaaaabbcc -> aaaa = major version number.  bb = minor version number.  cc = development version number.
*)

const
  FMOD_VERSION = $00043403;

(*
    FMOD types.
*)

type
  FMOD_BOOL          = LongBool;
  FMOD_SYSTEM        = Pointer;
  FMOD_SOUND         = Pointer;
  FMOD_CHANNEL       = Pointer;
  FMOD_CHANNELGROUP  = Pointer;
  FMOD_SOUNDGROUP    = Pointer;
  FMOD_REVERB        = Pointer;
  FMOD_DSP           = Pointer;
  FMOD_DSPCONNECTION = Pointer;
  FMOD_POLYGON       = Pointer;
  FMOD_GEOMETRY      = Pointer;
  FMOD_SYNCPOINT     = Pointer;
  FMOD_MODE          = Cardinal;
  FMOD_TIMEUNIT      = Cardinal;
  FMOD_INITFLAGS     = Cardinal;
  FMOD_CAPS          = Cardinal;
  FMOD_DEBUGLEVEL    = Cardinal;
  FMOD_MEMORY_TYPE   = Cardinal;

(*
[ENUM]
[
    [DESCRIPTION]
    error codes.  Returned from every function.

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
]
*)
type
  FMOD_RESULT =
  (
    FMOD_OK,                        (* No errors. *)
    FMOD_ERR_ALREADYLOCKED,         (* Tried to call lock a second time before unlock was called. *)
    FMOD_ERR_BADCOMMAND,            (* Tried to call a function on a data type that does not allow this type of functionality (ie calling Sound::lock on a streaming sound). *)
    FMOD_ERR_CDDA_DRIVERS,          (* Neither NTSCSI nor ASPI could be initialised. *)
    FMOD_ERR_CDDA_INIT,             (* An error occurred while initialising the CDDA subsystem. *)
    FMOD_ERR_CDDA_INVALID_DEVICE,   (* Couldn't find the specified device. *)
    FMOD_ERR_CDDA_NOAUDIO,          (* No audio tracks on the specified disc. *)
    FMOD_ERR_CDDA_NODEVICES,        (* No CD/DVD devices were found. *)
    FMOD_ERR_CDDA_NODISC,           (* No disc present in the specified drive. *)
    FMOD_ERR_CDDA_READ,             (* A CDDA read error occurred. *)
    FMOD_ERR_CHANNEL_ALLOC,         (* Error trying to allocate a channel. *)
    FMOD_ERR_CHANNEL_STOLEN,        (* The specified channel has been reused to play another sound. *)
    FMOD_ERR_COM,                   (* A Win32 COM related error occured. COM failed to initialize or a QueryInterface failed meaning a Windows codec or driver was not installed properly. *)
    FMOD_ERR_DMA,                   (* DMA Failure.  See debug output for more information. *)
    FMOD_ERR_DSP_CONNECTION,        (* DSP connection error.  Connection possibly caused a cyclic dependancy.  Or tried to connect a tree too many units deep (more than 128). *)
    FMOD_ERR_DSP_FORMAT,            (* DSP Format error.  A DSP unit may have attempted to connect to this network with the wrong format. *)
    FMOD_ERR_DSP_NOTFOUND,          (* DSP connection error.  Couldn't find the DSP unit specified. *)
    FMOD_ERR_DSP_RUNNING,           (* DSP error.  Cannot perform this operation while the network is in the middle of running.  This will most likely happen if a connection or disconnection is attempted in a DSP callback. *)
    FMOD_ERR_DSP_TOOMANYCONNECTIONS,(* DSP connection error.  The unit being connected to or disconnected should only have 1 input or output. *)
    FMOD_ERR_FILE_BAD,              (* Error loading file. *)
    FMOD_ERR_FILE_COULDNOTSEEK,     (* Couldn't perform seek operation.  This is a limitation of the medium (ie netstreams) or the file format. *)
    FMOD_ERR_FILE_DISKEJECTED,      (* Media was ejected while reading. *)
    FMOD_ERR_FILE_EOF,              (* End of file unexpectedly reached while trying to read essential data (truncated data?). *)
    FMOD_ERR_FILE_NOTFOUND,         (* File not found. *)
    FMOD_ERR_FILE_UNWANTED,         (* Unwanted file access occured. *)
    FMOD_ERR_FORMAT,                (* Unsupported file or audio format. *)
    FMOD_ERR_HTTP,                  (* A HTTP error occurred. This is a catch-all for HTTP errors not listed elsewhere. *)
    FMOD_ERR_HTTP_ACCESS,           (* The specified resource requires authentication or is forbidden. *)
    FMOD_ERR_HTTP_PROXY_AUTH,       (* Proxy authentication is required to access the specified resource. *)
    FMOD_ERR_HTTP_SERVER_ERROR,     (* A HTTP server error occurred. *)
    FMOD_ERR_HTTP_TIMEOUT,          (* The HTTP request timed out. *)
    FMOD_ERR_INITIALIZATION,        (* FMOD was not initialized correctly to support this function. *)
    FMOD_ERR_INITIALIZED,           (* Cannot call this command after System::init. *)
    FMOD_ERR_INTERNAL,              (* An error occured that wasn't supposed to.  Contact support. *)
    FMOD_ERR_INVALID_ADDRESS,       (* On Xbox 360, this memory address passed to FMOD must be physical, (ie allocated with XPhysicalAlloc.) *)
    FMOD_ERR_INVALID_FLOAT,         (* Value passed in was a NaN, Inf or denormalized float. *)
    FMOD_ERR_INVALID_HANDLE,        (* An invalid object handle was used. *)
    FMOD_ERR_INVALID_PARAM,         (* An invalid parameter was passed to this function. *)
    FMOD_ERR_INVALID_POSITION,      (* An invalid seek position was passed to this function. *)
    FMOD_ERR_INVALID_SPEAKER,       (* An invalid speaker was passed to this function based on the current speaker mode. *)
    FMOD_ERR_INVALID_SYNCPOINT,     (* The syncpoint did not come from this sound handle. *)
    FMOD_ERR_INVALID_VECTOR,        (* The vectors passed in are not unit length, or perpendicular. *)
    FMOD_ERR_IRX,                   (* PS2 only.  fmodex.irx failed to initialize.  This is most likely because you forgot to load it. *)
    FMOD_ERR_MAXAUDIBLE,            (* Reached maximum audible playback count for this sound's soundgroup. *)
    FMOD_ERR_MEMORY,                (* Not enough memory or resources. *)
    FMOD_ERR_MEMORY_CANTPOINT,      (* Can't use FMOD_OPENMEMORY_POINT on non PCM source data, or non mp3/xma/adpcm data if FMOD_CREATECOMPRESSEDSAMPLE was used. *)
    FMOD_ERR_MEMORY_IOP,            (* PS2 only.  Not enough memory or resources on PlayStation 2 IOP ram. *)
    FMOD_ERR_MEMORY_SRAM,           (* Not enough memory or resources on console sound ram. *)
    FMOD_ERR_NEEDS2D,               (* Tried to call a command on a 3d sound when the command was meant for 2d sound. *)
    FMOD_ERR_NEEDS3D,               (* Tried to call a command on a 2d sound when the command was meant for 3d sound. *)
    FMOD_ERR_NEEDSHARDWARE,         (* Tried to use a feature that requires hardware support.  (ie trying to play a VAG compressed sound in software on PS2). *)
    FMOD_ERR_NEEDSSOFTWARE,         (* Tried to use a feature that requires the software engine.  Software engine has either been turned off, or command was executed on a hardware channel which does not support this feature. *)
    FMOD_ERR_NET_CONNECT,           (* Couldn't connect to the specified host. *)
    FMOD_ERR_NET_SOCKET_ERROR,      (* A socket error occurred.  This is a catch-all for socket-related errors not listed elsewhere. *)
    FMOD_ERR_NET_URL,               (* The specified URL couldn't be resolved. *)
    FMOD_ERR_NET_WOULD_BLOCK,       (* Operation on a non-blocking socket could not complete immediately. *)
    FMOD_ERR_NOTREADY,              (* Operation could not be performed because specified sound/DSP connection is not ready. *)
    FMOD_ERR_OUTPUT_ALLOCATED,      (* Error initializing output device, but more specifically, the output device is already in use and cannot be reused. *)
    FMOD_ERR_OUTPUT_CREATEBUFFER,   (* Error creating hardware sound buffer. *)
    FMOD_ERR_OUTPUT_DRIVERCALL,     (* A call to a standard soundcard driver failed, which could possibly mean a bug in the driver or resources were missing or exhausted. *)
    FMOD_ERR_OUTPUT_ENUMERATION,    (* Error enumerating the available driver list. List may be inconsistent due to a recent device addition or removal. *)
    FMOD_ERR_OUTPUT_FORMAT,         (* Soundcard does not support the minimum features needed for this soundsystem (16bit stereo output). *)
    FMOD_ERR_OUTPUT_INIT,           (* Error initializing output device. *)
    FMOD_ERR_OUTPUT_NOHARDWARE,     (* FMOD_HARDWARE was specified but the sound card does not have the resources necessary to play it. *)
    FMOD_ERR_OUTPUT_NOSOFTWARE,     (* Attempted to create a software sound but no software channels were specified in System::init. *)
    FMOD_ERR_PAN,                   (* Panning only works with mono or stereo sound sources. *)
    FMOD_ERR_PLUGIN,                (* An unspecified error has been returned from a 3rd party plugin. *)
    FMOD_ERR_PLUGIN_INSTANCES,      (* The number of allowed instances of a plugin has been exceeded. *)
    FMOD_ERR_PLUGIN_MISSING,        (* A requested output, dsp unit type or codec was not available. *)
    FMOD_ERR_PLUGIN_RESOURCE,       (* A resource that the plugin requires cannot be found. (ie the DLS file for MIDI playback) *)
    FMOD_ERR_PRELOADED,             (* The specified sound is still in use by the event system, call EventSystem::unloadFSB before trying to release it. *)
    FMOD_ERR_PROGRAMMERSOUND,       (* The specified sound is still in use by the event system, wait for the event which is using it finish with it. *)
    FMOD_ERR_RECORD,                (* An error occured trying to initialize the recording device. *)
    FMOD_ERR_REVERB_INSTANCE,       (* Specified Instance in FMOD_REVERB_PROPERTIES couldn't be set. Most likely because it is an invalid instance number, or another application has locked the EAX4 FX slot. *)
    FMOD_ERR_SUBSOUND_ALLOCATED,    (* This subsound is already being used by another sound, you cannot have more than one parent to a sound.  Null out the other parent's entry first. *)
    FMOD_ERR_SUBSOUND_CANTMOVE,     (* Shared subsounds cannot be replaced or moved from their parent stream, such as when the parent stream is an FSB file. *)
    FMOD_ERR_SUBSOUND_MODE,         (* The subsound's mode bits do not match with the parent sound's mode bits.  See documentation for function that it was called with. *)
    FMOD_ERR_SUBSOUNDS,             (* The error occured because the sound referenced contains subsounds when it shouldn't have, or it doesn't contain subsounds when it should have.  The operation may also not be able to be performed on a parent sound, or a parent sound was played without setting up a sentence first. *)
    FMOD_ERR_TAGNOTFOUND,           (* The specified tag could not be found or there are no tags. *)
    FMOD_ERR_TOOMANYCHANNELS,       (* The sound created exceeds the allowable input channel count.  This can be increased using the maxinputchannels parameter in System::setSoftwareFormat. *)
    FMOD_ERR_UNIMPLEMENTED,         (* Something in FMOD hasn't been implemented when it should be! contact support! *)
    FMOD_ERR_UNINITIALIZED,         (* This command failed because System::init or System::setDriver was not called. *)
    FMOD_ERR_UNSUPPORTED,           (* A command issued was not supported by this object.  Possibly a plugin without certain callbacks specified. *)
    FMOD_ERR_UPDATE,                (* An error caused by System::update occured. *)
    FMOD_ERR_VERSION,               (* The version number of this file format is not supported. *)

    FMOD_ERR_EVENT_FAILED,          (* An Event failed to be retrieved, most likely due to 'just fail' being specified as the max playbacks behavior. *)
    FMOD_ERR_EVENT_INFOONLY,        (* Can't execute this command on an EVENT_INFOONLY event. *)
    FMOD_ERR_EVENT_INTERNAL,        (* An error occured that wasn't supposed to.  See debug log for reason. *)
    FMOD_ERR_EVENT_MAXSTREAMS,      (* Event failed because 'Max streams' was hit when FMOD_EVENT_INIT_FAIL_ON_MAXSTREAMS was specified. *)
    FMOD_ERR_EVENT_MISMATCH,        (* FSB mismatches the FEV it was compiled with, the stream/sample mode it was meant to be created with was different, or the FEV was built for a different platform. *)
    FMOD_ERR_EVENT_NAMECONFLICT,    (* A category with the same name already exists. *)
    FMOD_ERR_EVENT_NOTFOUND,        (* The requested event, event group, event category or event property could not be found. *)
    FMOD_ERR_EVENT_NEEDSSIMPLE,     (* Tried to call a function on a complex event that's only supported by simple events. *)
    FMOD_ERR_EVENT_GUIDCONFLICT,    (* An event with the same GUID already exists. *)
    FMOD_ERR_EVENT_ALREADY_LOADED,  (* The specified project has already been loaded. Having multiple copies of the same project loaded simultaneously is forbidden. *)

    FMOD_ERR_MUSIC_UNINITIALIZED,   (* Music system is not initialized probably because no music data is loaded. *)
    FMOD_ERR_MUSIC_NOTFOUND,        (* The requested music entity could not be found. *)
    FMOD_ERR_MUSIC_NOCALLBACK,      (* The music callback is required, but it has not been set. *)

    FMOD_RESULT_FORCEINT = 65536    (* Makes sure this enum is signed 32bit. *)
);

(*
[STRUCTURE] 
[
    [DESCRIPTION]   
    Structure describing a point in 3D space.

    [REMARKS]
    FMOD uses a left handed co-ordinate system by default.
    To use a right handed co-ordinate system specify FMOD_INIT_3D_RIGHTHANDED from FMOD_INITFLAGS in System::init.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    System::set3DListenerAttributes
    System::get3DListenerAttributes
    Channel::set3DAttributes
    Channel::get3DAttributes
    Channel::set3DCustomRolloff
    Channel::get3DCustomRolloff
    Sound::set3DCustomRolloff
    Sound::get3DCustomRolloff
    Geometry::addPolygon
    Geometry::setPolygonVertex
    Geometry::getPolygonVertex
    Geometry::setRotation
    Geometry::getRotation
    Geometry::setPosition
    Geometry::getPosition
    Geometry::setScale
    Geometry::getScale
    FMOD_INITFLAGS
]
*)
type
  FMOD_VECTOR = record
    x, y, z: Single;
  end;

(*
[STRUCTURE]
[
    [DESCRIPTION]
    Structure describing a globally unique identifier.

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::getDriverInfo
]
*)
type
  FMOD_GUID = record
    Data1 : Cardinal;  (* Specifies the first 8 hexadecimal digits of the GUID *)
    Data2 : Word;      (* Specifies the first group of 4 hexadecimal digits.   *)
    Data3 : Word;      (* Specifies the second group of 4 hexadecimal digits.  *)
    Data4 : String[8]; (* Array of 8 bytes. The first 2 bytes contain the third group of 4 hexadecimal digits. The remaining 6 bytes contain the final 12 hexadecimal digits. *)
  end;

(*
[STRUCTURE] 
[
    [DESCRIPTION]
    Structure that is passed into FMOD_FILE_ASYNCREADCALLBACK.  Use the information in this structure to perform

    [REMARKS]
    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.
    
    Instructions: write to 'buffer', and 'bytesread' <b>BEFORE</b> setting 'result'.  
    As soon as result is set, FMOD will asynchronously continue internally using the data provided in this structure.
    
    Set 'result' to the result expected from a normal file read callback.
    If the read was successful, set it to FMOD_OK.
    If it read some data but hit the end of the file, set it to FMOD_ERR_FILE_EOF.
    If a bad error occurred, return FMOD_ERR_FILE_BAD
    If a disk was ejected, return FMOD_ERR_FILE_DISKEJECTED.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    FMOD_FILE_ASYNCREADCALLBACK
    FMOD_FILE_ASYNCCANCELCALLBACK
]
*)
type
  FMOD_ASYNCREADINFO = record
    handle    : Pointer;     (* [r] The file handle that was filled out in the open callback. *)
    offset    : Cardinal;    (* [r] Seek position, make sure you read from this file offset. *)
    sizebytes : Cardinal;    (* [r] how many bytes requested for read. *)
    priority  : Integer;     (* [r] 0 = low importance.  100 = extremely important (ie 'must read now or stuttering may occur') *)

    buffer    : pointer;     (* [w] Buffer to read file data into. *)
    bytesread : Cardinal;    (* [w] Fill this in before setting result code to tell FMOD how many bytes were read. *)
    result    : FMOD_RESULT; (* [r/w] Result code, FMOD_OK tells the system it is ready to consume the data.  Set this last!  Default value = FMOD_ERR_NOTREADY. *)

    userdata  : Pointer;     (* [r] User data pointer. *)
  end;

(*
[ENUM]
[
    [DESCRIPTION]   
    These output types are used with System::setOutput / System::getOutput, to choose which output method to use.
  
    [REMARKS]
    To pass information to the driver when initializing fmod use the extradriverdata parameter in System::init for the following reasons.
    - FMOD_OUTPUTTYPE_WAVWRITER - extradriverdata is a pointer to a char * filename that the wav writer will output to.
    - FMOD_OUTPUTTYPE_WAVWRITER_NRT - extradriverdata is a pointer to a char * filename that the wav writer will output to.
    - FMOD_OUTPUTTYPE_DSOUND - extradriverdata is a pointer to a HWND so that FMOD can set the focus on the audio for a particular window.
    - FMOD_OUTPUTTYPE_PS2 - extradriverdata is a pointer to a FMOD_PS2_EXTRADRIVERDATA struct. This can be found in fmodps2.h.
    - FMOD_OUTPUTTYPE_PS3 - extradriverdata is a pointer to a FMOD_PS3_EXTRADRIVERDATA struct. This can be found in fmodps3.h.
    - FMOD_OUTPUTTYPE_GC - extradriverdata is a pointer to a FMOD_GC_INFO struct. This can be found in fmodgc.h.
    - FMOD_OUTPUTTYPE_WII - extradriverdata is a pointer to a FMOD_WII_INFO struct. This can be found in fmodwii.h.
    - FMOD_OUTPUTTYPE_ALSA - extradriverdata is a pointer to a FMOD_LINUX_EXTRADRIVERDATA struct. This can be found in fmodlinux.h.
    
    Currently these are the only FMOD drivers that take extra information.  Other unknown plugins may have different requirements.
    
    Note! If FMOD_OUTPUTTYPE_WAVWRITER_NRT or FMOD_OUTPUTTYPE_NOSOUND_NRT are used, and if the System::update function is being called
    very quickly (ie for a non realtime decode) it may be being called too quickly for the FMOD streamer thread to respond to.  
    The result will be a skipping/stuttering output in the captured audio.
    
    To remedy this, disable the FMOD Ex streamer thread, and use FMOD_INIT_STREAM_FROM_UPDATE to avoid skipping in the output stream,
    as it will lock the mixer and the streamer together in the same thread.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    System::setOutput
    System::getOutput
    System::setSoftwareFormat
    System::getSoftwareFormat
    System::init
    System::update
    FMOD_INITFLAGS
]
*)

type
  FMOD_OUTPUTTYPE =
  (
    FMOD_OUTPUTTYPE_AUTODETECT,      (* Picks the best output mode for the platform.  This is the default. *)

    FMOD_OUTPUTTYPE_UNKNOWN,         (* All                   - 3rd party plugin, unknown.  This is for use with System::getOutput only. *)
    FMOD_OUTPUTTYPE_NOSOUND,         (* All                   - All calls in this mode succeed but make no sound. *)
    FMOD_OUTPUTTYPE_WAVWRITER,       (* All                   - Writes output to fmodoutput.wav by default.  Use the 'extradriverdata' parameter in System::init, by simply passing the filename as a string, to set the wav filename. *)
    FMOD_OUTPUTTYPE_NOSOUND_NRT,     (* All                   - Non-realtime version of FMOD_OUTPUTTYPE_NOSOUND.  User can drive mixer with System::update at whatever rate they want. *)
    FMOD_OUTPUTTYPE_WAVWRITER_NRT,   (* All                   - Non-realtime version of FMOD_OUTPUTTYPE_WAVWRITER.  User can drive mixer with System::update at whatever rate they want. *)

    FMOD_OUTPUTTYPE_DSOUND,          (* Win32/Win64           - DirectSound output.  (Default on Windows OS earlier than Vista) *)
    FMOD_OUTPUTTYPE_WINMM,           (* Win32/Win64           - Windows Multimedia output. (Older audio interface, larger latency on output) *)
    FMOD_OUTPUTTYPE_WASAPI,          (* Win32                 - Windows Audio Session API. (Default on Windows Vista/Windows 7) *)
    FMOD_OUTPUTTYPE_ASIO,            (* Win32                 - Low latency ASIO 2.0 driver. *)
    FMOD_OUTPUTTYPE_OSS,             (* Linux/Linux64         - Open Sound System output. (Default on Linux/Linux64) *)
    FMOD_OUTPUTTYPE_ALSA,            (* Linux/Linux64         - Advanced Linux Sound Architecture output. *)
    FMOD_OUTPUTTYPE_ESD,             (* Linux/Linux64         - Enlightment Sound Daemon output. *)
    FMOD_OUTPUTTYPE_PULSEAUDIO,      (* Linux/Linux64         - PulseAudio output. *)
    FMOD_OUTPUTTYPE_COREAUDIO,       (* Mac                   - Macintosh CoreAudio output.  (Default on Mac) *)
    FMOD_OUTPUTTYPE_PS2,             (* PS2                   - Native hardware output. (Default on PS2) *)
    FMOD_OUTPUTTYPE_PS3,             (* PS3                   - Native hardware output. (Default on PS3) *)
    FMOD_OUTPUTTYPE_XBOX360,         (* Xbox 360              - Native hardware output. (Default on Xbox 360) *)
    FMOD_OUTPUTTYPE_PSP,             (* PSP                   - Native hardware output. (Default on PSP) *)
    FMOD_OUTPUTTYPE_WII,             (* Wii               - Native hardware output. (Default on Wii) *)
    FMOD_OUTPUTTYPE_ANDROID,         (* Android               - Native android output. *)
    FMOD_OUTPUTTYPE_3DS,             (* 3DS                   - Native 3DS output *)

    FMOD_OUTPUTTYPE_MAX,             (* Maximum number of output types supported. *)
    FMOD_OUTPUTTYPE_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );

(*
[DEFINE]
[
    [NAME]
    FMOD_CAPS

    [DESCRIPTION]
    Bit fields to use with System::getDriverCaps to determine the capabilities of a card / output device.

    [REMARKS]
    It is important to check FMOD_CAPS_HARDWARE_EMULATED on windows machines, to then adjust System::setDSPBufferSize to (1024, 10) to compensate for the higher latency.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::getDriverCaps
    System::setDSPBufferSize
]
*)
const
  FMOD_CAPS_NONE                   = $00000000; (* Device has no special capabilities. *)
  FMOD_CAPS_HARDWARE               = $00000001; (* Device supports hardware mixing. *)
  FMOD_CAPS_HARDWARE_EMULATED      = $00000002; (* User has device set to 'Hardware acceleration = off' in control panel, and now extra 200ms latency is incurred. *)
  FMOD_CAPS_OUTPUT_MULTICHANNEL    = $00000004; (* Device can do multichannel output, ie greater than 2 channels. *)
  FMOD_CAPS_OUTPUT_FORMAT_PCM8     = $00000008; (* Device can output to 8bit integer PCM. *)
  FMOD_CAPS_OUTPUT_FORMAT_PCM16    = $00000010; (* Device can output to 16bit integer PCM. *)
  FMOD_CAPS_OUTPUT_FORMAT_PCM24    = $00000020; (* Device can output to 24bit integer PCM. *)
  FMOD_CAPS_OUTPUT_FORMAT_PCM32    = $00000040; (* Device can output to 32bit integer PCM. *)
  FMOD_CAPS_OUTPUT_FORMAT_PCMFLOAT = $00000080; (* Device can output to 32bit floating point PCM. *)
  FMOD_CAPS_REVERB_EAX2            = $00000100; (* Device supports EAX2 reverb. *)
  FMOD_CAPS_REVERB_EAX3            = $00000200; (* Device supports EAX3 reverb. *)
  FMOD_CAPS_REVERB_EAX4            = $00000400; (* Device supports EAX4 reverb  *)
  FMOD_CAPS_REVERB_EAX5            = $00000800; (* Device supports EAX5 reverb  *)
  FMOD_CAPS_REVERB_I3DL2           = $00001000; (* Device supports I3DL2 reverb. *)
  FMOD_CAPS_REVERB_LIMITED         = $00002000; (* Device supports some form of limited hardware reverb, maybe parameterless and only selectable by environment. *)

(*
[DEFINE] 
[
    [NAME]
    FMOD_DEBUGLEVEL

    [DESCRIPTION]   
    Bit fields to use with FMOD::Debug_SetLevel / FMOD::Debug_GetLevel to control the level of tty debug output with logging versions of FMOD (fmodL).

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    Debug_SetLevel 
    Debug_GetLevel
]
*)
const
  FMOD_DEBUG_LEVEL_NONE           = $00000000;
  FMOD_DEBUG_LEVEL_LOG            = $00000001; (* Will display generic logging messages. *)
  FMOD_DEBUG_LEVEL_ERROR          = $00000002; (* Will display errors. *)
  FMOD_DEBUG_LEVEL_WARNING        = $00000004; (* Will display warnings that are not fatal. *)
  FMOD_DEBUG_LEVEL_HINT           = $00000008; (* Will hint to you if there is something possibly better you could be doing. *)
  FMOD_DEBUG_LEVEL_ALL            = $000000FF;
  FMOD_DEBUG_TYPE_MEMORY          = $00000100; (* Show FMOD memory related logging messages. *)
  FMOD_DEBUG_TYPE_THREAD          = $00000200; (* Show FMOD thread related logging messages. *)
  FMOD_DEBUG_TYPE_FILE            = $00000400; (* Show FMOD file system related logging messages. *)
  FMOD_DEBUG_TYPE_NET             = $00000800; (* Show FMOD network related logging messages. *)
  FMOD_DEBUG_TYPE_EVENT           = $00001000; (* Show FMOD Event related logging messages. *)
  FMOD_DEBUG_TYPE_ALL             = $0000FFFF;
  FMOD_DEBUG_DISPLAY_TIMESTAMPS   = $01000000; (* Display the timestamp of the log entry in milliseconds. *)
  FMOD_DEBUG_DISPLAY_LINENUMBERS  = $02000000; (* Display the FMOD Ex source code line numbers, for debugging purposes. *)
  FMOD_DEBUG_DISPLAY_COMPRESS     = $04000000; (* If a message is repeated more than 5 times it will stop displaying it and instead display the number of times the message was logged. *)
  FMOD_DEBUG_DISPLAY_THREAD       = $08000000; (* Display the thread ID of the calling function that caused this log entry to appear. *)
  FMOD_DEBUG_DISPLAY_ALL          = $0F000000;
  FMOD_DEBUG_ALL                  = $FFFFFFFF;

(*
[DEFINE] 
[
    [NAME]
    FMOD_MEMORY_TYPE

    [DESCRIPTION]
    Bit fields for memory allocation type being passed into FMOD memory callbacks.

    [REMARKS]
    Remember this is a bitfield.  You may get more than 1 bit set (ie physical + persistent) so do not simply switch on the types!  You must check each bit individually or clear out the bits that you do not want within the callback.
    Bits can be excluded if you want during Memory_Initialize so that you never get them.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    FMOD_MEMORY_ALLOCCALLBACK
    FMOD_MEMORY_REALLOCCALLBACK
    FMOD_MEMORY_FREECALLBACK
    Memory_Initialize

]
*)
const
  FMOD_MEMORY_NORMAL           = $00000000; (* Standard memory. *)
  FMOD_MEMORY_STREAM_FILE      = $00000001; (* Stream file buffer, size controllable with System::setStreamBufferSize. *)
  FMOD_MEMORY_STREAM_DECODE    = $00000002; (* Stream decode buffer, size controllable with FMOD_CREATESOUNDEXINFO::decodebuffersize. *)
  FMOD_MEMORY_SAMPLEDATA       = $00000004; (* Sample data buffer.  Raw audio data, usually PCM/MPEG/ADPCM/XMA data. *)
  FMOD_MEMORY_DSP_OUTPUTBUFFER = $00000008; (* DSP memory block allocated when more than 1 output exists on a DSP node. *)
  FMOD_MEMORY_XBOX360_PHYSICAL = $00100000; (* Requires XPhysicalAlloc / XPhysicalFree. *)
  FMOD_MEMORY_PERSISTENT       = $00200000; (* Persistent memory. Memory will be freed when System::release is called. *)
  FMOD_MEMORY_SECONDARY        = $00400000; (* Secondary memory. Allocation should be in secondary memory. For example RSX on the PS3. *)
  FMOD_MEMORY_ALL              = $FFFFFFFF;

(*
[ENUM]
[
    [DESCRIPTION]
    These are speaker types defined for use with the System::setSpeakerMode or System::getSpeakerMode command.

    [REMARKS]
    These are important notes on speaker modes in regards to sounds created with FMOD_SOFTWARE.
    Note below the phrase 'sound channels' is used.  These are the subchannels inside a sound, they are not related and 
    have nothing to do with the FMOD class "Channel".
    For example a mono sound has 1 sound channel, a stereo sound has 2 sound channels, and an AC3 or 6 channel wav file have 6 "sound channels".
    
    FMOD_SPEAKERMODE_RAW
    ---------------------
    This mode is for output devices that are not specifically mono/stereo/quad/surround/5.1 or 7.1, but are multichannel.
    Use System::setSoftwareFormat to specify the number of speakers you want to address, otherwise it will default to 2 (stereo).
    Sound channels map to speakers sequentially, so a mono sound maps to output speaker 0, stereo sound maps to output speaker 0 & 1.
    The user assumes knowledge of the speaker order.  FMOD_SPEAKER enumerations may not apply, so raw channel indices should be used.
    Multichannel sounds map input channels to output channels 1:1. 
    Channel::setPan and Channel::setSpeakerMix do not work.
    Speaker levels must be manually set with Channel::setSpeakerLevels.
    
    FMOD_SPEAKERMODE_MONO
    ---------------------
    This mode is for a 1 speaker arrangement.
    Panning does not work in this speaker mode.
    Mono, stereo and multichannel sounds have each sound channel played on the one speaker unity.
    Mix behavior for multichannel sounds can be set with Channel::setSpeakerLevels.
    Channel::setSpeakerMix does not work.
    
    FMOD_SPEAKERMODE_STEREO
    -----------------------
    This mode is for 2 speaker arrangements that have a left and right speaker.
    - Mono sounds default to an even distribution between left and right.  They can be panned with Channel::setPan.
    - Stereo sounds default to the middle, or full left in the left speaker and full right in the right speaker.  
    - They can be cross faded with Channel::setPan.
    - Multichannel sounds have each sound channel played on each speaker at unity.
    - Mix behavior for multichannel sounds can be set with Channel::setSpeakerLevels.
    - Channel::setSpeakerMix works but only front left and right parameters are used, the rest are ignored.
    
    FMOD_SPEAKERMODE_QUAD
    ------------------------
    This mode is for 4 speaker arrangements that have a front left, front right, rear left and a rear right speaker.
    - Mono sounds default to an even distribution between front left and front right.  They can be panned with Channel::setPan.
    - Stereo sounds default to the left sound channel played on the front left, and the right sound channel played on the front right.
    - They can be cross faded with Channel::setPan.
    - Multichannel sounds default to all of their sound channels being played on each speaker in order of input.
    - Mix behavior for multichannel sounds can be set with Channel::setSpeakerLevels.
    - Channel::setSpeakerMix works but side left, side right, center and lfe are ignored.
    
    FMOD_SPEAKERMODE_SURROUND
    ------------------------
    This mode is for 5 speaker arrangements that have a left/right/center/rear left/rear right.
    - Mono sounds default to the center speaker.  They can be panned with Channel::setPan.
    - Stereo sounds default to the left sound channel played on the front left, and the right sound channel played on the front right.  
    - They can be cross faded with Channel::setPan.
    - Multichannel sounds default to all of their sound channels being played on each speaker in order of input.  
    - Mix behavior for multichannel sounds can be set with Channel::setSpeakerLevels.
    - Channel::setSpeakerMix works but side left / side right are ignored.
    
    FMOD_SPEAKERMODE_5POINT1
    ------------------------
    This mode is for 5.1 speaker arrangements that have a left/right/center/rear left/rear right and a subwoofer speaker.
    - Mono sounds default to the center speaker.  They can be panned with Channel::setPan.
    - Stereo sounds default to the left sound channel played on the front left, and the right sound channel played on the front right.  
    - They can be cross faded with Channel::setPan.
    - Multichannel sounds default to all of their sound channels being played on each speaker in order of input.  
    - Mix behavior for multichannel sounds can be set with Channel::setSpeakerLevels.
    - Channel::setSpeakerMix works but side left / side right are ignored.
    
    FMOD_SPEAKERMODE_7POINT1
    ------------------------
    This mode is for 7.1 speaker arrangements that have a left/right/center/rear left/rear right/side left/side right 
    and a subwoofer speaker.
    - Mono sounds default to the center speaker.  They can be panned with Channel::setPan.
    - Stereo sounds default to the left sound channel played on the front left, and the right sound channel played on the front right.  
    - They can be cross faded with Channel::setPan.
    - Multichannel sounds default to all of their sound channels being played on each speaker in order of input.  
    - Mix behavior for multichannel sounds can be set with Channel::setSpeakerLevels.
    - Channel::setSpeakerMix works and every parameter is used to set the balance of a sound in any speaker.
    
    FMOD_SPEAKERMODE_PROLOGIC
    ------------------------------------------------------
    This mode is for mono, stereo, 5.1 and 7.1 speaker arrangements, as it is backwards and forwards compatible with stereo, 
    but to get a surround effect a Dolby Prologic or Prologic 2 hardware decoder / amplifier is needed.
    Pan behavior is the same as FMOD_SPEAKERMODE_5POINT1.
    
    If this function is called the numoutputchannels setting in System::setSoftwareFormat is overwritten.
    
    For 3D sounds, panning is determined at runtime by the 3D subsystem based on the speaker mode to determine which speaker the 
    sound should be placed in.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::setSpeakerMode
    System::getSpeakerMode
    System::getDriverCaps
    System::setSoftwareFormat
    Channel::setSpeakerLevels
]
*)
type
  FMOD_SPEAKERMODE =
  (
    FMOD_SPEAKERMODE_RAW,              (* There is no specific speakermode.  Sound channels are mapped in order of input to output.  Use System::setSoftwareFormat to specify speaker count. See remarks for more information. *)
    FMOD_SPEAKERMODE_MONO,             (* The speakers are monaural. *)
    FMOD_SPEAKERMODE_STEREO,           (* The speakers are stereo (DEFAULT). *)
    FMOD_SPEAKERMODE_QUAD,             (* 4 speaker setup.  This includes front left, front right, rear left, rear right.  *)
    FMOD_SPEAKERMODE_SURROUND,         (* 5 speaker setup.  This includes front left, front right, center, rear left, rear right. *)
    FMOD_SPEAKERMODE_5POINT1,          (* 5.1 speaker setup.  This includes front left, front right, center, rear left, rear right and a subwoofer. *)
    FMOD_SPEAKERMODE_7POINT1,          (* 7.1 speaker setup.  This includes front left, front right, center, rear left, rear right, side left, side right and a subwoofer. *)
    FMOD_SPEAKERMODE_PROLOGIC,         (* Stereo output, but data is encoded in a way that is picked up by a Prologic/Prologic2 decoder and split into a 5.1 speaker setup. *)

    FMOD_SPEAKERMODE_MAX,              (* Maximum number of speaker modes supported. *)
    FMOD_SPEAKERMODE_FORCEINT = 65536  (* Makes sure this enum is signed 32bit. *)
  );

(*
[ENUM]
[
    [DESCRIPTION]   
    These are speaker types defined for use with the Channel::setSpeakerLevels command.
    It can also be used for speaker placement in the System::set3DSpeakerPosition command.

    [REMARKS]
    If you are using FMOD_SPEAKERMODE_RAW and speaker assignments are meaningless, just cast a raw integer value to this type.
    For example (FMOD_SPEAKER)7 would use the 7th speaker (also the same as FMOD_SPEAKER_SIDE_RIGHT).
    Values higher than this can be used if an output system has more than 8 speaker types / output channels.  15 is the current maximum.
    
    NOTE: On Playstation 3 in 7.1, the extra 2 speakers are not side left/side right, they are 'surround back left'/'surround back right' which
    locate the speakers behind the listener instead of to the sides like on PC.  FMOD_SPEAKER_SBL/FMOD_SPEAKER_SBR are provided to make it 
    clearer what speaker is being addressed on that platform.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    FMOD_SPEAKERMODE
    Channel::setSpeakerLevels
    Channel::getSpeakerLevels
    System::set3DSpeakerPosition
    System::get3DSpeakerPosition
]
*)
type
  FMOD_SPEAKER =
  (
    FMOD_SPEAKER_FRONT_LEFT,
    FMOD_SPEAKER_FRONT_RIGHT,
    FMOD_SPEAKER_FRONT_CENTER,
    FMOD_SPEAKER_LOW_FREQUENCY,
    FMOD_SPEAKER_BACK_LEFT,
    FMOD_SPEAKER_BACK_RIGHT,
    FMOD_SPEAKER_SIDE_LEFT,
    FMOD_SPEAKER_SIDE_RIGHT,

    FMOD_SPEAKER_MAX,                                (* Maximum number of speaker types supported. *)
    FMOD_SPEAKER_MONO     = FMOD_SPEAKER_FRONT_LEFT, (* For use with FMOD_SPEAKERMODE_MONO and Channel::SetSpeakerLevels.  Mapped to same value as FMOD_SPEAKER_FRONT_LEFT. *)
    FMOD_SPEAKER_NULL     = FMOD_SPEAKER_MAX,        (* A non speaker.  Use this to send. *)
    FMOD_SPEAKER_SBL      = FMOD_SPEAKER_SIDE_LEFT,  (* For use with FMOD_SPEAKERMODE_7POINT1 on PS3 where the extra speakers are surround back inside of side speakers. *)
    FMOD_SPEAKER_SBR      = FMOD_SPEAKER_SIDE_RIGHT, (* For use with FMOD_SPEAKERMODE_7POINT1 on PS3 where the extra speakers are surround back inside of side speakers. *)
    FMOD_SPEAKER_FORCEINT = 65536                    (* Makes sure this enum is signed 32bit. *)
  );

(*
[ENUM]
[
    [DESCRIPTION]   
    These are plugin types defined for use with the System::getNumPlugins, 
    System::getPluginInfo and System::unloadPlugin functions.

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::getNumPlugins
    System::getPluginInfo
    System::unloadPlugin
]
*)
type
  FMOD_PLUGINTYPE =
  (
    FMOD_PLUGINTYPE_OUTPUT,          (* The plugin type is an output module.  FMOD mixed audio will play through one of these devices *)
    FMOD_PLUGINTYPE_CODEC,           (* The plugin type is a file format codec.  FMOD will use these codecs to load file formats for playback. *)
    FMOD_PLUGINTYPE_DSP,             (* The plugin type is a DSP unit.  FMOD will use these plugins as part of its DSP network to apply effects to output or generate sound in realtime. *)

    FMOD_PLUGINTYPE_MAX,             (* Maximum number of plugin types supported. *)
    FMOD_PLUGINTYPE_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );
  
(*
[DEFINE]
[
    [NAME]
    FMOD_INITFLAGS

    [DESCRIPTION]   
    Initialization flags.  Use them with System::init in the flags parameter to change various behavior.  

    [REMARKS]
    Use System::setAdvancedSettings to adjust settings for some of the features that are enabled by these flags.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::init
    System::update 
    System::setAdvancedSettings
    Channel::set3DOcclusion
]
*)
const
  FMOD_INIT_NORMAL                    = $00000000; (* All platforms - Initialize normally *)
  FMOD_INIT_STREAM_FROM_UPDATE        = $00000001; (* All platforms - No stream thread is created internally.  Streams are driven from System::update.  Mainly used with non-realtime outputs. *)
  FMOD_INIT_3D_RIGHTHANDED            = $00000002; (* All platforms - FMOD will treat +X as right, +Y as up and +Z as backwards (towards you). *)
  FMOD_INIT_SOFTWARE_DISABLE          = $00000004; (* All platforms - Disable software mixer to save memory.  Anything created with FMOD_SOFTWARE will fail and DSP will not work. *)
  FMOD_INIT_SOFTWARE_OCCLUSION        = $00000008; (* All platforms - All FMOD_SOFTWARE with FMOD_3D based voices will add a software lowpass filter effect into the DSP chain which is automatically used when Channel::set3DOcclusion is used or the geometry API. *)
  FMOD_INIT_SOFTWARE_HRTF             = $00000010; (* All platforms - All FMOD_SOFTWARE with FMOD_3D based voices will add a software lowpass filter effect into the DSP chain which causes sounds to sound duller when the sound goes behind the listener.  Use System::setAdvancedSettings to adjust cutoff frequency. *)
  FMOD_INIT_SOFTWARE_REVERB_LOWMEM    = $00000040; (* All platforms - SFX reverb is run using 22/24khz delay buffers, halving the memory required. *)
  FMOD_INIT_ENABLE_PROFILE            = $00000020; (* All platforms - Enable TCP/IP based host which allows FMOD Designer or FMOD Profiler to connect to it, and view memory, CPU and the DSP network graph in real-time. *)
  FMOD_INIT_VOL0_BECOMES_VIRTUAL      = $00000080; (* All platforms - Any sounds that are 0 volume will go virtual and not be processed except for having their positions updated virtually.  Use System::setAdvancedSettings to adjust what volume besides zero to switch to virtual at. *)
  FMOD_INIT_WASAPI_EXCLUSIVE          = $00000100; (* Win32 Vista only - for WASAPI output - Enable exclusive access to hardware, lower latency at the expense of excluding other applications from accessing the audio hardware. *)
  FMOD_INIT_PS2_DISABLECORE0REVERB    = $00010000; (* PS2 only - Disable reverb on CORE 0 to regain 256k SRAM. *)
  FMOD_INIT_PS2_DISABLECORE1REVERB    = $00020000; (* PS2 only - Disable reverb on CORE 1 to regain 256k SRAM. *)
  FMOD_INIT_PS2_DONTUSESCRATCHPAD     = $00040000; (* PS2 only - Disable FMOD's usage of the scratchpad. *)
  FMOD_INIT_PS2_SWAPDMACHANNELS       = $00080000; (* PS2 only - Changes FMOD from using SPU DMA channel 0 for software mixing, and 1 for sound data upload/file streaming, to 1 and 0 respectively. *)
  FMOD_INIT_PS3_PREFERDTS             = $00800000; (* PS3 only - Prefer DTS over Dolby Digital if both are supported. Note: 8 and 6 channel LPCM is always preferred over both DTS and Dolby Digital. *)
  FMOD_INIT_PS3_FORCE2CHLPCM          = $01000000; (* PS3 only - Force PS3 system output mode to 2 channel LPCM. *)
  FMOD_INIT_DISABLEDOLBY              = $00100000; (* Wii / 3DS - Disable Dolby Pro Logic surround. Speakermode will be set to STEREO even if user has selected surround in the system settings. *)
  FMOD_INIT_SYSTEM_MUSICMUTENOTPAUSE  = $00200000; (* Xbox 360 / PS3 - The "music" channelgroup which by default pauses when custom 360 dashboard / PS3 BGM music is played, can be changed to mute (therefore continues playing) instead of pausing, by using this flag. *)
  FMOD_INIT_SYNCMIXERWITHUPDATE       = $00400000; (* Win32/Wii/PS3/Xbox/Xbox 360 - FMOD Mixer thread is woken up to do a mix when System::update is called rather than waking periodically on its own timer. *)
  FMOD_INIT_DTS_NEURALSURROUND        = $02000000; (* Win32/Mac/Linux/PS3/Xbox360 - Use DTS Neural surround downmixing from 7.1 if speakermode set to FMOD_SPEAKERMODE_STEREO or FMOD_SPEAKERMODE_5POINT1. Always 7.1 -> 5.1 downmix for Xbox360 and PS3.  Internal DSP structure will be set to 7.1. *)
  FMOD_INIT_GEOMETRY_USECLOSEST       = $04000000; (* All platforms - With the geometry engine, only process the closest polygon rather than accumulating all polygons the sound to listener line intersects. *)
  FMOD_INIT_DISABLE_MYEARS            = $08000000; (* Win32 - Disables MyEars HRTF 7.1 downmixing.  MyEars will otherwise be disbaled if speakermode is not set to FMOD_SPEAKERMODE_STEREO or the data file is missing. *)


(*
[ENUM]
[
    [DESCRIPTION]
    These definitions describe the type of song being played.

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Sound::getFormat
]
*)
type
  FMOD_SOUND_TYPE =
  (
    FMOD_SOUND_TYPE_UNKNOWN,         (* 3rd party / unknown plugin format. *)
    FMOD_SOUND_TYPE_AAC,             (* AAC.  Currently unsupported. *)
    FMOD_SOUND_TYPE_AIFF,            (* AIFF. *)
    FMOD_SOUND_TYPE_ASF,             (* Microsoft Advanced Systems Format (ie WMA/ASF/WMV). *)
    FMOD_SOUND_TYPE_AT3,             (* Sony ATRAC 3 format *)
    FMOD_SOUND_TYPE_CDDA,            (* Digital CD audio. *)
    FMOD_SOUND_TYPE_DLS,             (* Sound font / downloadable sound bank. *)
    FMOD_SOUND_TYPE_FLAC,            (* FLAC lossless codec. *)
    FMOD_SOUND_TYPE_FSB,             (* FMOD Sample Bank. *)
    FMOD_SOUND_TYPE_GCADPCM,         (* GameCube ADPCM *)
    FMOD_SOUND_TYPE_IT,              (* Impulse Tracker. *)
    FMOD_SOUND_TYPE_MIDI,            (* MIDI. extracodecdata is a pointer to an FMOD_MIDI_EXTRACODECDATA structure. *)
    FMOD_SOUND_TYPE_MOD,             (* Protracker / Fasttracker MOD. *)
    FMOD_SOUND_TYPE_MPEG,            (* MP2/MP3 MPEG. *)
    FMOD_SOUND_TYPE_OGGVORBIS,       (* Ogg vorbis. *)
    FMOD_SOUND_TYPE_PLAYLIST,        (* Information only from ASX/PLS/M3U/WAX playlists *)
    FMOD_SOUND_TYPE_RAW,             (* Raw PCM data. *)
    FMOD_SOUND_TYPE_S3M,             (* ScreamTracker 3. *)
    FMOD_SOUND_TYPE_SF2,             (* Sound font 2 format. *)
    FMOD_SOUND_TYPE_USER,            (* User created sound. *)
    FMOD_SOUND_TYPE_WAV,             (* Microsoft WAV. *)
    FMOD_SOUND_TYPE_XM,              (* FastTracker 2 XM. *)
    FMOD_SOUND_TYPE_XMA,             (* Xbox360 XMA *)
    FMOD_SOUND_TYPE_VAG,             (* PlayStation 2 / PlayStation Portable adpcm VAG format. *)
    FMOD_SOUND_TYPE_AUDIOQUEUE,      (* iPhone hardware decoder, supports AAC, ALAC and MP3. extracodecdata is a pointer to an FMOD_AUDIOQUEUE_EXTRACODECDATA structure. *)
    FMOD_SOUND_TYPE_XWMA,            (* Xbox360 XWMA *)
    FMOD_SOUND_TYPE_BCWAV,           (* 3DS BCWAV container format for DSP ADPCM and PCM *)

    FMOD_SOUND_TYPE_MAX,             (* Maximum number of sound types supported. *)
    FMOD_SOUND_TYPE_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );

(*
[ENUM]
[
    [DESCRIPTION]   
    These definitions describe the native format of the hardware or software buffer that will be used.

    [REMARKS]
    This is the format the native hardware or software buffer will be or is created in.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::createSound
    Sound::getFormat
]
*)
type
  FMOD_SOUND_FORMAT =
  (
    FMOD_SOUND_FORMAT_NONE,             (* Unitialized / unknown. *)
    FMOD_SOUND_FORMAT_PCM8,             (* 8bit integer PCM data. *)
    FMOD_SOUND_FORMAT_PCM16,            (* 16bit integer PCM data. *)
    FMOD_SOUND_FORMAT_PCM24,            (* 24bit integer PCM data. *)
    FMOD_SOUND_FORMAT_PCM32,            (* 32bit integer PCM data. *)
    FMOD_SOUND_FORMAT_PCMFLOAT,         (* 32bit floating point PCM data. *)
    FMOD_SOUND_FORMAT_GCADPCM,          (* Compressed GameCube DSP data. *)
    FMOD_SOUND_FORMAT_IMAADPCM,         (* Compressed IMA ADPCM / Xbox ADPCM data. *)
    FMOD_SOUND_FORMAT_VAG,              (* Compressed PlayStation 2 / PlayStation Portable ADPCM data. *)
    FMOD_SOUND_FORMAT_XMA,              (* Compressed Xbox360 data. *)
    FMOD_SOUND_FORMAT_MPEG,             (* Compressed MPEG layer 2 or 3 data. *)
    FMOD_SOUND_FORMAT_CELT,             (* Compressed CELT data. *)

    FMOD_SOUND_FORMAT_MAX,              (* Maximum number of sound formats supported. *)
    FMOD_SOUND_FORMAT_FORCEINT = 65536  (* Makes sure this enum is signed 32bit. *)
  );

(*
[DEFINE]
[
    [NAME]
    FMOD_MODE

    [DESCRIPTION]
    Sound description bitfields, bitwise OR them together for loading and describing sounds.

    [REMARKS]
    By default a sound will open as a static sound that is decompressed fully into memory to PCM. (ie equivalent of FMOD_CREATESAMPLE)
    To have a sound stream instead, use FMOD_CREATESTREAM, or use the wrapper function System::createStream.
    Some opening modes (ie FMOD_OPENUSER, FMOD_OPENMEMORY, FMOD_OPENMEMORY_POINT, FMOD_OPENRAW) will need extra information.
    This can be provided using the FMOD_CREATESOUNDEXINFO structure.

    On Playstation 2, non VAG formats will default to FMOD_SOFTWARE if FMOD_HARDWARE is not specified.
    This is due to PS2 hardware not supporting PCM data.

    Specifying FMOD_OPENMEMORY_POINT will POINT to your memory rather allocating its own sound buffers and duplicating it internally.
    <b><u>This means you cannot free the memory while FMOD is using it, until after Sound::release is called.</b></u>
    With FMOD_OPENMEMORY_POINT, for PCM formats, only WAV, FSB, and RAW are supported.  For compressed formats, only those formats supported by FMOD_CREATECOMPRESSEDSAMPLE are supported.
    With FMOD_OPENMEMORY_POINT and FMOD_OPENRAW or PCM, if using them together, note that you must pad the data on each side by 16 bytes.  This is so fmod can modify the ends of the data for looping/interpolation/mixing purposes.  If a wav file, you will need to insert silence, and then reset loop points to stop the playback from playing that silence.
    With FMOD_OPENMEMORY_POINT, For Wii/PSP FMOD_HARDWARE supports this flag for the GCADPCM/VAG formats.  On other platforms FMOD_SOFTWARE must be used.

    <b>Xbox 360 memory</b> On Xbox 360 Specifying FMOD_OPENMEMORY_POINT to a virtual memory address will cause FMOD_ERR_INVALID_ADDRESS
    to be returned.  Use physical memory only for this functionality.

    FMOD_LOWMEM is used on a sound if you want to minimize the memory overhead, by having FMOD not allocate memory for certain
    features that are not likely to be used in a game environment.  These are :
    1. Sound::getName functionality is removed.  256 bytes per sound is saved.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::createSound
    System::createStream
    Sound::setMode
    Sound::getMode
    Channel::setMode
    Channel::getMode
    Sound::set3DCustomRolloff
    Channel::set3DCustomRolloff
    Sound::getOpenState
]
*)
const
  FMOD_DEFAULT                   = $00000000;  (* FMOD_DEFAULT is a default sound type.  Equivalent to all the defaults listed below.  FMOD_LOOP_OFF, FMOD_2D, FMOD_HARDWARE.  (Note - only Windows with a high spec soundcard, PS2, PSP, and Wii support FMOD_HARDWARE) *)
  FMOD_LOOP_OFF                  = $00000001;  (* For non looping sounds. (DEFAULT).  Overrides FMOD_LOOP_NORMAL / FMOD_LOOP_BIDI. *)
  FMOD_LOOP_NORMAL               = $00000002;  (* For forward looping sounds. *)
  FMOD_LOOP_BIDI                 = $00000004;  (* For bidirectional looping sounds. (only works on software mixed static sounds). *)
  FMOD_2D                        = $00000008;  (* Ignores any 3d processing. (DEFAULT). *)
  FMOD_3D                        = $00000010;  (* Makes the sound positionable in 3D.  Overrides FMOD_2D. *)
  FMOD_HARDWARE                  = $00000020;  (* Attempts to make sounds use hardware acceleration. (DEFAULT).  Note on platforms that don't support FMOD_HARDWARE (only Windows with a high spec soundcard, PS2, PSP, and Wii support FMOD_HARDWARE), this will be internally treated as FMOD_SOFTWARE. *)
  FMOD_SOFTWARE                  = $00000040;  (* Makes the sound be mixed by the FMOD CPU based software mixer.  Overrides FMOD_HARDWARE.  Use this for FFT, DSP, compressed sample support, 2D multi-speaker support and other software related features. *)
  FMOD_CREATESTREAM              = $00000080;  (* Decompress at runtime, streaming from the source provided (ie from disk).  Overrides FMOD_CREATESAMPLE and FMOD_CREATECOMPRESSEDSAMPLE.  Note a stream can only be played once at a time due to a stream only having 1 stream buffer and file handle.  Open multiple streams to have them play concurrently. *)
  FMOD_CREATESAMPLE              = $00000100;  (* Decompress at loadtime, decompressing or decoding whole file into memory as the target sample format (ie PCM).  Fastest for FMOD_SOFTWARE based playback and most flexible.  *)
  FMOD_CREATECOMPRESSEDSAMPLE    = $00000200;  (* Load MP2, MP3, IMAADPCM or XMA into memory and leave it compressed.  During playback the FMOD software mixer will decode it in realtime as a 'compressed sample'.  Can only be used in combination with FMOD_SOFTWARE.  Overrides FMOD_CREATESAMPLE.  If the sound data is not ADPCM, MPEG or XMA it will behave as if it was created with FMOD_CREATESAMPLE and decode the sound into PCM. *)
  FMOD_OPENUSER                  = $00000400;  (* Opens a user created static sample or stream. Use FMOD_CREATESOUNDEXINFO to specify format and/or read callbacks.  If a user created 'sample' is created with no read callback, the sample will be empty.  Use Sound::lock and Sound::unlock to place sound data into the sound if this is the case. *)
  FMOD_OPENMEMORY                = $00000800;  (* "name_or_data" will be interpreted as a pointer to memory instead of filename for creating sounds.  Use FMOD_CREATESOUNDEXINFO to specify length.  If used with FMOD_CREATESAMPLE or FMOD_CREATECOMPRESSEDSAMPLE, FMOD duplicates the memory into its own buffers.  Your own buffer can be freed after open.  If used with FMOD_CREATESTREAM, FMOD will stream out of the buffer whose pointer you passed in.  In this case, your own buffer should not be freed until you have finished with and released the stream.*)
  FMOD_OPENMEMORY_POINT          = $10000000;  (* "name_or_data" will be interpreted as a pointer to memory instead of filename for creating sounds.  Use FMOD_CREATESOUNDEXINFO to specify length.  This differs to FMOD_OPENMEMORY in that it uses the memory as is, without duplicating the memory into its own buffers.  For Wii/PSP FMOD_HARDWARE supports this flag for the GCADPCM/VAG formats.  On other platforms FMOD_SOFTWARE must be used, as sound hardware on the other platforms (ie PC) cannot access main ram.  Cannot be freed after open, only after Sound::release.   Will not work if the data is compressed and FMOD_CREATECOMPRESSEDSAMPLE is not used. *)
  FMOD_OPENRAW                   = $00001000;  (* Will ignore file format and treat as raw pcm.  Use FMOD_CREATESOUNDEXINFO to specify format.  Requires at least defaultfrequency, numchannels and format to be specified before it will open.  Must be little endian data. *)
  FMOD_OPENONLY                  = $00002000;  (* Just open the file, dont prebuffer or read.  Good for fast opens for info, or when sound::readData is to be used. *)
  FMOD_ACCURATETIME              = $00004000;  (* For System::createSound - for accurate Sound::getLength/Channel::setPosition on VBR MP3, and MOD/S3M/XM/IT/MIDI files.  Scans file first, so takes longer to open. FMOD_OPENONLY does not affect this. *)
  FMOD_MPEGSEARCH                = $00008000;  (* For corrupted / bad MP3 files.  This will search all the way through the file until it hits a valid MPEG header.  Normally only searches for 4k. *)
  FMOD_NONBLOCKING               = $00010000;  (* For opening sounds and getting streamed subsounds (seeking) asyncronously.  Use Sound::getOpenState to poll the state of the sound as it opens or retrieves the subsound in the background. *)
  FMOD_UNIQUE                    = $00020000;  (* Unique sound, can only be played one at a time *)
  FMOD_3D_HEADRELATIVE           = $00040000;  (* Make the sound's position, velocity and orientation relative to the listener. *)
  FMOD_3D_WORLDRELATIVE          = $00080000;  (* Make the sound's position, velocity and orientation absolute (relative to the world). (DEFAULT) *)
  FMOD_3D_INVERSEROLLOFF         = $00100000;  (* This sound will follow the inverse rolloff model where mindistance = full volume, maxdistance = where sound stops attenuating, and rolloff is fixed according to the global rolloff factor.  (DEFAULT) *)
  FMOD_3D_LINEARROLLOFF          = $00200000;  (* This sound will follow a linear rolloff model where mindistance = full volume, maxdistance = silence.  Rolloffscale is ignored. *)
  FMOD_3D_LINEARSQUAREROLLOFF    = $00400000;  (* This sound will follow a linear-square rolloff model where mindistance = full volume, maxdistance = silence.  Rolloffscale is ignored. *)
  FMOD_3D_CUSTOMROLLOFF          = $04000000;  (* This sound will follow a rolloff model defined by Sound::set3DCustomRolloff / Channel::set3DCustomRolloff.  *)
  FMOD_3D_IGNOREGEOMETRY         = $40000000;  (* Is not affect by geometry occlusion.  If not specified in Sound::setMode, or Channel::setMode, the flag is cleared and it is affected by geometry again. *)
  FMOD_UNICODE                   = $01000000;  (* Filename is double-byte unicode. *)
  FMOD_IGNORETAGS                = $02000000;  (* Skips id3v2/asf/etc tag checks when opening a sound, to reduce seek/read overhead when opening files (helps with CD performance). *)
  FMOD_LOWMEM                    = $08000000;  (* Removes some features from samples to give a lower memory overhead, like Sound::getName.  See remarks. *)
  FMOD_LOADSECONDARYRAM          = $20000000;  (* Load sound into the secondary RAM of supported platform. On PS3, sounds will be loaded into RSX/VRAM. *)
  FMOD_VIRTUAL_PLAYFROMSTART     = $80000000;  (* For sounds that start virtual (due to being quiet or low importance), instead of swapping back to audible, and playing at the correct offset according to time, this flag makes the sound play from the start. *)

(*
[ENUM]
[
    [DESCRIPTION]
    These values describe what state a sound is in after FMOD_NONBLOCKING has been used to open it.

    [REMARKS]
    With streams, if you are using FMOD_NONBLOCKING, note that if the user calls Sound::getSubSound, a stream will go into FMOD_OPENSTATE_SEEKING state and sound related commands will return FMOD_ERR_NOTREADY.
    With streams, if you are using FMOD_NONBLOCKING, note that if the user calls Channel::getPosition, a stream will go into FMOD_OPENSTATE_SETPOSITION state and sound related commands will return FMOD_ERR_NOTREADY.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    Sound::getOpenState
    FMOD_MODE
]
*)
type
  FMOD_OPENSTATE =
  (
    FMOD_OPENSTATE_READY = 0,       (* Opened and ready to play. *)
    FMOD_OPENSTATE_LOADING,         (* Initial load in progress. *)
    FMOD_OPENSTATE_ERROR,           (* Failed to open - file not found, out of memory etc.  See return value of Sound::getOpenState for what happened. *)
    FMOD_OPENSTATE_CONNECTING,      (* Connecting to remote host (internet sounds only). *)
    FMOD_OPENSTATE_BUFFERING,       (* Buffering data. *)
    FMOD_OPENSTATE_SEEKING,         (* Seeking to subsound and re-flushing stream buffer. *)
    FMOD_OPENSTATE_PLAYING,         (* Ready and playing, but not possible to release at this time without stalling the main thread. *)
    FMOD_OPENSTATE_SETPOSITION,     (* Seeking within a stream to a different position. *)

    FMOD_OPENSTATE_MAX,             (* Maximum number of open state types. *)
    FMOD_OPENSTATE_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );
(*
[ENUM]
[
    [DESCRIPTION]   
    These flags are used with SoundGroup::setMaxAudibleBehavior to determine what happens when more sounds 
    are played than are specified with SoundGroup::setMaxAudible.

    [REMARKS]
    When using FMOD_SOUNDGROUP_BEHAVIOR_MUTE, SoundGroup::setMuteFadeSpeed can be used to stop a sudden transition.  
    Instead, the time specified will be used to cross fade between the sounds that go silent and the ones that become audible.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    SoundGroup::setMaxAudibleBehavior
    SoundGroup::getMaxAudibleBehavior
    SoundGroup::setMaxAudible
    SoundGroup::getMaxAudible
    SoundGroup::setMuteFadeSpeed
    SoundGroup::getMuteFadeSpeed
]
*)
type
  FMOD_SOUNDGROUP_BEHAVIOR =
  (
    FMOD_SOUNDGROUP_BEHAVIOR_FAIL,            (* Any sound played that puts the sound count over the SoundGroup::setMaxAudible setting, will simply fail during System::playSound. *)
    FMOD_SOUNDGROUP_BEHAVIOR_MUTE,            (* Any sound played that puts the sound count over the SoundGroup::setMaxAudible setting, will be silent, then if another sound in the group stops the sound that was silent before becomes audible again. *)
    FMOD_SOUNDGROUP_BEHAVIOR_STEALLOWEST,     (* Any sound played that puts the sound count over the SoundGroup::setMaxAudible setting, will steal the quietest / least important sound playing in the group. *)

    FMOD_SOUNDGROUP_BEHAVIOR_MAX,             (* Maximum number of open state types. *)
    FMOD_SOUNDGROUP_BEHAVIOR_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );

(*
[ENUM]
[
    [DESCRIPTION]   
    These callback types are used with Channel::setCallback.

    [REMARKS]
    Each callback has commanddata parameters passed as int unique to the type of callback.
    See reference to FMOD_CHANNEL_CALLBACK to determine what they might mean for each type of callback.
    
    <b>Note!</b>  Currently the user must call System::update for these callbacks to trigger!

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Channel::setCallback
    FMOD_CHANNEL_CALLBACK
    System::update
]
*)
type
  FMOD_CHANNEL_CALLBACKTYPE =
  (
    FMOD_CHANNEL_CALLBACKTYPE_END,             (* Called when a sound ends. *)
    FMOD_CHANNEL_CALLBACKTYPE_VIRTUALVOICE,    (* Called when a voice is swapped out or swapped in. *)
    FMOD_CHANNEL_CALLBACKTYPE_SYNCPOINT,       (* Called when a syncpoint is encountered.  Can be from wav file markers. *)
    FMOD_CHANNEL_CALLBACKTYPE_OCCLUSION,       (* Called when the channel has its geometry occlusion value calculated.  Can be used to clamp or change the value. *)

    FMOD_CHANNEL_CALLBACKTYPE_MAX,             (* Maximum number of callback types supported. *)
    FMOD_CHANNEL_CALLBACKTYPE_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );

(*
[ENUM]
[
    [DESCRIPTION]   
    These callback types are used with System::setCallback.

    [REMARKS]
    Each callback has commanddata parameters passed as void* unique to the type of callback.
    See reference to FMOD_SYSTEM_CALLBACK to determine what they might mean for each type of callback.
    
    <b>Note!</b> Using FMOD_SYSTEM_CALLBACKTYPE_DEVICELISTCHANGED (on Mac only) requires the application to be running an event loop which will allow external changes to device list to be detected by FMOD.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    System::setCallback
    FMOD_SYSTEM_CALLBACK
    System::update
    DSP::addInput
]
*)
type
  FMOD_SYSTEM_CALLBACKTYPE =
  (
    FMOD_SYSTEM_CALLBACKTYPE_DEVICELISTCHANGED,      (* Called from System::update when the enumerated list of devices has changed. *)
    FMOD_SYSTEM_CALLBACKTYPE_DEVICELOST,             (* Called from System::update when an output device has been lost due to control panel parameter changes and FMOD cannot automatically recover. *)
    FMOD_SYSTEM_CALLBACKTYPE_MEMORYALLOCATIONFAILED, (* Called directly when a memory allocation fails somewhere in FMOD. *)
    FMOD_SYSTEM_CALLBACKTYPE_THREADCREATED,          (* Called directly when a thread is created. *)
    FMOD_SYSTEM_CALLBACKTYPE_BADDSPCONNECTION,       (* Called when a bad connection was made with DSP::addInput. Usually called from mixer thread because that is where the connections are made.  *)
    FMOD_SYSTEM_CALLBACKTYPE_BADDSPLEVEL,            (* Called when too many effects were added exceeding the maximum tree depth of 128.  This is most likely caused by accidentally adding too many DSP effects. Usually called from mixer thread because that is where the connections are made.  *)

    FMOD_SYSTEM_CALLBACKTYPE_MAX,                    (* Maximum number of callback types supported. *)
    FMOD_SYSTEM_CALLBACKTYPE_FORCEINT = 65536        (* Makes sure this enum is signed 32bit. *)
  );

(* 
    FMOD Callbacks
*)
FMOD_SYSTEM_CALLBACK          = function (system: FMOD_SYSTEM; callbacktype: FMOD_SYSTEM_CALLBACKTYPE; commanddata1: Pointer; commanddata2: Pointer):FMOD_RESULT; stdcall;
FMOD_CHANNEL_CALLBACK         = function (channel: FMOD_CHANNEL; callbacktype: FMOD_CHANNEL_CALLBACKTYPE; commanddata1: Pointer; commanddata2: Pointer):FMOD_RESULT; stdcall;

FMOD_SOUND_NONBLOCKCALLBACK   = function (sound:FMOD_SOUND; result: FMOD_RESULT):FMOD_RESULT; stdcall;
FMOD_SOUND_PCMREADCALLBACK    = function (sound:FMOD_SOUND; data: Pointer; datalen: Cardinal):FMOD_RESULT; stdcall;
FMOD_SOUND_PCMSETPOSCALLBACK  = function (sound:FMOD_SOUND; subsound: Integer; position: Cardinal; postype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;

FMOD_FILE_OPENCALLBACK        = function (const name: PChar; unicode: Integer; var filesize: Cardinal; handle: Pointer; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_FILE_CLOSECALLBACK       = function (handle: Pointer; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_FILE_READCALLBACK        = function (handle: Pointer; buffer: Pointer; sizebytes: Cardinal; var bytesread: Cardinal; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_FILE_SEEKCALLBACK        = function (handle: Pointer; pos: Cardinal; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_FILE_ASYNCREADCALLBACK   = function (info: FMOD_ASYNCREADINFO; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_FILE_ASYNCCANCELCALLBACK = function (handle: Pointer; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_MEMORY_ALLOCCALLBACK     = function (size: Cardinal; memorytype: FMOD_MEMORY_TYPE; const sourcestr: PChar):Pointer; stdcall;
FMOD_MEMORY_REALLOCCALLBACK   = function (ptr: Pointer; size: Cardinal; memorytype: FMOD_MEMORY_TYPE; const sourcestr: PChar):Pointer; stdcall;
FMOD_MEMORY_FREECALLBACK      = procedure (ptr: Pointer; memorytype: FMOD_MEMORY_TYPE; const sourcestr: PChar);

FMOD_3D_ROLLOFFCALLBACK       = function (channel: FMOD_CHANNEL; distance: Single):Single; stdcall;


(*
[ENUM]
[
    [DESCRIPTION]   
    List of windowing methods used in spectrum analysis to reduce leakage / transient signals intefering with the analysis.
    This is a problem with analysis of continuous signals that only have a small portion of the signal sample (the fft window size).
    Windowing the signal with a curve or triangle tapers the sides of the fft window to help alleviate this problem.

    [REMARKS]
    Cyclic signals such as a sine wave that repeat their cycle in a multiple of the window size do not need windowing.
    I.e. If the sine wave repeats every 1024, 512, 256 etc samples and the FMOD fft window is 1024, then the signal would not need windowing.
    Not windowing is the same as FMOD_DSP_FFT_WINDOW_RECT, which is the default.
    If the cycle of the signal (ie the sine wave) is not a multiple of the window size, it will cause frequency abnormalities, so a different windowing method is needed.
    
    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    System::getSpectrum
    Channel::getSpectrum
]
*)
type
  FMOD_DSP_FFT_WINDOW =
  (
    FMOD_DSP_FFT_WINDOW_RECT,            (* w[n] = 1.0                                                                                            *)
    FMOD_DSP_FFT_WINDOW_TRIANGLE,        (* w[n] = TRI(2n/N)                                                                                      *)
    FMOD_DSP_FFT_WINDOW_HAMMING,         (* w[n] = 0.54 - (0.46 * COS(n/N) )                                                                      *)
    FMOD_DSP_FFT_WINDOW_HANNING,         (* w[n] = 0.5 *  (1.0  - COS(n/N) )                                                                      *)
    FMOD_DSP_FFT_WINDOW_BLACKMAN,        (* w[n] = 0.42 - (0.5  * COS(n/N) ) + (0.08 * COS(2.0 * n/N) )                                           *)
    FMOD_DSP_FFT_WINDOW_BLACKMANHARRIS,  (* w[n] = 0.35875 - (0.48829 * COS(1.0 * n/N)) + (0.14128 * COS(2.0 * n/N)) - (0.01168 * COS(3.0 * n/N)) *)
    
    FMOD_DSP_FFT_WINDOW_MAX,             (* Maximum number of FFT window types supported. *)
    FMOD_DSP_FFT_WINDOW_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );


(*
[ENUM]
[
    [DESCRIPTION]   
    List of interpolation types that the FMOD Ex software mixer supports.  

    [REMARKS]
    The default resampler type is FMOD_DSP_RESAMPLER_LINEAR.
    Use System::setSoftwareFormat to tell FMOD the resampling quality you require for FMOD_SOFTWARE based sounds.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    System::setSoftwareFormat
    System::getSoftwareFormat
]
*)
type
  FMOD_DSP_RESAMPLER =
  (
    FMOD_DSP_RESAMPLER_NOINTERP,        (* No interpolation.  High frequency aliasing hiss will be audible depending on the sample rate of the sound. *)
    FMOD_DSP_RESAMPLER_LINEAR,          (* Linear interpolation (default method).  Fast and good quality, causes very slight lowpass effect on low frequency sounds. *)
    FMOD_DSP_RESAMPLER_CUBIC,           (* Cubic interpolation.  Slower than linear interpolation but better quality. *)
    FMOD_DSP_RESAMPLER_SPLINE,          (* 5 point spline interpolation.  Slowest resampling method but best quality. *)

    FMOD_DSP_RESAMPLER_MAX,             (* Maximum number of resample methods supported. *)
    FMOD_DSP_RESAMPLER_FORCEINT = 65536 (* Makes sure this enum is signed 32bit. *)
  );


(*
[ENUM]
[
    [DESCRIPTION]   
    List of tag types that could be stored within a sound.  These include id3 tags, metadata from netstreams and vorbis/asf data.

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Sound::getTag
]
*)
type
  FMOD_TAGTYPE =
  (
    FMOD_TAGTYPE_UNKNOWN = 0,
    FMOD_TAGTYPE_ID3V1,
    FMOD_TAGTYPE_ID3V2,
    FMOD_TAGTYPE_VORBISCOMMENT,
    FMOD_TAGTYPE_SHOUTCAST,
    FMOD_TAGTYPE_ICECAST,
    FMOD_TAGTYPE_ASF,
    FMOD_TAGTYPE_MIDI,
    FMOD_TAGTYPE_PLAYLIST,
    FMOD_TAGTYPE_FMOD,
    FMOD_TAGTYPE_USER,

    FMOD_TAGTYPE_MAX,               (* Maximum number of tag types supported. *)
    FMOD_TAGTYPE_FORCEINT = 65536   (* Makes sure this enum is signed 32bit. *)
  );


(*
[ENUM]
[
    [DESCRIPTION]   
    List of data types that can be returned by Sound::getTag

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Sound::getTag
]
*)
type
  FMOD_TAGDATATYPE =
  (
    FMOD_TAGDATATYPE_BINARY = 0,
    FMOD_TAGDATATYPE_INT,
    FMOD_TAGDATATYPE_FLOAT,
    FMOD_TAGDATATYPE_STRING,
    FMOD_TAGDATATYPE_STRING_UTF16,
    FMOD_TAGDATATYPE_STRING_UTF16BE,
    FMOD_TAGDATATYPE_STRING_UTF8,
    FMOD_TAGDATATYPE_CDTOC,

    FMOD_TAGDATATYPE_MAX,               (* Maximum number of tag datatypes supported. *)
    FMOD_TAGDATATYPE_FORCEINT = 65536   (* Makes sure this enum is signed 32bit. *)
  );


(*
[ENUM]
[
    [DESCRIPTION]   
    Types of delay that can be used with Channel::setDelay / Channel::getDelay.

    [REMARKS]
    If you haven't called Channel::setDelay yet, if you call Channel::getDelay with FMOD_DELAYTYPE_DSPCLOCK_START it will return the 
    equivalent global DSP clock value to determine when a channel started, so that you can use it for other channels to sync against.
    
    Use System::getDSPClock to also get the current dspclock time, a base for future calls to Channel::setDelay.
    
    Use FMOD_64BIT_ADD or FMOD_64BIT_SUB to add a hi/lo combination together and cope with wraparound.
    
    If FMOD_DELAYTYPE_END_MS is specified, the value is not treated as a 64 bit number, just the delayhi value is used and it is treated as milliseconds.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Channel::setDelay
    Channel::getDelay
    System::getDSPClock
]
*)
type
  FMOD_DELAYTYPE =
  (
    FMOD_DELAYTYPE_END_MS,              (* Delay at the end of the sound in milliseconds.  Use delayhi only.   Channel::isPlaying will remain true until this delay has passed even though the sound itself has stopped playing.*)
    FMOD_DELAYTYPE_DSPCLOCK_START,      (* Time the sound started if Channel::getDelay is used, or if Channel::setDelay is used, the sound will delay playing until this exact tick. *)
    FMOD_DELAYTYPE_DSPCLOCK_END,        (* Time the sound should end. If this is non-zero, the channel will go silent at this exact tick. *)
    FMOD_DELAYTYPE_DSPCLOCK_PAUSE,      (* Time the sound should pause. If this is non-zero, the channel will pause at this exact tick. *)

    FMOD_DELAYTYPE_MAX,                 (* Maximum number of tag datatypes supported. *)
    FMOD_DELAYTYPE_FORCEINT = 65536     (* Makes sure this enum is signed 32bit. *)
  );


(*
TODO
#define FMOD_64BIT_ADD(_hi1, _lo1, _hi2, _lo2) _hi1 += ((_hi2) + ((((_lo1) + (_lo2)) < (_lo1)) ? 1 : 0)); (_lo1) += (_lo2);
#define FMOD_64BIT_SUB(_hi1, _lo1, _hi2, _lo2) _hi1 -= ((_hi2) + ((((_lo1) - (_lo2)) > (_lo1)) ? 1 : 0)); (_lo1) -= (_lo2);
*)

(*
[STRUCTURE] 
[
    [DESCRIPTION]   
    Structure describing a piece of tag data.

    [REMARKS]
    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Sound::getTag
    FMOD_TAGTYPE
    FMOD_TAGDATATYPE
]
*)
type
  FMOD_TAG = record
    tagtype  : FMOD_TAGTYPE;     (* [r] The type of this tag. *)
    datatype : FMOD_TAGDATATYPE; (* [r] The type of data that this tag contains *)
    name     : PChar;            (* [r] The name of this tag i.e. "TITLE", "ARTIST" etc. *)
    data     : Pointer;          (* [r] Pointer to the tag data - its format is determined by the datatype member *)
    datalen  : Cardinal;         (* [r] Length of the data contained in this tag *)
    updated  : FMOD_BOOL;        (* [r] True if this tag has been updated since last being accessed with Sound::getTag *)
  end;


(*
[STRUCTURE] 
[
    [DESCRIPTION]   
    Structure describing a CD/DVD table of contents

    [REMARKS]
    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]      
    Sound::getTag
]
*)
type
  FMOD_CDTOC = record
    numtracks : Integer;             (* [r] The number of tracks on the CD *)
    min : array[0..99] of Integer;   (* [r] The start offset of each track in minutes *)
    sec : array[0..99] of Integer;   (* [r] The start offset of each track in seconds *)
    frame : array[0..99] of Integer; (* [r] The start offset of each track in frames *)
  end;


(*
[DEFINE]
[
    [NAME]
    FMOD_TIMEUNIT

    [DESCRIPTION]   
    List of time types that can be returned by Sound::getLength and used with Channel::setPosition or Channel::getPosition.

    [REMARKS]
    FMOD_TIMEUNIT_SENTENCE_MS, FMOD_TIMEUNIT_SENTENCE_PCM, FMOD_TIMEUNIT_SENTENCE_PCMBYTES, FMOD_TIMEUNIT_SENTENCE and FMOD_TIMEUNIT_SENTENCE_SUBSOUND are only supported by Channel functions.
    Do not combine flags except FMOD_TIMEUNIT_BUFFERED.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    Sound::getLength
    Channel::setPosition
    Channel::getPosition
]
*)
const
  FMOD_TIMEUNIT_MS                = $00000001;  (* Milliseconds. *)
  FMOD_TIMEUNIT_PCM               = $00000002;  (* PCM samples, related to milliseconds * samplerate / 1000. *)
  FMOD_TIMEUNIT_PCMBYTES          = $00000004;  (* Bytes, related to PCM samples * channels * datawidth (ie 16bit = 2 bytes). *)
  FMOD_TIMEUNIT_RAWBYTES          = $00000008;  (* Raw file bytes of (compressed) sound data (does not include headers).  Only used by Sound::getLength and Channel::getPosition. *)
  FMOD_TIMEUNIT_PCMFRACTION       = $00000010;  (* Fractions of 1 PCM sample.  Unsigned int range 0 to 0xFFFFFFFF.  Used for sub-sample granularity for DSP purposes. *)
  FMOD_TIMEUNIT_MODORDER          = $00000100;  (* MOD/S3M/XM/IT.  Order in a sequenced module format.  Use Sound::getFormat to determine the PCM format being decoded to. *)
  FMOD_TIMEUNIT_MODROW            = $00000200;  (* MOD/S3M/XM/IT.  Current row in a sequenced module format.  Sound::getLength will return the number of rows in the currently playing or seeked to pattern. *)
  FMOD_TIMEUNIT_MODPATTERN        = $00000400;  (* MOD/S3M/XM/IT.  Current pattern in a sequenced module format.  Sound::getLength will return the number of patterns in the song and Channel::getPosition will return the currently playing pattern. *)
  FMOD_TIMEUNIT_SENTENCE_MS       = $00010000;  (* Currently playing subsound in a sentence time in milliseconds. *)
  FMOD_TIMEUNIT_SENTENCE_PCM      = $00020000;  (* Currently playing subsound in a sentence time in PCM Samples, related to milliseconds * samplerate / 1000. *)
  FMOD_TIMEUNIT_SENTENCE_PCMBYTES = $00040000;  (* Currently playing subsound in a sentence time in bytes, related to PCM samples * channels * datawidth (ie 16bit = 2 bytes). *)
  FMOD_TIMEUNIT_SENTENCE          = $00080000;  (* Currently playing sentence index according to the channel. *)
  FMOD_TIMEUNIT_SENTENCE_SUBSOUND = $00100000;  (* Currently playing subsound index in a sentence. *)
  FMOD_TIMEUNIT_BUFFERED          = $10000000;  (* Time value as seen by buffered stream.  This is always ahead of audible time, and is only used for processing. *)
(* [DEFINE_END] *)


(*
[ENUM]
[
    [DESCRIPTION]
    When creating a multichannel sound, FMOD will pan them to their default speaker locations, for example a 6 channel sound will default to one channel per 5.1 output speaker.
    Another example is a stereo sound.  It will default to left = front left, right = front right.
    
    This is for sounds that are not 'default'.  For example you might have a sound that is 6 channels but actually made up of 3 stereo pairs, that should all be located in front left, front right only.

    [REMARKS]
    For full flexibility of speaker assignments, use Channel::setSpeakerLevels.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    FMOD_CREATESOUNDEXINFO
    Channel::setSpeakerLevels
]
*)
type
  FMOD_SPEAKERMAPTYPE =
  (
    FMOD_SPEAKERMAPTYPE_DEFAULT,     (* This is the default, and just means FMOD decides which speakers it puts the source channels. *)
    FMOD_SPEAKERMAPTYPE_ALLMONO,     (* This means the sound is made up of all mono sounds.  All voices will be panned to the front center by default in this case.  *)
    FMOD_SPEAKERMAPTYPE_ALLSTEREO,   (* This means the sound is made up of all stereo sounds.  All voices will be panned to front left and front right alternating every second channel.  *)
    FMOD_SPEAKERMAPTYPE_51_PROTOOLS  (* Map a 5.1 sound to use protools L C R Ls Rs LFE mapping.  Will return an error if not a 6 channel sound. *)
  );


(*
[STRUCTURE] 
[
    [DESCRIPTION]
    Use this structure with System::createSound when more control is needed over loading.
    The possible reasons to use this with System::createSound are:
    - Loading a file from memory.
    - Loading a file from within another larger (possibly wad/pak) file, by giving the loader an offset and length.
    - To create a user created / non file based sound.
    - To specify a starting subsound to seek to within a multi-sample sounds (ie FSB/DLS/SF2) when created as a stream.
    - To specify which subsounds to load for multi-sample sounds (ie FSB/DLS/SF2) so that memory is saved and only a subset is actually loaded/read from disk.
    - To specify 'piggyback' read and seek callbacks for capture of sound data as fmod reads and decodes it.  Useful for ripping decoded PCM data from sounds as they are loaded / played.
    - To specify a MIDI DLS/SF2 sample set file to load when opening a MIDI file.
    See below on what members to fill for each of the above types of sound you want to create.

    [REMARKS]
    This structure is optional!  Specify 0 or NULL in System::createSound if you don't need it!
    
    <u>Loading a file from memory.</u>
    - Create the sound using the FMOD_OPENMEMORY flag.
    - Mandatory.  Specify 'length' for the size of the memory block in bytes.
    - Other flags are optional.
    
    
    <u>Loading a file from within another larger (possibly wad/pak) file, by giving the loader an offset and length.</u>
    - Mandatory.  Specify 'fileoffset' and 'length'.
    - Other flags are optional.
    
    
    <u>To create a user created / non file based sound.</u>
    - Create the sound using the FMOD_OPENUSER flag.
    - Mandatory.  Specify 'defaultfrequency, 'numchannels' and 'format'.
    - Other flags are optional.
    
    
    <u>To specify a starting subsound to seek to and flush with, within a multi-sample stream (ie FSB/DLS/SF2).</u>
    
    - Mandatory.  Specify 'initialsubsound'.


    <u>To specify which subsounds to load for multi-sample sounds (ie FSB/DLS/SF2) so that memory is saved and only a subset is actually loaded/read from disk.</u>

    - Mandatory.  Specify 'inclusionlist' and 'inclusionlistnum'.

    
    <u>To specify 'piggyback' read and seek callbacks for capture of sound data as fmod reads and decodes it.  Useful for ripping decoded PCM data from sounds as they are loaded / played.</u>
    
    - Mandatory.  Specify 'pcmreadcallback' and 'pcmseekcallback'.
    
    
    <u>To specify a MIDI DLS/SF2 sample set file to load when opening a MIDI file.</u>
    
    - Mandatory.  Specify 'dlsname'.
    
    
    Setting the 'decodebuffersize' is for cpu intensive codecs that may be causing stuttering, not file intensive codecs (ie those from CD or netstreams) which are normally 
    altered with System::setStreamBufferSize.  As an example of cpu intensive codecs, an mp3 file will take more cpu to decode than a PCM wav file.
    If you have a stuttering effect, then it is using more cpu than the decode buffer playback rate can keep up with.  Increasing the decode buffersize will most likely solve this problem.
    
    
    FSB codec.  If inclusionlist and numsubsounds are used together, this will trigger a special mode where subsounds are shuffled down to save memory.  (useful for large FSB 
    files where you only want to load 1 sound).  There will be no gaps, ie no null subsounds.  As an example, if there are 10,000 subsounds and there is an inclusionlist with only 1 entry, 
    and numsubsounds = 1, then subsound 0 will be that entry, and there will only be the memory allocated for 1 subsound.  Previously there would still be 10,000 subsound pointers and other
    associated codec entries allocated along with it multiplied by 10,000.
    
    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::createSound
    System::setStreamBufferSize
    FMOD_MODE
    FMOD_SOUND_FORMAT
    FMOD_SOUND_TYPE
    FMOD_SPEAKERMAPTYPE
]
*)
type
  PFMOD_CREATESOUNDEXINFO = ^FMOD_CREATESOUNDEXINFO;
  FMOD_CREATESOUNDEXINFO = record
    cbsize: Integer;             (* [w] Size of this structure.  This is used so the structure can be expanded in the future and still work on older versions of FMOD Ex. *)
    length : Cardinal;             (* [w] Optional. Specify 0 to ignore. Size in bytes of file to load, or sound to create (in this case only if FMOD_OPENUSER is used).  Required if loading from memory.  If 0 is specified, then it will use the size of the file (unless loading from memory then an error will be returned). *)
    fileoffset : Cardinal;         (* [w] Optional. Specify 0 to ignore. Offset from start of the file to start loading from.  This is useful for loading files from inside big data files. *)
    numchannels: Integer;        (* [w] Optional. Specify 0 to ignore. Number of channels in a sound mandatory if FMOD_OPENUSER or FMOD_OPENRAW is used. *)
    defaultfrequency: Integer;   (* [w] Optional. Specify 0 to ignore. Default frequency of sound in a sound mandatory if FMOD_OPENUSER or FMOD_OPENRAW is used.  Other formats use the frequency determined by the file format. *)
    format: FMOD_SOUND_FORMAT;             (* [w] Optional. Specify 0 or FMOD_SOUND_FORMAT_NONE to ignore. Format of the sound mandatory if FMOD_OPENUSER or FMOD_OPENRAW is used.  Other formats use the format determined by the file format.   *)
    decodebuffersize : Cardinal;   (* [w] Optional. Specify 0 to ignore. For streams.  This determines the size of the double buffer (in PCM samples) that a stream uses.  Use this for user created streams if you want to determine the size of the callback buffer passed to you.  Specify 0 to use FMOD's default size which is currently equivalent to 400ms of the sound format created/loaded. *)
    initialsubsound: Integer;    (* [w] Optional. Specify 0 to ignore. In a multi-sample file format such as .FSB/.DLS/.SF2, specify the initial subsound to seek to, only if FMOD_CREATESTREAM is used. *)
    numsubsounds: Integer;       (* [w] Optional. Specify 0 to ignore or have no subsounds.  In a sound created with FMOD_OPENUSER, specify the number of subsounds that are accessable with Sound::getSubSound.  If not created with FMOD_OPENUSER, this will limit the number of subsounds loaded within a multi-subsound file.  If using FSB, then if FMOD_CREATESOUNDEXINFO::inclusionlist is used, this will shuffle subsounds down so that there are not any gaps.  It will mean that the indices of the sounds will be different. *)
    inclusionlist: PInteger;      (* [w] Optional. Specify 0 to ignore. In a multi-sample format such as .FSB/.DLS/.SF2 it may be desirable to specify only a subset of sounds to be loaded out of the whole file.  This is an array of subsound indices to load into memory when created. *)
    inclusionlistnum: Integer;   (* [w] Optional. Specify 0 to ignore. This is the number of integers contained within the inclusionlist array. *)
    pcmreadcallback: FMOD_SOUND_PCMREADCALLBACK;    (* [w] Optional. Specify 0 to ignore. Callback to 'piggyback' on FMOD's read functions and accept or even write PCM data while FMOD is opening the sound.  Used for user sounds created with FMOD_OPENUSER or for capturing decoded data as FMOD reads it. *)
    pcmsetposcallback: FMOD_SOUND_PCMSETPOSCALLBACK;  (* [w] Optional. Specify 0 to ignore. Callback for when the user calls a seeking function such as Channel::setTime or Channel::setPosition within a multi-sample sound, and for when it is opened.*)
    nonblockcallback: FMOD_SOUND_NONBLOCKCALLBACK;   (* [w] Optional. Specify 0 to ignore. Callback for successful completion, or error while loading a sound that used the FMOD_NONBLOCKING flag.*)
    dlsname:PChar;            (* [w] Optional. Specify 0 to ignore. Filename for a DLS or SF2 sample set when loading a MIDI file. If not specified, on Windows it will attempt to open /windows/system32/drivers/gm.dls or /windows/system32/drivers/etc/gm.dls, on Mac it will attempt to load /System/Library/Components/CoreAudio.component/Contents/Resources/gs_instruments.dls, otherwise the MIDI will fail to open. Current DLS support is for level 1 of the specification. *)
    encryptionkey: PChar;      (* [w] Optional. Specify 0 to ignore. Key for encrypted FSB file.  Without this key an encrypted FSB file will not load. *)
    maxpolyphony: Integer;       (* [w] Optional. Specify 0 to ignore. For sequenced formats with dynamic channel allocation such as .MID and .IT, this specifies the maximum voice count allowed while playing.  .IT defaults to 64.  .MID defaults to 32. *)
    userdata: Pointer;           (* [w] Optional. Specify 0 to ignore. This is user data to be attached to the sound during creation.  Access via Sound::getUserData.  Note: This is not passed to FMOD_FILE_OPENCALLBACK, that is a different userdata that is file specific. *)
    suggestedsoundtype: FMOD_SOUND_TYPE; (* [w] Optional. Specify 0 or FMOD_SOUND_TYPE_UNKNOWN to ignore.  Instead of scanning all codec types, use this to speed up loading by making it jump straight to this codec. *)
    useropen:          FMOD_FILE_OPENCALLBACK;        (* [w] Optional. Specify 0 to ignore. Callback for opening this file. *)
    userclose:         FMOD_FILE_CLOSECALLBACK;       (* [w] Optional. Specify 0 to ignore. Callback for closing this file. *)
    userread:          FMOD_FILE_READCALLBACK;        (* [w] Optional. Specify 0 to ignore. Callback for reading from this file. *)
    userseek:          FMOD_FILE_SEEKCALLBACK;        (* [w] Optional. Specify 0 to ignore. Callback for seeking within this file. *)
    userasyncread:     FMOD_FILE_ASYNCREADCALLBACK;   (* [w] Optional. Specify 0 to ignore. Callback for seeking within this file. *)
    userasynccancel:   FMOD_FILE_ASYNCCANCELCALLBACK; (* [w] Optional. Specify 0 to ignore. Callback for seeking within this file. *)
    speakermap:        FMOD_SPEAKERMAPTYPE;           (* [w] Optional. Specify 0 to ignore. Use this to differ the way fmod maps multichannel sounds to speakers.  See FMOD_SPEAKERMAPTYPE for more. *)
    initialsoundgroup: FMOD_SOUNDGROUP;  (* [w] Optional. Specify 0 to ignore. Specify a sound group if required, to put sound in as it is created. *)
    initialseekposition: Cardinal;(* [w] Optional. Specify 0 to ignore. For streams. Specify an initial position to seek the stream to. *)
    initialseekpostype: FMOD_TIMEUNIT; (* [w] Optional. Specify 0 to ignore. For streams. Specify the time unit for the position set in initialseekposition. *)
    ignoresetfilesystem: Integer;(* [w] Optional. Specify 0 to ignore. Set to 1 to use fmod's built in file system. Ignores setFileSystem callbacks and also FMOD_CREATESOUNEXINFO file callbacks.  Useful for specific cases where you don't want to use your own file system but want to use fmod's file system (ie net streaming). *)
    cddaforceaspi: Integer;      (* [w] Optional. Specify 0 to ignore. For CDDA sounds only - if non-zero use ASPI instead of NTSCSI to access the specified CD/DVD device. *)
    audioqueuepolicy: Cardinal;   (* [w] Optional. Specify 0 or FMOD_AUDIOQUEUE_CODECPOLICY_DEFAULT to ignore. Policy used to determine whether hardware or software is used for decoding, see FMOD_AUDIOQUEUE_CODECPOLICY for options (iOS >= 3.0 required, otherwise only hardware is available) *) 
    minmidigranularity: Cardinal; (* [w] Optional. Specify 0 to ignore. Allows you to set a minimum desired MIDI mixer granularity. Values smaller than 512 give greater than default accuracy at the cost of more CPU and vise versa. Specify 0 for default (512 samples). *)
  end;


(*
[STRUCTURE] 
[
    [DESCRIPTION]
    Structure defining a reverb environment.
    
    For more indepth descriptions of the reverb properties under win32, please see the EAX2 and EAX3
    documentation at http://developer.creative.com/ under the 'downloads' section.
    If they do not have the EAX3 documentation, then most information can be attained from
    the EAX2 documentation, as EAX3 only adds some more parameters and functionality on top of 
    EAX2.

    [REMARKS]
    Note the default reverb properties are the same as the FMOD_PRESET_GENERIC preset.
    Note that integer values that typically range from -10,000 to 1000 are represented in 
    decibels, and are of a logarithmic scale, not linear, wheras float values are always linear.
    
    The numerical values listed below are the maximum, minimum and default values for each variable respectively.
    
    <b>SUPPORTED</b> next to each parameter means the platform the parameter can be set on.  Some platforms support all parameters and some don't.
    EAX   means hardware reverb on FMOD_OUTPUTTYPE_DSOUND on windows only (must use FMOD_HARDWARE), on soundcards that support EAX 1 to 4.
    EAX4  means hardware reverb on FMOD_OUTPUTTYPE_DSOUND on windows only (must use FMOD_HARDWARE), on soundcards that support EAX 4.
    I3DL2 means hardware reverb on FMOD_OUTPUTTYPE_DSOUND on windows only (must use FMOD_HARDWARE), on soundcards that support I3DL2 non EAX native reverb.
    WII   means Nintendo Wii hardware reverb (must use FMOD_HARDWARE).
    PS2   means Playstation 2 hardware reverb (must use FMOD_HARDWARE).
    SFX   means FMOD SFX software reverb.  This works on any platform that uses FMOD_SOFTWARE for loading sounds.
    
    Nintendo Wii Notes:
    This structure supports only limited parameters, and maps them to the Wii hardware reverb as follows.
    DecayTime = 'time'
    ReverbDelay = 'predelay'
    ModulationDepth = 'damping'
    Reflections = 'coloration'
    EnvDiffusion = 'crosstalk'
    Room = 'mix'
    
    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.
    Members marked with [r/w] are either read or write depending on if you are using System::setReverbProperties (w) or System::getReverbProperties (r).

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::setReverbProperties
    System::getReverbProperties
    FMOD_REVERB_PRESETS
    FMOD_REVERB_FLAGS
]
*)
type
  FMOD_REVERB_PROPERTIES = Record
                                   (*       MIN     MAX     DEFAULT  DESCRIPTION *)
    Instance          : Integer;   (* [w]   0     , 3     , 0      , Environment Instance. Simultaneous HW reverbs are possible on some platforms. (SUPPORTED:EAX4/SFX(4 instances) and Wii (3 instances)) *)
    Environment       : Integer;   (* [r/w] -1    , 25    , -1     , sets all listener properties.  -1 = OFF.                                      (SUPPORTED:EAX/PS2) *)
    EnvSize           : Single;    (* [r/w] 1.0   , 100.0 , 7.5    , environment size in meters                                                    (SUPPORTED:EAX) *)
    EnvDiffusion      : Single;    (* [r/w] 0.0   , 1.0   , 1.0    , environment diffusion                                                         (SUPPORTED:EAX/WII) *)
    Room              : Integer;   (* [r/w] -10000, 0     , -1000  , room effect level (at mid frequencies)                                        (SUPPORTED:EAX/I3DL2/SFX/WII) *)
    RoomHF            : Integer;   (* [r/w] -10000, 0     , -100   , relative room effect level at high frequencies                                (SUPPORTED:EAX/I3DL2/SFX) *)
    RoomLF            : Integer;   (* [r/w] -10000, 0     , 0      , relative room effect level at low frequencies                                 (SUPPORTED:EAX/SFX) *)
    DecayTime         : Single;    (* [r/w] 0.1   , 20.0  , 1.49   , reverberation decay time at mid frequencies                                   (SUPPORTED:EAX/I3DL2/SFX/WII) *)
    DecayHFRatio      : Single;    (* [r/w] 0.1   , 2.0   , 0.83   , high-frequency to mid-frequency decay time ratio                              (SUPPORTED:EAX/I3DL2/SFX) *)
    DecayLFRatio      : Single;    (* [r/w] 0.1   , 2.0   , 1.0    , low-frequency to mid-frequency decay time ratio                               (SUPPORTED:EAX) *)
    Reflections       : Integer;   (* [r/w] -10000, 1000  , -2602  , early reflections level relative to room effect                               (SUPPORTED:EAX/I3DL2/SFX/WII) *)
    ReflectionsDelay  : Single;    (* [r/w] 0.0   , 0.3   , 0.007  , initial reflection delay time                                                 (SUPPORTED:EAX/I3DL2/SFX) *)
    ReflectionsPan    : array[0..2] of Single;
                                   (* [r/w]       ,       , [0,0,0], early reflections panning vector                                              (SUPPORTED:EAX) *)
    Reverb            : Integer;   (* [r/w] -10000, 2000  , 200    , late reverberation level relative to room effect                              (SUPPORTED:EAX/I3DL2/SFX) *)
    ReverbDelay       : Single;    (* [r/w] 0.0   , 0.1   , 0.011  , late reverberation delay time relative to initial reflection                  (SUPPORTED:EAX/I3DL2/SFX/WII) *)
    ReverbPan         : array[0..2] of Single;
                                   (* [r/w]       ,       , [0,0,0], late reverberation panning vector                                             (SUPPORTED:EAX) *)
    EchoTime          : Single;    (* [r/w] .075  , 0.25  , 0.25   , echo time                                                                     (SUPPORTED:EAX/PS2(FMOD_PRESET_PS2_ECHO/FMOD_PRESET_PS2_DELAY only) *)
    EchoDepth         : Single;    (* [r/w] 0.0   , 1.0   , 0.0    , echo depth                                                                    (SUPPORTED:EAX/PS2(FMOD_PRESET_PS2_ECHO only) *)
    ModulationTime    : Single;    (* [r/w] 0.04  , 4.0   , 0.25   , modulation time                                                               (SUPPORTED:EAX) *)
    ModulationDepth   : Single;    (* [r/w] 0.0   , 1.0   , 0.0    , modulation depth                                                              (SUPPORTED:EAX/WII) *)
    AirAbsorptionHF   : Single;    (* [r/w] -100  , 0.0   , -5.0   , change in level per meter at high frequencies                                 (SUPPORTED:EAX) *)
    HFReference       : Single;    (* [r/w] 1000.0, 20000 , 5000.0 , reference high frequency (hz)                                                 (SUPPORTED:EAX/I3DL2/SFX) *)
    LFReference       : Single;    (* [r/w] 20.0  , 1000.0, 250.0  , reference low frequency (hz)                                                  (SUPPORTED:EAX/SFX) *)
    RoomRolloffFactor : Single;    (* [r/w] 0.0   , 10.0  , 0.0    , like rolloffscale in System::set3DSettings but for reverb room size effect    (SUPPORTED:EAX/I3DL2) *)
    Diffusion         : Single;    (* [r/w] 0.0   , 100.0 , 100.0  , Value that controls the echo density in the late reverberation decay.         (SUPPORTED:I3DL2/SFX) *)
    Density           : Single;    (* [r/w] 0.0   , 100.0 , 100.0  , Value that controls the modal density in the late reverberation decay         (SUPPORTED:I3DL2/SFX) *)
    Flags             : Cardinal;  (* [r/w] FMOD_REVERB_FLAGS - modifies the behavior of above properties                                          (SUPPORTED:EAX/PS2/WII) *)
  end;


(*
[DEFINE] 
[
    [NAME] 
    FMOD_REVERB_FLAGS

    [DESCRIPTION]
    Values for the Flags member of the FMOD_REVERB_PROPERTIES structure.

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    FMOD_REVERB_PROPERTIES
]
*)
const
  FMOD_REVERB_FLAGS_DECAYTIMESCALE        = $00000001; (* 'EnvSize' affects reverberation decay time *)
  FMOD_REVERB_FLAGS_REFLECTIONSSCALE      = $00000002; (* 'EnvSize' affects reflection level *)
  FMOD_REVERB_FLAGS_REFLECTIONSDELAYSCALE = $00000004; (* 'EnvSize' affects initial reflection delay time *)
  FMOD_REVERB_FLAGS_REVERBSCALE           = $00000008; (* 'EnvSize' affects reflections level *)
  FMOD_REVERB_FLAGS_REVERBDELAYSCALE      = $00000010; (* 'EnvSize' affects late reverberation delay time *)
  FMOD_REVERB_FLAGS_DECAYHFLIMIT          = $00000020; (* AirAbsorptionHF affects DecayHFRatio *)
  FMOD_REVERB_FLAGS_ECHOTIMESCALE         = $00000040; (* 'EnvSize' affects echo time *)
  FMOD_REVERB_FLAGS_MODULATIONTIMESCALE   = $00000080; (* 'EnvSize' affects modulation time *)
  FMOD_REVERB_FLAGS_CORE0                 = $00000100; (* PS2 Only - Reverb is applied to CORE0 (hw voices 0-23) *)
  FMOD_REVERB_FLAGS_CORE1                 = $00000200; (* PS2 Only - Reverb is applied to CORE1 (hw voices 24-47) *)
  FMOD_REVERB_FLAGS_HIGHQUALITYREVERB     = $00000400; (* GameCube/Wii. Use high quality reverb *)
  FMOD_REVERB_FLAGS_HIGHQUALITYDPL2REVERB = $00000800; (* GameCube/Wii. Use high quality DPL2 reverb *)

  FMOD_REVERB_FLAGS_DEFAULT =
    (FMOD_REVERB_FLAGS_DECAYTIMESCALE or
     FMOD_REVERB_FLAGS_REFLECTIONSSCALE or
     FMOD_REVERB_FLAGS_REFLECTIONSDELAYSCALE or
     FMOD_REVERB_FLAGS_REVERBSCALE or
     FMOD_REVERB_FLAGS_REVERBDELAYSCALE or
     FMOD_REVERB_FLAGS_DECAYHFLIMIT or
     FMOD_REVERB_FLAGS_CORE0 or
     FMOD_REVERB_FLAGS_CORE1);
(* [DEFINE_END] *)


(*
[DEFINE] 
[
    [NAME] 
    FMOD_REVERB_PRESETS

    [DESCRIPTION]   
    A set of predefined environment PARAMETERS, created by Creative Labs
    These are used to initialize an FMOD_REVERB_PROPERTIES structure statically.
    ie
    FMOD_REVERB_PROPERTIES prop = FMOD_PRESET_GENERIC;

    [REMARKS]

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::setReverbProperties
]
*)
(* TODO *)
//(*                                    Inst Env  Size    Diffus  Room   RoomHF  RmLF DecTm   DecHF  DecLF   Refl  RefDel  RefPan               Revb  RevDel  ReverbPan           EchoTm  EchDp  ModTm  ModDp  AirAbs  HFRef    LFRef  RRlOff Diffus  Densty  FLAGS *)
//#define FMOD_PRESET_OFF              {  0, -1,  7.5f,   1.00f, -10000, -10000, 0,   1.00f,  1.00f, 1.0f,  -2602, 0.007f, { 0.0f,0.0f,0.0f },   200, 0.011f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f,   0.0f,   0.0f, 0x33f }
//#define FMOD_PRESET_GENERIC          {  0,  0,  7.5f,   1.00f, -1000,  -100,   0,   1.49f,  0.83f, 1.0f,  -2602, 0.007f, { 0.0f,0.0f,0.0f },   200, 0.011f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_PADDEDCELL       {  0,  1,  1.4f,   1.00f, -1000,  -6000,  0,   0.17f,  0.10f, 1.0f,  -1204, 0.001f, { 0.0f,0.0f,0.0f },   207, 0.002f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_ROOM             {  0,  2,  1.9f,   1.00f, -1000,  -454,   0,   0.40f,  0.83f, 1.0f,  -1646, 0.002f, { 0.0f,0.0f,0.0f },    53, 0.003f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_BATHROOM         {  0,  3,  1.4f,   1.00f, -1000,  -1200,  0,   1.49f,  0.54f, 1.0f,   -370, 0.007f, { 0.0f,0.0f,0.0f },  1030, 0.011f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f,  60.0f, 0x3f }
//#define FMOD_PRESET_LIVINGROOM       {  0,  4,  2.5f,   1.00f, -1000,  -6000,  0,   0.50f,  0.10f, 1.0f,  -1376, 0.003f, { 0.0f,0.0f,0.0f }, -1104, 0.004f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_STONEROOM        {  0,  5,  11.6f,  1.00f, -1000,  -300,   0,   2.31f,  0.64f, 1.0f,   -711, 0.012f, { 0.0f,0.0f,0.0f },    83, 0.017f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_AUDITORIUM       {  0,  6,  21.6f,  1.00f, -1000,  -476,   0,   4.32f,  0.59f, 1.0f,   -789, 0.020f, { 0.0f,0.0f,0.0f },  -289, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_CONCERTHALL      {  0,  7,  19.6f,  1.00f, -1000,  -500,   0,   3.92f,  0.70f, 1.0f,  -1230, 0.020f, { 0.0f,0.0f,0.0f },    -2, 0.029f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_CAVE             {  0,  8,  14.6f,  1.00f, -1000,  0,      0,   2.91f,  1.30f, 1.0f,   -602, 0.015f, { 0.0f,0.0f,0.0f },  -302, 0.022f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x1f }
//#define FMOD_PRESET_ARENA            {  0,  9,  36.2f,  1.00f, -1000,  -698,   0,   7.24f,  0.33f, 1.0f,  -1166, 0.020f, { 0.0f,0.0f,0.0f },    16, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_HANGAR           {  0,  10, 50.3f,  1.00f, -1000,  -1000,  0,   10.05f, 0.23f, 1.0f,   -602, 0.020f, { 0.0f,0.0f,0.0f },   198, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_CARPETTEDHALLWAY {  0,  11, 1.9f,   1.00f, -1000,  -4000,  0,   0.30f,  0.10f, 1.0f,  -1831, 0.002f, { 0.0f,0.0f,0.0f }, -1630, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_HALLWAY          {  0,  12, 1.8f,   1.00f, -1000,  -300,   0,   1.49f,  0.59f, 1.0f,  -1219, 0.007f, { 0.0f,0.0f,0.0f },   441, 0.011f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_STONECORRIDOR    {  0,  13, 13.5f,  1.00f, -1000,  -237,   0,   2.70f,  0.79f, 1.0f,  -1214, 0.013f, { 0.0f,0.0f,0.0f },   395, 0.020f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_ALLEY            {  0,  14, 7.5f,   0.30f, -1000,  -270,   0,   1.49f,  0.86f, 1.0f,  -1204, 0.007f, { 0.0f,0.0f,0.0f },    -4, 0.011f, { 0.0f,0.0f,0.0f }, 0.125f, 0.95f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_FOREST           {  0,  15, 38.0f,  0.30f, -1000,  -3300,  0,   1.49f,  0.54f, 1.0f,  -2560, 0.162f, { 0.0f,0.0f,0.0f },  -229, 0.088f, { 0.0f,0.0f,0.0f }, 0.125f, 1.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f,  79.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_CITY             {  0,  16, 7.5f,   0.50f, -1000,  -800,   0,   1.49f,  0.67f, 1.0f,  -2273, 0.007f, { 0.0f,0.0f,0.0f }, -1691, 0.011f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f,  50.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_MOUNTAINS        {  0,  17, 100.0f, 0.27f, -1000,  -2500,  0,   1.49f,  0.21f, 1.0f,  -2780, 0.300f, { 0.0f,0.0f,0.0f }, -1434, 0.100f, { 0.0f,0.0f,0.0f }, 0.250f, 1.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f,  27.0f, 100.0f, 0x1f }
//#define FMOD_PRESET_QUARRY           {  0,  18, 17.5f,  1.00f, -1000,  -1000,  0,   1.49f,  0.83f, 1.0f, -10000, 0.061f, { 0.0f,0.0f,0.0f },   500, 0.025f, { 0.0f,0.0f,0.0f }, 0.125f, 0.70f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_PLAIN            {  0,  19, 42.5f,  0.21f, -1000,  -2000,  0,   1.49f,  0.50f, 1.0f,  -2466, 0.179f, { 0.0f,0.0f,0.0f }, -1926, 0.100f, { 0.0f,0.0f,0.0f }, 0.250f, 1.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f,  21.0f, 100.0f, 0x3f }
//#define FMOD_PRESET_PARKINGLOT       {  0,  20, 8.3f,   1.00f, -1000,  0,      0,   1.65f,  1.50f, 1.0f,  -1363, 0.008f, { 0.0f,0.0f,0.0f }, -1153, 0.012f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x1f }
//#define FMOD_PRESET_SEWERPIPE        {  0,  21, 1.7f,   0.80f, -1000,  -1000,  0,   2.81f,  0.14f, 1.0f,    429, 0.014f, { 0.0f,0.0f,0.0f },  1023, 0.021f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 0.000f, -5.0f, 5000.0f, 250.0f, 0.0f,  80.0f,  60.0f, 0x3f }
//#define FMOD_PRESET_UNDERWATER       {  0,  22, 1.8f,   1.00f, -1000,  -4000,  0,   1.49f,  0.10f, 1.0f,   -449, 0.007f, { 0.0f,0.0f,0.0f },  1700, 0.011f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 1.18f, 0.348f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x3f }
//
//(* Non I3DL2 presets *)
//
//#define FMOD_PRESET_DRUGGED          {  0,  23, 1.9f,   0.50f, -1000,  0,      0,   8.39f,  1.39f, 1.0f,  -115,  0.002f, { 0.0f,0.0f,0.0f },   985, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.25f, 1.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x1f }
//#define FMOD_PRESET_DIZZY            {  0,  24, 1.8f,   0.60f, -1000,  -400,   0,   17.23f, 0.56f, 1.0f,  -1713, 0.020f, { 0.0f,0.0f,0.0f },  -613, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 1.00f, 0.81f, 0.310f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x1f }
//#define FMOD_PRESET_PSYCHOTIC        {  0,  25, 1.0f,   0.50f, -1000,  -151,   0,   7.56f,  0.91f, 1.0f,  -626,  0.020f, { 0.0f,0.0f,0.0f },   774, 0.030f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 4.00f, 1.000f, -5.0f, 5000.0f, 250.0f, 0.0f, 100.0f, 100.0f, 0x1f }
//
//(* PlayStation 2 / PlayStation Portable Only presets *)
//
//#define FMOD_PRESET_PS2_ROOM         {  0,  1,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_STUDIO_A     {  0,  2,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_STUDIO_B     {  0,  3,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_STUDIO_C     {  0,  4,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_HALL         {  0,  5,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_SPACE        {  0,  6,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_ECHO         {  0,  7,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.75f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_DELAY        {  0,  8,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//#define FMOD_PRESET_PS2_PIPE         {  0,  9,  0,      0,     0,      0,      0,   0.0f,   0.0f,  0.0f,     0,  0.000f, { 0.0f,0.0f,0.0f },     0, 0.000f, { 0.0f,0.0f,0.0f }, 0.250f, 0.00f, 0.00f, 0.000f,  0.0f, 0000.0f,   0.0f, 0.0f,   0.0f,   0.0f, 0x31f }
//(* [DEFINE_END] *)


(*
[STRUCTURE] 
[
    [DESCRIPTION]
    Structure defining the properties for a reverb source, related to a FMOD channel.
    
    For more indepth descriptions of the reverb properties under win32, please see the EAX3
    documentation at http://developer.creative.com/ under the 'downloads' section.
    If they do not have the EAX3 documentation, then most information can be attained from
    the EAX2 documentation, as EAX3 only adds some more parameters and functionality on top of 
    EAX2.
    
    Note the default reverb properties are the same as the FMOD_PRESET_GENERIC preset.
    Note that integer values that typically range from -10,000 to 1000 are represented in 
    decibels, and are of a logarithmic scale, not linear, wheras float values are typically linear.
    PORTABILITY: Each member has the platform it supports in braces ie (win32/Xbox).
    Some reverb parameters are only supported in win32 and some only on Xbox. If all parameters are set then
    the reverb should product a similar effect on either platform.
    
    The numerical values listed below are the maximum, minimum and default values for each variable respectively.

    [REMARKS]
    <b>SUPPORTED</b> next to each parameter means the platform the parameter can be set on.  Some platforms support all parameters and some don't.
    EAX   means hardware reverb on FMOD_OUTPUTTYPE_DSOUND on windows only (must use FMOD_HARDWARE), on soundcards that support EAX 1 to 4.
    EAX4  means hardware reverb on FMOD_OUTPUTTYPE_DSOUND on windows only (must use FMOD_HARDWARE), on soundcards that support EAX 4.
    I3DL2 means hardware reverb on FMOD_OUTPUTTYPE_DSOUND on windows only (must use FMOD_HARDWARE), on soundcards that support I3DL2 non EAX native reverb.
    WII   means Nintendo Wii hardware reverb (must use FMOD_HARDWARE).
    PS2   means Playstation 2 hardware reverb (must use FMOD_HARDWARE).
    SFX   means FMOD SFX software reverb.  This works on any platform that uses FMOD_SOFTWARE for loading sounds.
    
    
    <b>'ConnectionPoint' Parameter.</b>  This parameter is for the FMOD software reverb only (known as SFX in the list above).
    By default the dsp network connection for a channel and its reverb is between the 'SFX Reverb' unit, and the channel's wavetable/resampler/dspcodec/oscillator unit (the unit below the channel DSP head).  NULL can be used for this parameter to make it use this default behaviour.
    This parameter allows the user to connect the SFX reverb to somewhere else internally, for example the channel DSP head, or a related channelgroup.  The event system uses this so that it can have the output of an event going to the reverb, instead of just the output of the event's channels (thereby ignoring event effects/submixes etc).
    Do not use if you are unaware of DSP network connection issues.  Leave it at the default of NULL instead.
    
    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.
    Members marked with [r/w] are either read or write depending on if you are using Channel::setReverbProperties (w) or Channel::getReverbProperties (r).

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    Channel::setReverbProperties
    Channel::getReverbProperties
    FMOD_REVERB_CHANNELFLAGS 
]
*)
type
  FMOD_REVERB_CHANNELPROPERTIES = Record
                                   (* MIN     MAX    DEFAULT  DESCRIPTION *)
    Direct: Integer;               (* [r/w] -10000, 1000,  0,       direct path level (at low and mid frequencies)              (SUPPORTED:EAX/I3DL2/SFX) *)
    DirectHF: Integer;             (* [r/w] -10000, 0,     0,       relative direct path level at high frequencies              (SUPPORTED:EAX/I3DL2) *)
    Room: Integer;                 (* [r/w] -10000, 1000,  0,       room effect level (at low and mid frequencies)              (SUPPORTED:EAX/I3DL2/SFX) *)
    RoomHF: Integer;               (* [r/w] -10000, 0,     0,       relative room effect level at high frequencies              (SUPPORTED:EAX/I3DL2) *)
    Obstruction: Integer;          (* [r/w] -10000, 0,     0,       main obstruction control (attenuation at high frequencies)  (SUPPORTED:EAX/I3DL2) *)
    ObstructionLFRatio: Single;    (* [r/w] 0.0,    1.0,   0.0,     obstruction low-frequency level re. main control            (SUPPORTED:EAX/I3DL2) *)
    Occlusion: Integer;            (* [r/w] -10000, 0,     0,       main occlusion control (attenuation at high frequencies)    (SUPPORTED:EAX/I3DL2) *)
    OcclusionLFRatio: Single;      (* [r/w] 0.0,    1.0,   0.25,    occlusion low-frequency level re. main control              (SUPPORTED:EAX/I3DL2) *)
    OcclusionRoomRatio: Single;    (* [r/w] 0.0,    10.0,  1.5,     relative occlusion control for room effect                  (SUPPORTED:EAX only) *)
    OcclusionDirectRatio: Single;  (* [r/w] 0.0,    10.0,  1.0,     relative occlusion control for direct path                  (SUPPORTED:EAX only) *)
    Exclusion: Integer;            (* [r/w] -10000, 0,     0,       main exlusion control (attenuation at high frequencies)     (SUPPORTED:EAX only) *)
    ExclusionLFRatio: Single;      (* [r/w] 0.0,    1.0,   1.0,     exclusion low-frequency level re. main control              (SUPPORTED:EAX only) *)
    OutsideVolumeHF: Integer;      (* [r/w] -10000, 0,     0,       outside sound cone level at high frequencies                (SUPPORTED:EAX only) *)
    DopplerFactor: Single;         (* [r/w] 0.0,    10.0,  0.0,     like DS3D flDopplerFactor but per source                    (SUPPORTED:EAX only) *)
    RolloffFactor: Single;         (* [r/w] 0.0,    10.0,  0.0,     like DS3D flRolloffFactor but per source                    (SUPPORTED:EAX only) *)
    RoomRolloffFactor: Single;     (* [r/w] 0.0,    10.0,  0.0,     like DS3D flRolloffFactor but for room effect               (SUPPORTED:EAX/I3DL2) *)
    AirAbsorptionFactor: Single;   (* [r/w] 0.0,    10.0,  1.0,     multiplies AirAbsorptionHF member of FMOD_REVERB_PROPERTIES (SUPPORTED:EAX only) *)
    Flags: Cardinal;               (* [r/w] FMOD_REVERB_CHANNELFLAGS - modifies the behavior of properties                      (SUPPORTED:EAX/SFX) *)
    ConnectionPoint: FMOD_DSP;     (* [r/w] See remarks.            DSP network location to connect reverb for this channel.    (SUPPORTED:SFX only).*)
end;


(*
[DEFINE] 
[
    [NAME] 
    FMOD_REVERB_CHANNELFLAGS

    [DESCRIPTION]
    Values for the Flags member of the FMOD_REVERB_CHANNELPROPERTIES structure.

    [REMARKS]
    For EAX4,EAX5 and SFX, there is support for multiple reverb environments.
    Use FMOD_REVERB_CHANNELFLAGS_ENVIRONMENT0 to FMOD_REVERB_CHANNELFLAGS_ENVIRONMENT3 in the flags member 
    of FMOD_REVERB_CHANNELPROPERTIES to specify which environment instance(s) to target. 
    - If you do not specify any instance the first reverb instance will be used.
    - If you specify more than one instance with getReverbProperties, the first instance will be used.
    - If you specify more than one instance with setReverbProperties, it will set more than 1 instance at once.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    FMOD_REVERB_CHANNELPROPERTIES
]
*)
const
  FMOD_REVERB_CHANNELFLAGS_DIRECTHFAUTO  = $00000001; (* Automatic setting of 'Direct'  due to distance from listener *)
  FMOD_REVERB_CHANNELFLAGS_ROOMAUTO      = $00000002; (* Automatic setting of 'Room'  due to distance from listener *)
  FMOD_REVERB_CHANNELFLAGS_ROOMHFAUTO    = $00000004; (* Automatic setting of 'RoomHF' due to distance from listener *)
  FMOD_REVERB_CHANNELFLAGS_INSTANCE0     = $00000010; (* EAX4/SFX/Wii. Specify channel to target reverb instance 0.  Default target. *)
  FMOD_REVERB_CHANNELFLAGS_INSTANCE1     = $00000020; (* EAX4/SFX/Wii. Specify channel to target reverb instance 1. *)
  FMOD_REVERB_CHANNELFLAGS_INSTANCE2     = $00000040; (* EAX4/SFX/Wii. Specify channel to target reverb instance 2. *)
  FMOD_REVERB_CHANNELFLAGS_INSTANCE3     = $00000080; (* EAX5/SFX. Specify channel to target reverb instance 3. *)

  FMOD_REVERB_CHANNELFLAGS_DEFAULT       = 
    (FMOD_REVERB_CHANNELFLAGS_DIRECTHFAUTO or
     FMOD_REVERB_CHANNELFLAGS_ROOMAUTO or
     FMOD_REVERB_CHANNELFLAGS_ROOMHFAUTO or
     FMOD_REVERB_CHANNELFLAGS_INSTANCE0);
(* [DEFINE_END] *)


(*
[STRUCTURE] 
[
    [DESCRIPTION]
    Settings for advanced features like configuring memory and cpu usage for the FMOD_CREATECOMPRESSEDSAMPLE feature.

    [REMARKS]
    maxMPEGcodecs / maxADPCMcodecs / maxXMAcodecs will determine the maximum cpu usage of playing realtime samples.  Use this to lower potential excess cpu usage and also control memory usage.
    
    maxPCMcodecs is for use with PS3 only. It will determine the maximum number of PCM voices that can be played at once. This includes streams of any format and all sounds created
    *without* the FMOD_CREATECOMPRESSEDSAMPLE flag.
    
    Memory will be allocated for codecs 'up front' (during System::init) if these values are specified as non zero.  If any are zero, it allocates memory for the codec whenever a file of the type in question is loaded.  So if maxMPEGcodecs is 0 for example, it will allocate memory for the mpeg codecs the first time an mp3 is loaded or an mp3 based .FSB file is loaded.

    Due to inefficient encoding techniques on certain .wav based ADPCM files, FMOD can can need an extra 29720 bytes per codec.  This means for lowest memory consumption.  Use FSB as it uses an optimal/small ADPCM block size.

    Members marked with [r] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.
    Members marked with [w] mean the variable can be written to.  The user can set the value.
    Members marked with [r/w] are either read or write depending on if you are using System::setAdvancedSettings (w) or System::getAdvancedSettings (r).

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::setAdvancedSettings
    System::getAdvancedSettings
    System::init
    FMOD_MODE
]
*)
type
  FMOD_ADVANCEDSETTINGS = Record
    cbsize: Integer;                     (* [w]   Size of this structure.  Use sizeof(FMOD_ADVANCEDSETTINGS)  NOTE: This must be set before calling System::getAdvancedSettings! *)
    maxMPEGcodecs: Integer;              (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_CREATECOMPRESSEDSAMPLE only.  Mpeg  codecs consume 21,684 bytes per instance and this number will determine how many mpeg channels can be played simultaneously.   Default = 32. *)
    maxADPCMcodecs: Integer;             (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_CREATECOMPRESSEDSAMPLE only.  ADPCM codecs consume  2,136 bytes per instance and this number will determine how many ADPCM channels can be played simultaneously.  Default = 32. *)
    maxXMAcodecs: Integer;               (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_CREATECOMPRESSEDSAMPLE only.  XMA   codecs consume 14,836 bytes per instance and this number will determine how many XMA channels can be played simultaneously.    Default = 32. *)
    maxCELTcodecs: Integer;              (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_CREATECOMPRESSEDSAMPLE only.  CELT  codecs consume 11,500 bytes per instance and this number will determine how many CELT channels can be played simultaneously.   Default = 32. *)    
    maxPCMcodecs: Integer;               (* [r/w] Optional. Specify 0 to ignore. For use with PS3 only.                          PCM   codecs consume 12,672 bytes per instance and this number will determine how many streams and PCM voices can be played simultaneously. Default = 16. *)
    ASIONumChannels: Integer;            (* [r/w] Optional. Specify 0 to ignore. Number of channels available on the ASIO device. *)
    ASIOChannelList: Pointer;            (* [r/w] Optional. Specify 0 to ignore. Pointer to an array of strings (number of entries defined by ASIONumChannels) with ASIO channel names. *)
    ASIOSpeakerList:Pointer;            (* [r/w] Optional. Specify 0 to ignore. Pointer to a list of speakers that the ASIO channels map to.  This can be called after System::init to remap ASIO output. *)
    max3DReverbDSPs: Integer;            (* [r/w] Optional. Specify 0 to ignore. The max number of 3d reverb DSP's in the system. (NOTE: CURRENTLY DISABLED / UNUSED) *)
    HRTFMinAngle: Single;               (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_INIT_SOFTWARE_HRTF.  The angle range (0-360) of a 3D sound in relation to the listener, at which the HRTF function begins to have an effect. 0 = in front of the listener. 180 = from 90 degrees to the left of the listener to 90 degrees to the right. 360 = behind the listener. Default = 180.0. *)
    HRTFMaxAngle: Single;               (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_INIT_SOFTWARE_HRTF.  The angle range (0-360) of a 3D sound in relation to the listener, at which the HRTF function has maximum effect. 0 = front of the listener. 180 = from 90 degrees to the left of the listener to 90 degrees to the right. 360 = behind the listener. Default = 360.0. *)
    HRTFFreq: Single;                   (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_INIT_SOFTWARE_HRTF.  The cutoff frequency of the HRTF's lowpass filter function when at maximum effect. (i.e. at HRTFMaxAngle).  Default = 4000.0. *)
    vol0virtualvol: Single;             (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_INIT_VOL0_BECOMES_VIRTUAL.  If this flag is used, and the volume is 0.0, then the sound will become virtual.  Use this value to raise the threshold to a different point where a sound goes virtual. *)
    eventqueuesize: Integer;             (* [r/w] Optional. Specify 0 to ignore. For use with FMOD Event system only.  Specifies the number of slots available for simultaneous non blocking loads.  Default = 32. *)
    defaultDecodeBufferSize: Cardinal;    (* [r/w] Optional. Specify 0 to ignore. For streams. This determines the default size of the double buffer (in milliseconds) that a stream uses.  Default = 400ms *)
    debugLogFilename: Pchar;           (* [r/w] Optional. Specify 0 to ignore. Gives fmod's logging system a path/filename.  Normally the log is placed in the same directory as the executable and called fmod.log. When using System::getAdvancedSettings, provide at least 256 bytes of memory to copy into. *)
    profileport: Word;                (* [r/w] Optional. Specify 0 to ignore. For use with FMOD_INIT_ENABLE_PROFILE.  Specify the port to listen on for connections by the profiler application. *)
    geometryMaxFadeTime: Cardinal;        (* [r/w] Optional. Specify 0 to ignore. The maximum time in miliseconds it takes for a channel to fade to the new level when its occlusion changes. *)
    maxSpectrumWaveDataBuffers: Cardinal; (* [r/w] Optional. Specify 0 to ignore. Tells System::init to allocate a pool of wavedata/spectrum buffers to prevent memory fragmentation, any additional buffers will be allocated normally. *)
    musicSystemCacheDelay: Cardinal;      (* [r/w] Optional. Specify 0 to ignore. The delay the music system should allow for loading a sample from disk (in milliseconds). Default = 400 ms. *)
  end;


(*
[ENUM]
[
    [DESCRIPTION]
    Special channel index values for FMOD functions.

    [REMARKS]
    To get 'all' of the channels, use System::getMasterChannelGroup.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, iPhone

    [SEE_ALSO]
    System::playSound
    System::playDSP
    System::getChannel
    System::getMasterChannelGroup
]
*)
type
  FMOD_CHANNELINDEX =
  (
    FMOD_CHANNEL_FREE  = -1,      (* For a channel index, FMOD chooses a free voice using the priority system. *)
    FMOD_CHANNEL_REUSE = -2       (* For a channel index, re-use the channel handle that was passed in. *)
  );

{$I fmod_memoryinfo.inc.pas}
{$I fmod_dsp.inc.pas}
{$I fmod_codec.inc.pas}

(* ========================================================================================== *)
(* FUNCTION PROTOTYPES                                                                        *)
(* ========================================================================================== *)

(*
    FMOD global system functions (optional).
*)

FMOD_Memory_Initialize = function (poolmem: Pointer; poollen: Integer; useralloc: FMOD_MEMORY_ALLOCCALLBACK;userrealloc: FMOD_MEMORY_REALLOCCALLBACK;userfree: FMOD_MEMORY_FREECALLBACK;memtypeflags: FMOD_MEMORY_TYPE):FMOD_RESULT; stdcall;
FMOD_Memory_GetStats = function (var currentalloced: Integer; var maxalloced: Integer; blocking: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Debug_SetLevel = function (level: FMOD_DEBUGLEVEL):FMOD_RESULT; stdcall;
FMOD_Debug_GetLevel = function (var level:FMOD_DEBUGLEVEL):FMOD_RESULT; stdcall;
FMOD_File_SetDiskBusy = function (busy: Integer):FMOD_RESULT; stdcall;
FMOD_File_GetDiskBusy = function (var busy: Integer):FMOD_RESULT; stdcall;

(*
    FMOD System factory functions.  Use this to create an FMOD System Instance.  below you will see FMOD_System_Init/Close to get started.
*)

FMOD_System_Create = function (var system:FMOD_SYSTEM):FMOD_RESULT; stdcall;
FMOD_System_Release = function (system:FMOD_SYSTEM):FMOD_RESULT; stdcall;


(*
    'System' API
*)

(*
     Pre-init functions.
*)

FMOD_System_SetOutput           = function (system: FMOD_SYSTEM; output: FMOD_OUTPUTTYPE ):FMOD_RESULT; stdcall;
FMOD_System_GetOutput           = function (system: FMOD_SYSTEM; var output: FMOD_OUTPUTTYPE):FMOD_RESULT; stdcall;
FMOD_System_GetNumDrivers       = function (system: FMOD_SYSTEM; var numdrivers: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetDriverInfo       = function (system: FMOD_SYSTEM; id: Integer; name: PChar; namelen: Integer; var guid: FMOD_GUID):FMOD_RESULT; stdcall;
FMOD_System_GetDriverInfoW      = function (system: FMOD_SYSTEM; id: Integer; var name:SmallInt; namelen: Integer; var guid: FMOD_GUID):FMOD_RESULT; stdcall;
FMOD_System_GetDriverCaps       = function (system: FMOD_SYSTEM; id: Integer; var caps: FMOD_CAPS; var minfrequency: Integer; var maxfrequency: Integer; var controlpanelspeakermode: FMOD_SPEAKERMODE):FMOD_RESULT; stdcall;
FMOD_System_SetDriver           = function (system: FMOD_SYSTEM; driver: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetDriver           = function (system: FMOD_SYSTEM; var driver: Integer):FMOD_RESULT; stdcall;
FMOD_System_SetHardwareChannels = function (system: FMOD_SYSTEM; min2d: Integer; max2d: Integer; min3d: Integer; max3d: Integer):FMOD_RESULT; stdcall;
FMOD_System_SetSoftwareChannels = function (system: FMOD_SYSTEM; numsoftwarechannels: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetSoftwareChannels = function (system: FMOD_SYSTEM; var numsoftwarechannels: Integer):FMOD_RESULT; stdcall;
FMOD_System_SetSoftwareFormat   = function (system: FMOD_SYSTEM; samplerate: Integer; format:FMOD_SOUND_FORMAT; numoutputchannels: Integer; maxinputchannels: Integer; resamplemethod: FMOD_DSP_RESAMPLER):FMOD_RESULT; stdcall;
FMOD_System_GetSoftwareFormat   = function (system: FMOD_SYSTEM; var samplerate: Integer; var format: FMOD_SOUND_FORMAT; var numoutputchannels: Integer; var maxinputchannels: Integer; var resamplemethod: FMOD_DSP_RESAMPLER; var bits: Integer):FMOD_RESULT; stdcall;
FMOD_System_SetDSPBufferSize    = function (system: FMOD_SYSTEM; bufferlength: Cardinal; numbuffers: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetDSPBufferSize    = function (system: FMOD_SYSTEM; var bufferlength: Cardinal; var numbuffers: Integer):FMOD_RESULT; stdcall;
FMOD_System_SetFileSystem       = function (system: FMOD_SYSTEM; useropen: FMOD_FILE_OPENCALLBACK; userclose: FMOD_FILE_CLOSECALLBACK; userread: FMOD_FILE_READCALLBACK; userseek: FMOD_FILE_SEEKCALLBACK; userasyncread: FMOD_FILE_ASYNCREADCALLBACK; userasynccancel: FMOD_FILE_ASYNCCANCELCALLBACK; blockalign: Integer):FMOD_RESULT; stdcall;
FMOD_System_AttachFileSystem    = function (system: FMOD_SYSTEM; useropen: FMOD_FILE_OPENCALLBACK; userclose: FMOD_FILE_CLOSECALLBACK; userread: FMOD_FILE_READCALLBACK; userseek: FMOD_FILE_SEEKCALLBACK):FMOD_RESULT; stdcall;
FMOD_System_SetAdvancedSettings = function (system: FMOD_SYSTEM; var settings: FMOD_ADVANCEDSETTINGS):FMOD_RESULT; stdcall;
FMOD_System_GetAdvancedSettings = function (system: FMOD_SYSTEM; var settings: FMOD_ADVANCEDSETTINGS):FMOD_RESULT; stdcall;
FMOD_System_SetSpeakerMode      = function (system: FMOD_SYSTEM; speakermode: FMOD_SPEAKERMODE):FMOD_RESULT; stdcall;
FMOD_System_GetSpeakerMode      = function (system: FMOD_SYSTEM; speakermode: FMOD_SPEAKERMODE):FMOD_RESULT; stdcall;
FMOD_System_SetCallback         = function (system: FMOD_SYSTEM; callback: FMOD_SYSTEM_CALLBACK):FMOD_RESULT; stdcall;

(*
     Plug-in support                       
*)

FMOD_System_SetPluginPath     = function (system: FMOD_SYSTEM; const path: PChar):FMOD_RESULT; stdcall;
FMOD_System_LoadPlugin        = function (system: FMOD_SYSTEM; const filename: PChar; var handle: Cardinal; priority: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_UnloadPlugin      = function (system: FMOD_SYSTEM; handle: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_GetNumPlugins     = function (system: FMOD_SYSTEM; plugintype: FMOD_PLUGINTYPE; var numplugins: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetPluginHandle   = function (system: FMOD_SYSTEM; plugintype: FMOD_PLUGINTYPE; index: Integer; var handle: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_GetPluginInfo     = function (system: FMOD_SYSTEM; handle: Cardinal; var plugintype: FMOD_PLUGINTYPE; name: PChar; namelen: Integer; var version: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_SetOutputByPlugin = function (system: FMOD_SYSTEM; handle: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_GetOutputByPlugin = function (system: FMOD_SYSTEM; var handle: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_CreateDSPByPlugin = function (system: FMOD_SYSTEM; handle: Cardinal; var dsp: FMOD_DSP):FMOD_RESULT; stdcall;
{ (!) var description: FMOD_CODEC_DESCRIPTION => error => FMOD_CODEC_DESCRIPTION is unknown
      => using "Pointer" but may be wrong: description: Pointer; }
FMOD_System_CreateCodec       = function (system: FMOD_SYSTEM; description: Pointer; priority: Cardinal):FMOD_RESULT; stdcall;

(*
     Init/Close                            
*)

FMOD_System_Init  = function (system: FMOD_SYSTEM; maxchannels: Integer; flags: FMOD_INITFLAGS; extradriverdata: Pointer):FMOD_RESULT; stdcall;
FMOD_System_Close = function (system: FMOD_SYSTEM):FMOD_RESULT; stdcall;

(*
     General post-init system functions    
*)

FMOD_System_Update = function (system: FMOD_SYSTEM):FMOD_RESULT; stdcall;

FMOD_System_Set3DSettings           = function (system: FMOD_SYSTEM; dopplerscale: Single; distancefactor: Single; rolloffscale: Single):FMOD_RESULT; stdcall;
FMOD_System_Get3DSettings           = function (system: FMOD_SYSTEM; var dopplerscale: Single; var distancefactor: Single; var rolloffscale: Single):FMOD_RESULT; stdcall;
FMOD_System_Set3DNumListeners       = function (system: FMOD_SYSTEM; numlisteners: Integer):FMOD_RESULT; stdcall;
FMOD_System_Get3DNumListeners       = function (system: FMOD_SYSTEM; var numlisteners: Integer):FMOD_RESULT; stdcall;
FMOD_System_Set3DListenerAttributes = function (system: FMOD_SYSTEM; listener: Integer; pos: FMOD_VECTOR; vel: FMOD_VECTOR; forward: FMOD_VECTOR; up: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_System_Get3DListenerAttributes = function (system: FMOD_SYSTEM; listener: Integer; pos: FMOD_VECTOR; vel: FMOD_VECTOR; forward: FMOD_VECTOR; up: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_System_Set3DRolloffCallback    = function (system: FMOD_SYSTEM; callback: FMOD_3D_ROLLOFFCALLBACK):FMOD_RESULT; stdcall;
FMOD_System_Set3DSpeakerPosition    = function (system: FMOD_SYSTEM; speaker: FMOD_SPEAKER; x: Single; y: Single; active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_System_Get3DSpeakerPosition    = function (system: FMOD_SYSTEM; speaker: FMOD_SPEAKER; var x: Single; var y: Single; var active: FMOD_BOOL):FMOD_RESULT; stdcall;

FMOD_System_SetStreamBufferSize = function (system: FMOD_SYSTEM; filebuffersize: Cardinal; filebuffersizetype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_System_GetStreamBufferSize = function (system: FMOD_SYSTEM; var filebuffersize: Cardinal; filebuffersizetype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;

(*
     System information functions.        
*)

FMOD_System_GetVersion          = function (system: FMOD_SYSTEM; var version: Cardinal):FMOD_RESULT; stdcall;
FMOD_System_GetOutputHandle     = function (system: FMOD_SYSTEM; handle: Pointer):FMOD_RESULT; stdcall;
FMOD_System_GetChannelsPlaying  = function (system: FMOD_SYSTEM; var channels: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetHardwareChannels = function (system: FMOD_SYSTEM; var num2d: Integer; var num3d: Integer; var total: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetCPUUsage         = function (system: FMOD_SYSTEM; var dsp: Single; var stream: Single; var geometry: Single; var update: Single; var total: Single):FMOD_RESULT; stdcall;
FMOD_System_GetSoundRAM         = function (system: FMOD_SYSTEM; var currentalloced: Integer; var maxalloced: Integer; var total: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetNumCDROMDrives   = function (system: FMOD_SYSTEM; var numdrives: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetCDROMDriveName   = function (system: FMOD_SYSTEM; drive: Integer; drivename: PChar; drivenamelen: Integer; scsiname: PChar; scsinamelen: Integer; devicename: PChar; devicenamelen: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetSpectrum         = function (system: FMOD_SYSTEM; var spectrumarray: Single; numvalues: Integer; channeloffset: Integer; windowtype: FMOD_DSP_FFT_WINDOW):FMOD_RESULT; stdcall;
FMOD_System_GetWaveData         = function (system: FMOD_SYSTEM; var wavearray: Single; numvalues: Integer; channeloffset: Integer):FMOD_RESULT; stdcall;

(*
     Sound/DSP/Channel/FX creation and retrieval.
*)

FMOD_System_CreateSound        = function (system: FMOD_SYSTEM; const name_or_data: PChar; mode: FMOD_MODE; exinfo: PFMOD_CREATESOUNDEXINFO; var sound: FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_System_CreateStream       = function (system: FMOD_SYSTEM; name_or_data: PChar; mode: FMOD_MODE; var exinfo: FMOD_CREATESOUNDEXINFO; var sound: FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_System_CreateDSP          = function (system: FMOD_SYSTEM; description: FMOD_DSP_DESCRIPTION; var dsp: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_System_CreateDSPByType    = function (system: FMOD_SYSTEM; dsptype: FMOD_DSP_TYPE; var dsp: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_System_CreateChannelGroup = function (system: FMOD_SYSTEM; name: PChar; var channelgroup: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_System_CreateSoundGroup   = function (system: FMOD_SYSTEM; name: PChar; var soundgroup: FMOD_SOUNDGROUP):FMOD_RESULT; stdcall;
FMOD_System_CreateReverb       = function (system: FMOD_SYSTEM; var reverb: FMOD_REVERB):FMOD_RESULT; stdcall;

FMOD_System_PlaySound             = function (system: FMOD_SYSTEM; channelid: FMOD_CHANNELINDEX; sound: FMOD_SOUND; paused:FMOD_BOOL; var channel:FMOD_CHANNEL):FMOD_RESULT; stdcall;
FMOD_System_PlayDSP               = function (system: FMOD_SYSTEM; channelid: FMOD_CHANNELINDEX; dsp: FMOD_DSP; paused: FMOD_BOOL; var channel: FMOD_CHANNEL):FMOD_RESULT; stdcall;
FMOD_System_GetChannel            = function (system: FMOD_SYSTEM; channelid: Integer; var channel:FMOD_CHANNEL):FMOD_RESULT; stdcall;
FMOD_System_GetMasterChannelGroup = function (system: FMOD_SYSTEM; var channelgroup: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_System_GetMasterSoundGroup   = function (system: FMOD_SYSTEM; var soundgroup: FMOD_SOUNDGROUP):FMOD_RESULT; stdcall;

(*
     Reverb API
*)

FMOD_System_SetReverbProperties        = function (system: FMOD_SYSTEM; prop: FMOD_REVERB_PROPERTIES):FMOD_RESULT; stdcall;
FMOD_System_GetReverbProperties        = function (system: FMOD_SYSTEM; var prop: FMOD_REVERB_PROPERTIES):FMOD_RESULT; stdcall;
FMOD_System_SetReverbAmbientProperties = function (system: FMOD_SYSTEM; prop: FMOD_REVERB_PROPERTIES):FMOD_RESULT; stdcall;
FMOD_System_GetReverbAmbientProperties = function (system: FMOD_SYSTEM; var prop: FMOD_REVERB_PROPERTIES):FMOD_RESULT; stdcall;

(*
     System level DSP access.
*)

FMOD_System_GetDSPHead  = function (system: FMOD_SYSTEM; var dsp:FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_System_AddDSP      = function (system: FMOD_SYSTEM; dsp: FMOD_DSP; var connection: FMOD_DSPCONNECTION):FMOD_RESULT; stdcall;
FMOD_System_LockDSP     = function (system: FMOD_SYSTEM):FMOD_RESULT; stdcall;
FMOD_System_UnlockDSP   = function (system: FMOD_SYSTEM):FMOD_RESULT; stdcall;
FMOD_System_GetDSPClock = function (system: FMOD_SYSTEM; var hi: Cardinal; var lo: Cardinal):FMOD_RESULT; stdcall;

(*
     Recording API.
*)

FMOD_System_GetRecordNumDrivers  = function (system: FMOD_SYSTEM; var numdrivers: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetRecordDriverInfo  = function (system: FMOD_SYSTEM; id: Integer; name: PChar; namelen: Integer; guid: FMOD_GUID):FMOD_RESULT; stdcall;
FMOD_System_GetRecordDriverInfoW = function (system: FMOD_SYSTEM; id: Integer; name:array of Smallint; namelen: Integer; guid: FMOD_GUID):FMOD_RESULT; stdcall;
FMOD_System_GetRecordDriverCaps  = function (system: FMOD_SYSTEM; id: Integer; caps: FMOD_CAPS; var minfrequency: Integer; var maxfrequency: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetRecordPosition    = function (system: FMOD_SYSTEM; id: Integer; var position: Cardinal):FMOD_RESULT; stdcall;

FMOD_System_RecordStart = function (system: FMOD_SYSTEM; id: Integer; sound: FMOD_SOUND; loop: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_System_RecordStop  = function (system: FMOD_SYSTEM; id: Integer):FMOD_RESULT; stdcall;
FMOD_System_IsRecording = function (system: FMOD_SYSTEM; id: Integer; recording: FMOD_BOOL):FMOD_RESULT; stdcall;

(*
     Geometry API.
*)

FMOD_System_CreateGeometry       = function (system: FMOD_SYSTEM; maxpolygons: Integer; maxvertices: Integer; var geometry: FMOD_GEOMETRY):FMOD_RESULT; stdcall;
FMOD_System_SetGeometrySettings  = function (system: FMOD_SYSTEM; maxworldsize: Single):FMOD_RESULT; stdcall;
FMOD_System_GetGeometrySettings  = function (system: FMOD_SYSTEM; var maxworldsize: Single):FMOD_RESULT; stdcall;
FMOD_System_LoadGeometry         = function (system: FMOD_SYSTEM; data: Pointer; datasize: Integer; var geometry: FMOD_GEOMETRY):FMOD_RESULT; stdcall;
FMOD_System_GetGeometryOcclusion = function (system: FMOD_SYSTEM; listener: FMOD_VECTOR; source: FMOD_VECTOR; var direct: Single; var reverb: Single):FMOD_RESULT; stdcall;

(*
     Network functions.
*)

FMOD_System_SetNetworkProxy   = function (system: FMOD_SYSTEM; const proxy: PChar):FMOD_RESULT; stdcall;
FMOD_System_GetNetworkProxy   = function (system: FMOD_SYSTEM; proxy: PChar; proxylen: Integer):FMOD_RESULT; stdcall;
FMOD_System_SetNetworkTimeout = function (system: FMOD_SYSTEM; timeout: Integer):FMOD_RESULT; stdcall;
FMOD_System_GetNetworkTimeout = function (system: FMOD_SYSTEM; var timeout: Integer):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_System_SetUserData = function (system: FMOD_SYSTEM; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_System_GetUserData = function (system: FMOD_SYSTEM; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_System_GetMemoryInfo = function (system: FMOD_SYSTEM; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'Sound' API
*)

FMOD_Sound_Release         = function (sound: FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_Sound_GetSystemObject = function (sound: FMOD_SOUND; var system:FMOD_SYSTEM):FMOD_RESULT; stdcall;

(*
     Standard sound manipulation functions.                                                
*)

FMOD_Sound_Lock                = function (sound: FMOD_SOUND; offset: Cardinal; length: Cardinal; ptr1: Pointer; ptr2: Pointer; var len1: Cardinal; var len2: Cardinal):FMOD_RESULT; stdcall;
FMOD_Sound_Unlock              = function (sound: FMOD_SOUND; ptr1: Pointer; ptr2: Pointer; len1: Cardinal; len2: Cardinal):FMOD_RESULT; stdcall;
FMOD_Sound_SetDefaults         = function (sound: FMOD_SOUND; frequency: Single; volume: Single; pan: Single; priority: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetDefaults         = function (sound: FMOD_SOUND; var frequency: Single; var volume: Single; var pan: Single; var priority: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_SetVariations       = function (sound: FMOD_SOUND; frequencyvar: Single; volumevar: Single; panvar: Single):FMOD_RESULT; stdcall;
FMOD_Sound_GetVariations       = function (sound: FMOD_SOUND; var frequencyvar: Single; var volumevar: Single; var panvar: Single):FMOD_RESULT; stdcall;
FMOD_Sound_Set3DMinMaxDistance = function (sound: FMOD_SOUND; min: Single; max: Single):FMOD_RESULT; stdcall;
FMOD_Sound_Get3DMinMaxDistance = function (sound: FMOD_SOUND; var min: Single; var max: Single):FMOD_RESULT; stdcall;
FMOD_Sound_Set3DConeSettings   = function (sound: FMOD_SOUND; insideconeangle: Single; outsideconeangle: Single; outsidevolume: Single):FMOD_RESULT; stdcall;
FMOD_Sound_Get3DConeSettings   = function (sound: FMOD_SOUND; var insideconeangle: Single; var outsideconeangle: Single; var outsidevolume: Single):FMOD_RESULT; stdcall;
FMOD_Sound_Set3DCustomRolloff  = function (sound: FMOD_SOUND; points:FMOD_VECTOR; numpoints: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_Get3DCustomRolloff  = function (sound: FMOD_SOUND; var points:FMOD_VECTOR; var numpoints: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_SetSubSound         = function (sound: FMOD_SOUND; index: Integer; subsound:FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_Sound_GetSubSound         = function (sound: FMOD_SOUND; index: Integer; var subsound:FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_Sound_SetSubSoundSentence = function (sound: FMOD_SOUND; var subsoundlist: Integer; numsubsounds: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetName             = function (sound: FMOD_SOUND; name: PChar; namelen: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetLength           = function (sound: FMOD_SOUND; var length: Cardinal; lengthtype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_Sound_GetFormat           = function (sound: FMOD_SOUND; soundtype: FMOD_SOUND_TYPE; format: FMOD_SOUND_FORMAT; var channels: Integer; var bits: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetNumSubSounds     = function (sound: FMOD_SOUND; var numsubsounds: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetNumTags          = function (sound: FMOD_SOUND; var numtags: Integer; var numtagsupdated: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetTag              = function (sound: FMOD_SOUND; const name: PChar; index: Integer; tag: FMOD_TAG):FMOD_RESULT; stdcall;
FMOD_Sound_GetOpenState        = function (sound: FMOD_SOUND; openstate: FMOD_OPENSTATE; var percentbuffered: Cardinal; starving: FMOD_BOOL; diskbusy: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Sound_ReadData            = function (sound: FMOD_SOUND; buffer: Pointer; lenbytes: Cardinal; var read: Cardinal):FMOD_RESULT; stdcall;
FMOD_Sound_SeekData            = function (sound: FMOD_SOUND; pcm: Cardinal):FMOD_RESULT; stdcall;

FMOD_Sound_SetSoundGroup = function (sound: FMOD_SOUND; soundgroup: FMOD_SOUNDGROUP):FMOD_RESULT; stdcall;
FMOD_Sound_GetSoundGroup = function (sound: FMOD_SOUND; var soundgroup: FMOD_SOUNDGROUP):FMOD_RESULT; stdcall;

(*
     Synchronization point API.  These points can come from markers embedded in wav files, and can also generate channel callbacks.
*)

FMOD_Sound_GetNumSyncPoints = function (sound: FMOD_SOUND; var numsyncpoints: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetSyncPoint     = function (sound: FMOD_SOUND; index: Integer; var point:FMOD_SYNCPOINT):FMOD_RESULT; stdcall;
FMOD_Sound_GetSyncPointInfo = function (sound: FMOD_SOUND; point: FMOD_SYNCPOINT; name: PChar; namelen: Integer; var offset: Cardinal; offsettype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_Sound_AddSyncPoint     = function (sound: FMOD_SOUND; offset: Cardinal; offsettype: FMOD_TIMEUNIT; const name: PChar; var point: FMOD_SYNCPOINT):FMOD_RESULT; stdcall;
FMOD_Sound_DeleteSyncPoint  = function (sound: FMOD_SOUND; point: FMOD_SYNCPOINT):FMOD_RESULT; stdcall;

(*
     Functions also in Channel class but here they are the 'default' to save having to change it in Channel all the time.
*)

FMOD_Sound_SetMode       = function (sound: FMOD_SOUND; mode: FMOD_MODE):FMOD_RESULT; stdcall;
FMOD_Sound_GetMode       = function (sound: FMOD_SOUND; var mode: FMOD_MODE):FMOD_RESULT; stdcall;
FMOD_Sound_SetLoopCount  = function (sound: FMOD_SOUND; loopcount: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_GetLoopCount  = function (sound: FMOD_SOUND; var loopcount: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_SetLoopPoints = function (sound: FMOD_SOUND; loopstart: Cardinal; loopstarttype: FMOD_TIMEUNIT; loopend: Cardinal; loopendtype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_Sound_GetLoopPoints = function (sound: FMOD_SOUND; var loopstart: Cardinal; loopstarttype: FMOD_TIMEUNIT; var loopend: Cardinal; loopendtype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;

(*
     For MOD/S3M/XM/IT/MID sequenced formats only.
*)

FMOD_Sound_GetMusicNumChannels   = function (sound: FMOD_SOUND; var numchannels: Integer):FMOD_RESULT; stdcall;
FMOD_Sound_SetMusicChannelVolume = function (sound: FMOD_SOUND; channel: Integer; volume: Single):FMOD_RESULT; stdcall;
FMOD_Sound_GetMusicChannelVolume = function (sound: FMOD_SOUND; channel: Integer; var volume: Single):FMOD_RESULT; stdcall;
FMOD_Sound_SetMusicSpeed         = function (sound: FMOD_SOUND; speed: Single):FMOD_RESULT; stdcall;
FMOD_Sound_GetMusicSpeed         = function (sound: FMOD_SOUND; var speed: Single):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_Sound_SetUserData = function (sound: FMOD_SOUND; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_Sound_GetUserData = function (sound: FMOD_SOUND; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_Sound_GetMemoryInfo = function (sound: FMOD_SOUND; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'Channel' API
*)

FMOD_Channel_GetSystemObject = function (channel: FMOD_CHANNEL; var system: FMOD_SYSTEM):FMOD_RESULT; stdcall;

FMOD_Channel_Stop                = function (channel: FMOD_CHANNEL):FMOD_RESULT; stdcall;
FMOD_Channel_SetPaused           = function (channel: FMOD_CHANNEL; paused: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Channel_GetPaused           = function (channel: FMOD_CHANNEL; var paused: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Channel_SetVolume           = function (channel: FMOD_CHANNEL; volume: Single):FMOD_RESULT; stdcall;
FMOD_Channel_GetVolume           = function (channel: FMOD_CHANNEL; var volume: Single):FMOD_RESULT; stdcall;
FMOD_Channel_SetFrequency        = function (channel: FMOD_CHANNEL; frequency: Single):FMOD_RESULT; stdcall;
FMOD_Channel_GetFrequency        = function (channel: FMOD_CHANNEL; var frequency: Single):FMOD_RESULT; stdcall;
FMOD_Channel_SetPan              = function (channel: FMOD_CHANNEL; pan: Single):FMOD_RESULT; stdcall;
FMOD_Channel_GetPan              = function (channel: FMOD_CHANNEL; var pan: Single):FMOD_RESULT; stdcall;
FMOD_Channel_SetDelay            = function (channel: FMOD_CHANNEL; delaytype: FMOD_DELAYTYPE; delayhi: Cardinal; delaylo: Cardinal):FMOD_RESULT; stdcall;
FMOD_Channel_GetDelay            = function (channel: FMOD_CHANNEL; delaytype: FMOD_DELAYTYPE; var delayhi: Cardinal; var delaylo: Cardinal):FMOD_RESULT; stdcall;
FMOD_Channel_SetSpeakerMix       = function (channel: FMOD_CHANNEL; frontleft: Single; frontright: Single; center: Single; lfe: Single; backleft: Single; backright: Single; sideleft: Single; sideright: Single):FMOD_RESULT; stdcall;
FMOD_Channel_GetSpeakerMix       = function (channel: FMOD_CHANNEL; var frontleft: Single; var frontright: Single; var center: Single; var lfe: Single; var backleft: Single; var backright: Single; var sideleft: Single; var sideright: Single):FMOD_RESULT; stdcall;
FMOD_Channel_SetSpeakerLevels    = function (channel: FMOD_CHANNEL; speaker: FMOD_SPEAKER; var levels: Single; numlevels: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_GetSpeakerLevels    = function (channel: FMOD_CHANNEL; speaker: FMOD_SPEAKER; var levels: Single; numlevels: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_SetInputChannelMix  = function (channel: FMOD_CHANNEL; var levels: Single; numlevels: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_GetInputChannelMix  = function (channel: FMOD_CHANNEL; var levels: Single; numlevels: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_SetMute             = function (channel: FMOD_CHANNEL; mute: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Channel_GetMute             = function (channel: FMOD_CHANNEL; var mute: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Channel_SetPriority         = function (channel: FMOD_CHANNEL; priority: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_GetPriority         = function (channel: FMOD_CHANNEL; var priority: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_SetPosition         = function (channel: FMOD_CHANNEL; position: Cardinal; postype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_Channel_GetPosition         = function (channel: FMOD_CHANNEL; var position: Cardinal; postype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_Channel_SetReverbProperties = function (channel: FMOD_CHANNEL; prop: FMOD_REVERB_CHANNELPROPERTIES):FMOD_RESULT; stdcall;
FMOD_Channel_GetReverbProperties = function (channel: FMOD_CHANNEL; prop: FMOD_REVERB_CHANNELPROPERTIES):FMOD_RESULT; stdcall;
FMOD_Channel_SetLowPassGain      = function (channel: FMOD_CHANNEL; gain: Single):FMOD_RESULT; stdcall;
FMOD_Channel_GetLowPassGain      = function (channel: FMOD_CHANNEL; var gain: Single):FMOD_RESULT; stdcall;

FMOD_Channel_SetChannelGroup = function (channel: FMOD_CHANNEL; channelgroup: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_Channel_GetChannelGroup = function (channel: FMOD_CHANNEL; var channelgroup: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_Channel_SetCallback     = function (channel: FMOD_CHANNEL; callback: FMOD_CHANNEL_CALLBACK):FMOD_RESULT; stdcall;

(*
     3D functionality.
*)

FMOD_Channel_Set3DAttributes      = function (channel: FMOD_CHANNEL; pos: FMOD_VECTOR; vel: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DAttributes      = function (channel: FMOD_CHANNEL; pos: FMOD_VECTOR; vel: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DMinMaxDistance  = function (channel: FMOD_CHANNEL; mindistance: Single; maxdistance: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DMinMaxDistance  = function (channel: FMOD_CHANNEL; var mindistance: Single; var maxdistance: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DConeSettings    = function (channel: FMOD_CHANNEL; insideconeangle: Single; outsideconeangle: Single; outsidevolume: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DConeSettings    = function (channel: FMOD_CHANNEL; var insideconeangle: Single; var outsideconeangle: Single; var outsidevolume: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DConeOrientation = function (channel: FMOD_CHANNEL; orientation: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DConeOrientation = function (channel: FMOD_CHANNEL; orientation: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DCustomRolloff   = function (channel: FMOD_CHANNEL; points: FMOD_VECTOR; numpoints: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DCustomRolloff   = function (channel: FMOD_CHANNEL; var points: FMOD_VECTOR; var numpoints: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DOcclusion       = function (channel: FMOD_CHANNEL; directocclusion: Single; reverbocclusion: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DOcclusion       = function (channel: FMOD_CHANNEL; var directocclusion: Single; var reverbocclusion: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DSpread          = function (channel: FMOD_CHANNEL; angle: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DSpread          = function (channel: FMOD_CHANNEL; var angle: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DPanLevel        = function (channel: FMOD_CHANNEL; level: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DPanLevel        = function (channel: FMOD_CHANNEL; var level: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Set3DDopplerLevel    = function (channel: FMOD_CHANNEL; level: Single):FMOD_RESULT; stdcall;
FMOD_Channel_Get3DDopplerLevel    = function (channel: FMOD_CHANNEL; var level: Single):FMOD_RESULT; stdcall;

(*
     DSP functionality only for channels playing sounds created with FMOD_SOFTWARE.
*)

FMOD_Channel_GetDSPHead = function (channel: FMOD_CHANNEL; var dsp: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_Channel_AddDSP     = function (channel: FMOD_CHANNEL; dsp: FMOD_DSP; var connection: FMOD_DSPCONNECTION):FMOD_RESULT; stdcall;

(*
     Information only functions.
*)

FMOD_Channel_IsPlaying       = function (channel: FMOD_CHANNEL; var isplaying: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Channel_IsVirtual       = function (channel: FMOD_CHANNEL; var isvirtual: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Channel_GetAudibility   = function (channel: FMOD_CHANNEL; var audibility: Single):FMOD_RESULT; stdcall;
FMOD_Channel_GetCurrentSound = function (channel: FMOD_CHANNEL; var sound: FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_Channel_GetSpectrum     = function (channel: FMOD_CHANNEL; var spectrumarray: Single; numvalues: Integer; channeloffset: Integer; windowtype: FMOD_DSP_FFT_WINDOW):FMOD_RESULT; stdcall;
FMOD_Channel_GetWaveData     = function (channel: FMOD_CHANNEL; var wavearray: Single; numvalues: Integer; channeloffset: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_GetIndex        = function (channel: FMOD_CHANNEL; var index: Integer):FMOD_RESULT; stdcall;

(*
     Functions also found in Sound class but here they can be set per channel.
*)

FMOD_Channel_SetMode       = function (channel: FMOD_CHANNEL; mode: FMOD_MODE):FMOD_RESULT; stdcall;
FMOD_Channel_GetMode       = function (channel: FMOD_CHANNEL; var mode: FMOD_MODE):FMOD_RESULT; stdcall;
FMOD_Channel_SetLoopCount  = function (channel: FMOD_CHANNEL; loopcount: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_GetLoopCount  = function (channel: FMOD_CHANNEL; var loopcount: Integer):FMOD_RESULT; stdcall;
FMOD_Channel_SetLoopPoints = function (channel: FMOD_CHANNEL; loopstart: Cardinal; loopstarttype: FMOD_TIMEUNIT; loopend: Cardinal; loopendtype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;
FMOD_Channel_GetLoopPoints = function (channel: FMOD_CHANNEL; var loopstart: Cardinal; loopstarttype: FMOD_TIMEUNIT; var loopend: Cardinal; loopendtype: FMOD_TIMEUNIT):FMOD_RESULT; stdcall;

(*
     Userdata set/get.                                                
*)

FMOD_Channel_SetUserData = function (channel: FMOD_CHANNEL; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_Channel_GetUserData = function (channel: FMOD_CHANNEL; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_Channel_GetMemoryInfo = function (channel: FMOD_CHANNEL; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'ChannelGroup' API
*)

FMOD_ChannelGroup_Release = function (channelgroup: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetSystemObject = function (channelgroup: FMOD_CHANNELGROUP; var system: FMOD_SYSTEM):FMOD_RESULT; stdcall;

(*
     Channelgroup scale values.  (changes attributes relative to the channels, doesn't overwrite them)
*)

FMOD_ChannelGroup_SetVolume      = function (channelgroup: FMOD_CHANNELGROUP; volume: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetVolume      = function (channelgroup: FMOD_CHANNELGROUP; var volume: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_SetPitch       = function (channelgroup: FMOD_CHANNELGROUP; pitch: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetPitch       = function (channelgroup: FMOD_CHANNELGROUP; var pitch: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_Set3DOcclusion = function (channelgroup: FMOD_CHANNELGROUP; directocclusion: Single; reverbocclusion: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_Get3DOcclusion = function (channelgroup: FMOD_CHANNELGROUP; var directocclusion: Single; var reverbocclusion: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_SetPaused      = function (channelgroup: FMOD_CHANNELGROUP; paused: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetPaused      = function (channelgroup: FMOD_CHANNELGROUP; var paused: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_SetMute        = function (channelgroup: FMOD_CHANNELGROUP; mute: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetMute        = function (channelgroup: FMOD_CHANNELGROUP; var mute: FMOD_BOOL):FMOD_RESULT; stdcall;

(*
     Channelgroup override values.  (recursively overwrites whatever settings the channels had)
*)

FMOD_ChannelGroup_Stop                     = function (channelgroup: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_OverrideVolume           = function (channelgroup: FMOD_CHANNELGROUP; volume: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_OverrideFrequency        = function (channelgroup: FMOD_CHANNELGROUP; frequency: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_OverridePan              = function (channelgroup: FMOD_CHANNELGROUP; pan: Single):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_OverrideReverbProperties = function (channelgroup: FMOD_CHANNELGROUP; prop: FMOD_REVERB_CHANNELPROPERTIES):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_Override3DAttributes     = function (channelgroup: FMOD_CHANNELGROUP; pos: FMOD_VECTOR; vel: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_OverrideSpeakerMix       = function (channelgroup: FMOD_CHANNELGROUP; frontleft: Single; frontright: Single; center: Single; lfe: Single; backleft: Single; backright: Single; sideleft: Single; sideright: Single):FMOD_RESULT; stdcall;

(*
     Nested channel groups.
*)

FMOD_ChannelGroup_AddGroup       = function (channelgroup: FMOD_CHANNELGROUP; group: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetNumGroups   = function (channelgroup: FMOD_CHANNELGROUP; var numgroups: Integer):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetGroup       = function (channelgroup: FMOD_CHANNELGROUP; index: Integer; var group: FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetParentGroup = function (channelgroup: FMOD_CHANNELGROUP; var group:FMOD_CHANNELGROUP):FMOD_RESULT; stdcall;

(*
     DSP functionality only for channel groups playing sounds created with FMOD_SOFTWARE.
*)

FMOD_ChannelGroup_GetDSPHead = function (channelgroup: FMOD_CHANNELGROUP; var dsp: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_AddDSP     = function (channelgroup: FMOD_CHANNELGROUP; dsp: FMOD_DSP; var connection: FMOD_DSPCONNECTION):FMOD_RESULT; stdcall;

(*
     Information only functions.
*)

FMOD_ChannelGroup_GetName        = function (channelgroup: FMOD_CHANNELGROUP; name: PChar; namelen: Integer):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetNumChannels = function (channelgroup: FMOD_CHANNELGROUP; var numchannels: Integer):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetChannel     = function (channelgroup: FMOD_CHANNELGROUP; index: Integer; var channel: FMOD_CHANNEL):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetSpectrum    = function (channelgroup: FMOD_CHANNELGROUP; var spectrumarray: Single; numvalues: Integer; channeloffset: Integer; windowtype: FMOD_DSP_FFT_WINDOW):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetWaveData    = function (channelgroup: FMOD_CHANNELGROUP; var wavearray: Single; numvalues: Integer; channeloffset: Integer):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_ChannelGroup_SetUserData = function (channelgroup: FMOD_CHANNELGROUP; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_ChannelGroup_GetUserData = function (channelgroup: FMOD_CHANNELGROUP; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_ChannelGroup_GetMemoryInfo = function (channelgroup: FMOD_CHANNELGROUP; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'SoundGroup' API
*)

FMOD_SoundGroup_Release = function (soundgroup: FMOD_SOUNDGROUP):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetSystemObject = function (soundgroup: FMOD_SOUNDGROUP; var system: FMOD_SYSTEM):FMOD_RESULT; stdcall;

(*
     SoundGroup control functions.
*)

FMOD_SoundGroup_SetMaxAudible         = function (soundgroup: FMOD_SOUNDGROUP; maxaudible: Integer):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetMaxAudible         = function (soundgroup: FMOD_SOUNDGROUP; var maxaudible: Integer):FMOD_RESULT; stdcall;
FMOD_SoundGroup_SetMaxAudibleBehavior = function (soundgroup: FMOD_SOUNDGROUP; behavior: FMOD_SOUNDGROUP_BEHAVIOR):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetMaxAudibleBehavior = function (soundgroup: FMOD_SOUNDGROUP; var behavior: FMOD_SOUNDGROUP_BEHAVIOR):FMOD_RESULT; stdcall;
FMOD_SoundGroup_SetMuteFadeSpeed      = function (soundgroup: FMOD_SOUNDGROUP; speed: Single):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetMuteFadeSpeed      = function (soundgroup: FMOD_SOUNDGROUP; var speed: Single):FMOD_RESULT; stdcall;
FMOD_SoundGroup_SetVolume             = function (soundgroup: FMOD_SOUNDGROUP; volume: Single):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetVolume             = function (soundgroup: FMOD_SOUNDGROUP; var volume: Single):FMOD_RESULT; stdcall;
FMOD_SoundGroup_Stop                  = function (soundgroup: FMOD_SOUNDGROUP):FMOD_RESULT; stdcall;

(*
     Information only functions.
*)

FMOD_SoundGroup_GetName       = function (soundgroup: FMOD_SOUNDGROUP; name: PChar; namelen: Integer):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetNumSounds  = function (soundgroup: FMOD_SOUNDGROUP; var numsounds: Integer):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetSound      = function (soundgroup: FMOD_SOUNDGROUP; index: Integer; var sound: FMOD_SOUND):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetNumPlaying = function (soundgroup: FMOD_SOUNDGROUP; var numplaying: Integer):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_SoundGroup_SetUserData = function (soundgroup: FMOD_SOUNDGROUP; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_SoundGroup_GetUserData = function (soundgroup: FMOD_SOUNDGROUP; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_SoundGroup_GetMemoryInfo = function (soundgroup: FMOD_SOUNDGROUP; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'DSP' API
*)

FMOD_DSP_Release         = function (dsp: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_DSP_GetSystemObject = function (dsp: FMOD_DSP; var system: FMOD_SYSTEM):FMOD_RESULT; stdcall;

(*
     Connection / disconnection / input and output enumeration.
*)

FMOD_DSP_AddInput       = function (dsp: FMOD_DSP; target: FMOD_DSP; var connection:FMOD_DSPCONNECTION):FMOD_RESULT; stdcall;
FMOD_DSP_DisconnectFrom = function (dsp: FMOD_DSP; target: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_DSP_DisconnectAll  = function (dsp: FMOD_DSP; inputs: FMOD_BOOL; outputs: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_Remove         = function (dsp: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_DSP_GetNumInputs   = function (dsp: FMOD_DSP; var numinputs: Integer):FMOD_RESULT; stdcall;
FMOD_DSP_GetNumOutputs  = function (dsp: FMOD_DSP; var numoutputs: Integer):FMOD_RESULT; stdcall;
FMOD_DSP_GetInput       = function (dsp: FMOD_DSP; index: Integer; var input: FMOD_DSP; var inputconnection: FMOD_DSPCONNECTION):FMOD_RESULT; stdcall;
FMOD_DSP_GetOutput      = function (dsp: FMOD_DSP; index: Integer; var output: FMOD_DSP; var outputconnection: FMOD_DSPCONNECTION):FMOD_RESULT; stdcall;

(*
     DSP unit control.
*)

FMOD_DSP_SetActive        = function (dsp: FMOD_DSP; active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_GetActive        = function (dsp: FMOD_DSP; var active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_SetBypass        = function (dsp: FMOD_DSP; bypass: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_GetBypass        = function (dsp: FMOD_DSP; var bypass: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_SetSpeakerActive = function (dsp: FMOD_DSP; speaker: FMOD_SPEAKER; active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_GetSpeakerActive = function (dsp: FMOD_DSP; speaker: FMOD_SPEAKER; var active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_DSP_Reset            = function (dsp: FMOD_DSP):FMOD_RESULT; stdcall;

(*
     DSP parameter control.
*)

FMOD_DSP_SetParameter     = function (dsp: FMOD_DSP; index: Integer; value: Single):FMOD_RESULT; stdcall;
FMOD_DSP_GetParameter     = function (dsp: FMOD_DSP; index: Integer; var value: Single; valuestr: PChar; valuestrlen: Integer):FMOD_RESULT; stdcall;
FMOD_DSP_GetNumParameters = function (dsp: FMOD_DSP; var numparams: Integer):FMOD_RESULT; stdcall;
FMOD_DSP_GetParameterInfo = function (dsp: FMOD_DSP; index: Integer; name: PChar; plabel: PChar; description: PChar; descriptionlen: Integer; var min: Single; var max: Single):FMOD_RESULT; stdcall;
FMOD_DSP_ShowConfigDialog = function (dsp: FMOD_DSP; hwnd: Pointer; show: FMOD_BOOL):FMOD_RESULT; stdcall;

(*
     DSP attributes.        
*)

FMOD_DSP_GetInfo     = function (dsp: FMOD_DSP; name: PChar; var version: Cardinal; var channels: Integer; var configwidth: Integer; var configheight: Integer):FMOD_RESULT; stdcall;
FMOD_DSP_GetType     = function (dsp: FMOD_DSP; dsptype: FMOD_DSP_TYPE):FMOD_RESULT; stdcall;
FMOD_DSP_SetDefaults = function (dsp: FMOD_DSP; frequency: Single; volume: Single; pan: Single; priority: Integer):FMOD_RESULT; stdcall;
FMOD_DSP_GetDefaults = function (dsp: FMOD_DSP; var frequency: Single; var volume: Single; var pan: Single; var priority: Integer):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_DSP_SetUserData = function (dsp: FMOD_DSP; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_DSP_GetUserData = function (dsp: FMOD_DSP; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_DSP_GetMemoryInfo = function (dsp: FMOD_DSP; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details:FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'DSPConnection' API
*)

FMOD_DSPConnection_GetInput  = function (dspconnection: FMOD_DSPCONNECTION; var input: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_DSPConnection_GetOutput = function (dspconnection: FMOD_DSPCONNECTION; var output: FMOD_DSP):FMOD_RESULT; stdcall;
FMOD_DSPConnection_SetMix    = function (dspconnection: FMOD_DSPCONNECTION; volume: Single):FMOD_RESULT; stdcall;
FMOD_DSPConnection_GetMix    = function (dspconnection: FMOD_DSPCONNECTION; var volume: Single):FMOD_RESULT; stdcall;
FMOD_DSPConnection_SetLevels = function (dspconnection: FMOD_DSPCONNECTION; speaker: FMOD_SPEAKER; var levels: Single; numlevels: Integer):FMOD_RESULT; stdcall;
FMOD_DSPConnection_GetLevels = function (dspconnection: FMOD_DSPCONNECTION; speaker: FMOD_SPEAKER; var levels: Single; numlevels: Integer):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_DSPConnection_SetUserData = function (dspconnection: FMOD_DSPCONNECTION; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_DSPConnection_GetUserData = function (dspconnection: FMOD_DSPCONNECTION; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_DSPConnection_GetMemoryInfo = function (dspconnection: FMOD_DSPCONNECTION; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details:FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'Geometry' API
*)

FMOD_Geometry_Release = function (geometry: FMOD_GEOMETRY):FMOD_RESULT; stdcall;

(*
     Polygon manipulation.
*)

FMOD_Geometry_AddPolygon            = function (geometry: FMOD_GEOMETRY; directocclusion: Single; reverbocclusion: Single; doublesided: FMOD_BOOL; numvertices: Integer; vertices: FMOD_VECTOR; var polygonindex: Integer):FMOD_RESULT; stdcall;
FMOD_Geometry_GetNumPolygons        = function (geometry: FMOD_GEOMETRY; var numpolygons: Integer):FMOD_RESULT; stdcall;
FMOD_Geometry_GetMaxPolygons        = function (geometry: FMOD_GEOMETRY; var maxpolygons: Integer; var maxvertices: Integer):FMOD_RESULT; stdcall;
FMOD_Geometry_GetPolygonNumVertices = function (geometry: FMOD_GEOMETRY; index: Integer; var numvertices: Integer):FMOD_RESULT; stdcall;
FMOD_Geometry_SetPolygonVertex      = function (geometry: FMOD_GEOMETRY; index: Integer; vertexindex: Integer; vertex: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_GetPolygonVertex      = function (geometry: FMOD_GEOMETRY; index: Integer; vertexindex: Integer; var vertex: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_SetPolygonAttributes  = function (geometry: FMOD_GEOMETRY; index: Integer; directocclusion: Single; reverbocclusion: Single; doublesided: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Geometry_GetPolygonAttributes  = function (geometry: FMOD_GEOMETRY; index: Integer; var directocclusion: Single; var reverbocclusion: Single; var doublesided: FMOD_BOOL):FMOD_RESULT; stdcall;

(*
     Object manipulation.
*)

FMOD_Geometry_SetActive   = function (geometry: FMOD_GEOMETRY; active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Geometry_GetActive   = function (geometry: FMOD_GEOMETRY; var active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Geometry_SetRotation = function (geometry: FMOD_GEOMETRY; forward: FMOD_VECTOR; up: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_GetRotation = function (geometry: FMOD_GEOMETRY; var forward: FMOD_VECTOR; var up: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_SetPosition = function (geometry: FMOD_GEOMETRY; position: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_GetPosition = function (geometry: FMOD_GEOMETRY; var position: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_SetScale    = function (geometry: FMOD_GEOMETRY; scale: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_GetScale    = function (geometry: FMOD_GEOMETRY; var scale: FMOD_VECTOR):FMOD_RESULT; stdcall;
FMOD_Geometry_Save        = function (geometry: FMOD_GEOMETRY; data: Pointer; var datasize: Integer):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_Geometry_SetUserData = function (geometry: FMOD_GEOMETRY; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_Geometry_GetUserData = function (geometry: FMOD_GEOMETRY; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_Geometry_GetMemoryInfo = function (geometry: FMOD_GEOMETRY; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

(*
    'Reverb' API
*)

FMOD_Reverb_Release = function (reverb: FMOD_REVERB):FMOD_RESULT; stdcall;

(*
     Reverb manipulation.
*)

FMOD_Reverb_Set3DAttributes = function (reverb: FMOD_REVERB; position: FMOD_VECTOR; mindistance: Single; maxdistance: Single):FMOD_RESULT; stdcall;
FMOD_Reverb_Get3DAttributes = function (reverb: FMOD_REVERB; var position: FMOD_VECTOR; var mindistance: Single; var maxdistance: Single):FMOD_RESULT; stdcall;
FMOD_Reverb_SetProperties   = function (reverb: FMOD_REVERB; properties: FMOD_REVERB_PROPERTIES):FMOD_RESULT; stdcall;
FMOD_Reverb_GetProperties   = function (reverb: FMOD_REVERB; properties: FMOD_REVERB_PROPERTIES):FMOD_RESULT; stdcall;
FMOD_Reverb_SetActive       = function (reverb: FMOD_REVERB; active: FMOD_BOOL):FMOD_RESULT; stdcall;
FMOD_Reverb_GetActive       = function (reverb: FMOD_REVERB; var active: FMOD_BOOL):FMOD_RESULT; stdcall;

(*
     Userdata set/get.
*)

FMOD_Reverb_SetUserData = function (reverb: FMOD_REVERB; userdata: Pointer):FMOD_RESULT; stdcall;
FMOD_Reverb_GetUserData = function (reverb: FMOD_REVERB; userdata: Pointer):FMOD_RESULT; stdcall;

FMOD_Reverb_GetMemoryInfo = function (reverb: FMOD_REVERB; memorybits: Cardinal; event_memorybits: Cardinal; var memoryused: Cardinal; var memoryused_details: FMOD_MEMORY_USAGE_DETAILS):FMOD_RESULT; stdcall;

var
  FcFMOD_Memory_Initialize : FMOD_Memory_Initialize;
  FcFMOD_Memory_GetStats : FMOD_Memory_GetStats;
  FcFMOD_Debug_SetLevel : FMOD_Debug_SetLevel;
  FcFMOD_Debug_GetLevel : FMOD_Debug_GetLevel;
  FcFMOD_File_SetDiskBusy : FMOD_File_SetDiskBusy;
  FcFMOD_File_GetDiskBusy : FMOD_File_GetDiskBusy;
  FcFMOD_System_Create : FMOD_System_Create;
  FcFMOD_System_Release : FMOD_System_Release;
  FcFMOD_System_SetOutput : FMOD_System_SetOutput;
  FcFMOD_System_GetOutput : FMOD_System_GetOutput;
  FcFMOD_System_GetNumDrivers : FMOD_System_GetNumDrivers;
  FcFMOD_System_GetDriverInfo : FMOD_System_GetDriverInfo;
  FcFMOD_System_GetDriverInfoW : FMOD_System_GetDriverInfoW;
  FcFMOD_System_GetDriverCaps : FMOD_System_GetDriverCaps;
  FcFMOD_System_SetDriver : FMOD_System_SetDriver;
  FcFMOD_System_GetDriver : FMOD_System_GetDriver;
  FcFMOD_System_SetHardwareChannels : FMOD_System_SetHardwareChannels;
  FcFMOD_System_SetSoftwareChannels : FMOD_System_SetSoftwareChannels;
  FcFMOD_System_GetSoftwareChannels : FMOD_System_GetSoftwareChannels;
  FcFMOD_System_SetSoftwareFormat : FMOD_System_SetSoftwareFormat;
  FcFMOD_System_GetSoftwareFormat : FMOD_System_GetSoftwareFormat;
  FcFMOD_System_SetDSPBufferSize : FMOD_System_SetDSPBufferSize;
  FcFMOD_System_GetDSPBufferSize : FMOD_System_GetDSPBufferSize;
  FcFMOD_System_SetFileSystem : FMOD_System_SetFileSystem;
  FcFMOD_System_AttachFileSystem : FMOD_System_AttachFileSystem;
  FcFMOD_System_SetAdvancedSettings : FMOD_System_SetAdvancedSettings;
  FcFMOD_System_GetAdvancedSettings : FMOD_System_GetAdvancedSettings;
  FcFMOD_System_SetSpeakerMode : FMOD_System_SetSpeakerMode;
  FcFMOD_System_GetSpeakerMode : FMOD_System_GetSpeakerMode;
  FcFMOD_System_SetCallback : FMOD_System_SetCallback;
  FcFMOD_System_SetPluginPath : FMOD_System_SetPluginPath;
  FcFMOD_System_LoadPlugin : FMOD_System_LoadPlugin;
  FcFMOD_System_UnloadPlugin : FMOD_System_UnloadPlugin;
  FcFMOD_System_GetNumPlugins : FMOD_System_GetNumPlugins;
  FcFMOD_System_GetPluginHandle : FMOD_System_GetPluginHandle;
  FcFMOD_System_GetPluginInfo : FMOD_System_GetPluginInfo;
  FcFMOD_System_SetOutputByPlugin : FMOD_System_SetOutputByPlugin;
  FcFMOD_System_GetOutputByPlugin : FMOD_System_GetOutputByPlugin;
  FcFMOD_System_CreateDSPByPlugin : FMOD_System_CreateDSPByPlugin;
  FcFMOD_System_CreateCodec : FMOD_System_CreateCodec;
  FcFMOD_System_Init : FMOD_System_Init;
  FcFMOD_System_Close : FMOD_System_Close;
  FcFMOD_System_Update : FMOD_System_Update;
  FcFMOD_System_Set3DSettings : FMOD_System_Set3DSettings;
  FcFMOD_System_Get3DSettings : FMOD_System_Get3DSettings;
  FcFMOD_System_Set3DNumListeners : FMOD_System_Set3DNumListeners;
  FcFMOD_System_Get3DNumListeners : FMOD_System_Get3DNumListeners;
  FcFMOD_System_Set3DListenerAttributes : FMOD_System_Set3DListenerAttributes;
  FcFMOD_System_Get3DListenerAttributes : FMOD_System_Get3DListenerAttributes;
  FcFMOD_System_Set3DRolloffCallback : FMOD_System_Set3DRolloffCallback;
  FcFMOD_System_Set3DSpeakerPosition : FMOD_System_Set3DSpeakerPosition;
  FcFMOD_System_Get3DSpeakerPosition : FMOD_System_Get3DSpeakerPosition;
  FcFMOD_System_SetStreamBufferSize : FMOD_System_SetStreamBufferSize;
  FcFMOD_System_GetStreamBufferSize : FMOD_System_GetStreamBufferSize;
  FcFMOD_System_GetVersion : FMOD_System_GetVersion;
  FcFMOD_System_GetOutputHandle : FMOD_System_GetOutputHandle;
  FcFMOD_System_GetChannelsPlaying : FMOD_System_GetChannelsPlaying;
  FcFMOD_System_GetHardwareChannels : FMOD_System_GetHardwareChannels;
  FcFMOD_System_GetCPUUsage : FMOD_System_GetCPUUsage;
  FcFMOD_System_GetSoundRAM : FMOD_System_GetSoundRAM;
  FcFMOD_System_GetNumCDROMDrives : FMOD_System_GetNumCDROMDrives;
  FcFMOD_System_GetCDROMDriveName : FMOD_System_GetCDROMDriveName;
  FcFMOD_System_GetSpectrum : FMOD_System_GetSpectrum;
  FcFMOD_System_GetWaveData : FMOD_System_GetWaveData;
  FcFMOD_System_CreateSound : FMOD_System_CreateSound;
  FcFMOD_System_CreateStream : FMOD_System_CreateStream;
  FcFMOD_System_CreateDSP : FMOD_System_CreateDSP;
  FcFMOD_System_CreateDSPByType : FMOD_System_CreateDSPByType;
  FcFMOD_System_CreateChannelGroup : FMOD_System_CreateChannelGroup;
  FcFMOD_System_CreateSoundGroup : FMOD_System_CreateSoundGroup;
  FcFMOD_System_CreateReverb : FMOD_System_CreateReverb;
  FcFMOD_System_PlaySound : FMOD_System_PlaySound;
  FcFMOD_System_PlayDSP : FMOD_System_PlayDSP;
  FcFMOD_System_GetChannel : FMOD_System_GetChannel;
  FcFMOD_System_GetMasterChannelGroup : FMOD_System_GetMasterChannelGroup;
  FcFMOD_System_GetMasterSoundGroup : FMOD_System_GetMasterSoundGroup;
  FcFMOD_System_SetReverbProperties : FMOD_System_SetReverbProperties;
  FcFMOD_System_GetReverbProperties : FMOD_System_GetReverbProperties;
  FcFMOD_System_SetReverbAmbientProperties : FMOD_System_SetReverbAmbientProperties;
  FcFMOD_System_GetReverbAmbientProperties : FMOD_System_GetReverbAmbientProperties;
  FcFMOD_System_GetDSPHead : FMOD_System_GetDSPHead;
  FcFMOD_System_AddDSP : FMOD_System_AddDSP;
  FcFMOD_System_LockDSP : FMOD_System_LockDSP;
  FcFMOD_System_UnlockDSP : FMOD_System_UnlockDSP;
  FcFMOD_System_GetDSPClock : FMOD_System_GetDSPClock;
  FcFMOD_System_GetRecordNumDrivers : FMOD_System_GetRecordNumDrivers;
  FcFMOD_System_GetRecordDriverInfo : FMOD_System_GetRecordDriverInfo;
  FcFMOD_System_GetRecordDriverInfoW : FMOD_System_GetRecordDriverInfoW;
  FcFMOD_System_GetRecordDriverCaps : FMOD_System_GetRecordDriverCaps;
  FcFMOD_System_GetRecordPosition : FMOD_System_GetRecordPosition;
  FcFMOD_System_RecordStart : FMOD_System_RecordStart;
  FcFMOD_System_RecordStop : FMOD_System_RecordStop;
  FcFMOD_System_IsRecording : FMOD_System_IsRecording;
  FcFMOD_System_CreateGeometry : FMOD_System_CreateGeometry;
  FcFMOD_System_SetGeometrySettings : FMOD_System_SetGeometrySettings;
  FcFMOD_System_GetGeometrySettings : FMOD_System_GetGeometrySettings;
  FcFMOD_System_LoadGeometry : FMOD_System_LoadGeometry;
  FcFMOD_System_GetGeometryOcclusion : FMOD_System_GetGeometryOcclusion;
  FcFMOD_System_SetNetworkProxy : FMOD_System_SetNetworkProxy;
  FcFMOD_System_GetNetworkProxy : FMOD_System_GetNetworkProxy;
  FcFMOD_System_SetNetworkTimeout : FMOD_System_SetNetworkTimeout;
  FcFMOD_System_GetNetworkTimeout : FMOD_System_GetNetworkTimeout;
  FcFMOD_System_SetUserData : FMOD_System_SetUserData;
  FcFMOD_System_GetUserData : FMOD_System_GetUserData;
  FcFMOD_System_GetMemoryInfo : FMOD_System_GetMemoryInfo;
  FcFMOD_Sound_Release : FMOD_Sound_Release;
  FcFMOD_Sound_GetSystemObject : FMOD_Sound_GetSystemObject;
  FcFMOD_Sound_Lock : FMOD_Sound_Lock;
  FcFMOD_Sound_Unlock : FMOD_Sound_Unlock;
  FcFMOD_Sound_SetDefaults : FMOD_Sound_SetDefaults;
  FcFMOD_Sound_GetDefaults : FMOD_Sound_GetDefaults;
  FcFMOD_Sound_SetVariations : FMOD_Sound_SetVariations;
  FcFMOD_Sound_GetVariations : FMOD_Sound_GetVariations;
  FcFMOD_Sound_Set3DMinMaxDistance : FMOD_Sound_Set3DMinMaxDistance;
  FcFMOD_Sound_Get3DMinMaxDistance : FMOD_Sound_Get3DMinMaxDistance;
  FcFMOD_Sound_Set3DConeSettings : FMOD_Sound_Set3DConeSettings;
  FcFMOD_Sound_Get3DConeSettings : FMOD_Sound_Get3DConeSettings;
  FcFMOD_Sound_Set3DCustomRolloff : FMOD_Sound_Set3DCustomRolloff;
  FcFMOD_Sound_Get3DCustomRolloff : FMOD_Sound_Get3DCustomRolloff;
  FcFMOD_Sound_SetSubSound : FMOD_Sound_SetSubSound;
  FcFMOD_Sound_GetSubSound : FMOD_Sound_GetSubSound;
  FcFMOD_Sound_SetSubSoundSentence : FMOD_Sound_SetSubSoundSentence;
  FcFMOD_Sound_GetName : FMOD_Sound_GetName;
  FcFMOD_Sound_GetLength : FMOD_Sound_GetLength;
  FcFMOD_Sound_GetFormat : FMOD_Sound_GetFormat;
  FcFMOD_Sound_GetNumSubSounds : FMOD_Sound_GetNumSubSounds;
  FcFMOD_Sound_GetNumTags : FMOD_Sound_GetNumTags;
  FcFMOD_Sound_GetTag : FMOD_Sound_GetTag;
  FcFMOD_Sound_GetOpenState : FMOD_Sound_GetOpenState;
  FcFMOD_Sound_ReadData : FMOD_Sound_ReadData;
  FcFMOD_Sound_SeekData : FMOD_Sound_SeekData;
  FcFMOD_Sound_SetSoundGroup : FMOD_Sound_SetSoundGroup;
  FcFMOD_Sound_GetSoundGroup : FMOD_Sound_GetSoundGroup;
  FcFMOD_Sound_GetNumSyncPoints : FMOD_Sound_GetNumSyncPoints;
  FcFMOD_Sound_GetSyncPoint : FMOD_Sound_GetSyncPoint;
  FcFMOD_Sound_GetSyncPointInfo : FMOD_Sound_GetSyncPointInfo;
  FcFMOD_Sound_AddSyncPoint : FMOD_Sound_AddSyncPoint;
  FcFMOD_Sound_DeleteSyncPoint : FMOD_Sound_DeleteSyncPoint;
  FcFMOD_Sound_SetMode : FMOD_Sound_SetMode;
  FcFMOD_Sound_GetMode : FMOD_Sound_GetMode;
  FcFMOD_Sound_SetLoopCount : FMOD_Sound_SetLoopCount;
  FcFMOD_Sound_GetLoopCount : FMOD_Sound_GetLoopCount;
  FcFMOD_Sound_SetLoopPoints : FMOD_Sound_SetLoopPoints;
  FcFMOD_Sound_GetLoopPoints : FMOD_Sound_GetLoopPoints;
  FcFMOD_Sound_GetMusicNumChannels : FMOD_Sound_GetMusicNumChannels;
  FcFMOD_Sound_SetMusicChannelVolume : FMOD_Sound_SetMusicChannelVolume;
  FcFMOD_Sound_GetMusicChannelVolume : FMOD_Sound_GetMusicChannelVolume;
  FcFMOD_Sound_SetMusicSpeed : FMOD_Sound_SetMusicSpeed;
  FcFMOD_Sound_GetMusicSpeed : FMOD_Sound_GetMusicSpeed;
  FcFMOD_Sound_SetUserData : FMOD_Sound_SetUserData;
  FcFMOD_Sound_GetUserData : FMOD_Sound_GetUserData;
  FcFMOD_Sound_GetMemoryInfo : FMOD_Sound_GetMemoryInfo;
  FcFMOD_Channel_GetSystemObject : FMOD_Channel_GetSystemObject;
  FcFMOD_Channel_Stop : FMOD_Channel_Stop;
  FcFMOD_Channel_SetPaused : FMOD_Channel_SetPaused;
  FcFMOD_Channel_GetPaused : FMOD_Channel_GetPaused;
  FcFMOD_Channel_SetVolume : FMOD_Channel_SetVolume;
  FcFMOD_Channel_GetVolume : FMOD_Channel_GetVolume;
  FcFMOD_Channel_SetFrequency : FMOD_Channel_SetFrequency;
  FcFMOD_Channel_GetFrequency : FMOD_Channel_GetFrequency;
  FcFMOD_Channel_SetPan : FMOD_Channel_SetPan;
  FcFMOD_Channel_GetPan : FMOD_Channel_GetPan;
  FcFMOD_Channel_SetDelay : FMOD_Channel_SetDelay;
  FcFMOD_Channel_GetDelay : FMOD_Channel_GetDelay;
  FcFMOD_Channel_SetSpeakerMix : FMOD_Channel_SetSpeakerMix;
  FcFMOD_Channel_GetSpeakerMix : FMOD_Channel_GetSpeakerMix;
  FcFMOD_Channel_SetSpeakerLevels : FMOD_Channel_SetSpeakerLevels;
  FcFMOD_Channel_GetSpeakerLevels : FMOD_Channel_GetSpeakerLevels;
  FcFMOD_Channel_SetInputChannelMix : FMOD_Channel_SetInputChannelMix;
  FcFMOD_Channel_GetInputChannelMix : FMOD_Channel_GetInputChannelMix;
  FcFMOD_Channel_SetMute : FMOD_Channel_SetMute;
  FcFMOD_Channel_GetMute : FMOD_Channel_GetMute;
  FcFMOD_Channel_SetPriority : FMOD_Channel_SetPriority;
  FcFMOD_Channel_GetPriority : FMOD_Channel_GetPriority;
  FcFMOD_Channel_SetPosition : FMOD_Channel_SetPosition;
  FcFMOD_Channel_GetPosition : FMOD_Channel_GetPosition;
  FcFMOD_Channel_SetReverbProperties : FMOD_Channel_SetReverbProperties;
  FcFMOD_Channel_GetReverbProperties : FMOD_Channel_GetReverbProperties;
  FcFMOD_Channel_SetLowPassGain : FMOD_Channel_SetLowPassGain;
  FcFMOD_Channel_GetLowPassGain : FMOD_Channel_GetLowPassGain;
  FcFMOD_Channel_SetChannelGroup : FMOD_Channel_SetChannelGroup;
  FcFMOD_Channel_GetChannelGroup : FMOD_Channel_GetChannelGroup;
  FcFMOD_Channel_SetCallback : FMOD_Channel_SetCallback;
  FcFMOD_Channel_Set3DAttributes : FMOD_Channel_Set3DAttributes;
  FcFMOD_Channel_Get3DAttributes : FMOD_Channel_Get3DAttributes;
  FcFMOD_Channel_Set3DMinMaxDistance : FMOD_Channel_Set3DMinMaxDistance;
  FcFMOD_Channel_Get3DMinMaxDistance : FMOD_Channel_Get3DMinMaxDistance;
  FcFMOD_Channel_Set3DConeSettings : FMOD_Channel_Set3DConeSettings;
  FcFMOD_Channel_Get3DConeSettings : FMOD_Channel_Get3DConeSettings;
  FcFMOD_Channel_Set3DConeOrientation : FMOD_Channel_Set3DConeOrientation;
  FcFMOD_Channel_Get3DConeOrientation : FMOD_Channel_Get3DConeOrientation;
  FcFMOD_Channel_Set3DCustomRolloff : FMOD_Channel_Set3DCustomRolloff;
  FcFMOD_Channel_Get3DCustomRolloff : FMOD_Channel_Get3DCustomRolloff;
  FcFMOD_Channel_Set3DOcclusion : FMOD_Channel_Set3DOcclusion;
  FcFMOD_Channel_Get3DOcclusion : FMOD_Channel_Get3DOcclusion;
  FcFMOD_Channel_Set3DSpread : FMOD_Channel_Set3DSpread;
  FcFMOD_Channel_Get3DSpread : FMOD_Channel_Get3DSpread;
  FcFMOD_Channel_Set3DPanLevel : FMOD_Channel_Set3DPanLevel;
  FcFMOD_Channel_Get3DPanLevel : FMOD_Channel_Get3DPanLevel;
  FcFMOD_Channel_Set3DDopplerLevel : FMOD_Channel_Set3DDopplerLevel;
  FcFMOD_Channel_Get3DDopplerLevel : FMOD_Channel_Get3DDopplerLevel;
  FcFMOD_Channel_GetDSPHead : FMOD_Channel_GetDSPHead;
  FcFMOD_Channel_AddDSP : FMOD_Channel_AddDSP;
  FcFMOD_Channel_IsPlaying : FMOD_Channel_IsPlaying;
  FcFMOD_Channel_IsVirtual : FMOD_Channel_IsVirtual;
  FcFMOD_Channel_GetAudibility : FMOD_Channel_GetAudibility;
  FcFMOD_Channel_GetCurrentSound : FMOD_Channel_GetCurrentSound;
  FcFMOD_Channel_GetSpectrum : FMOD_Channel_GetSpectrum;
  FcFMOD_Channel_GetWaveData : FMOD_Channel_GetWaveData;
  FcFMOD_Channel_GetIndex : FMOD_Channel_GetIndex;
  FcFMOD_Channel_SetMode : FMOD_Channel_SetMode;
  FcFMOD_Channel_GetMode : FMOD_Channel_GetMode;
  FcFMOD_Channel_SetLoopCount : FMOD_Channel_SetLoopCount;
  FcFMOD_Channel_GetLoopCount : FMOD_Channel_GetLoopCount;
  FcFMOD_Channel_SetLoopPoints : FMOD_Channel_SetLoopPoints;
  FcFMOD_Channel_GetLoopPoints : FMOD_Channel_GetLoopPoints;
  FcFMOD_Channel_SetUserData : FMOD_Channel_SetUserData;
  FcFMOD_Channel_GetUserData : FMOD_Channel_GetUserData;
  FcFMOD_Channel_GetMemoryInfo : FMOD_Channel_GetMemoryInfo;
  FcFMOD_ChannelGroup_Release : FMOD_ChannelGroup_Release;
  FcFMOD_ChannelGroup_GetSystemObject : FMOD_ChannelGroup_GetSystemObject;
  FcFMOD_ChannelGroup_SetVolume : FMOD_ChannelGroup_SetVolume;
  FcFMOD_ChannelGroup_GetVolume : FMOD_ChannelGroup_GetVolume;
  FcFMOD_ChannelGroup_SetPitch : FMOD_ChannelGroup_SetPitch;
  FcFMOD_ChannelGroup_GetPitch : FMOD_ChannelGroup_GetPitch;
  FcFMOD_ChannelGroup_Set3DOcclusion : FMOD_ChannelGroup_Set3DOcclusion;
  FcFMOD_ChannelGroup_Get3DOcclusion : FMOD_ChannelGroup_Get3DOcclusion;
  FcFMOD_ChannelGroup_SetPaused : FMOD_ChannelGroup_SetPaused;
  FcFMOD_ChannelGroup_GetPaused : FMOD_ChannelGroup_GetPaused;
  FcFMOD_ChannelGroup_SetMute : FMOD_ChannelGroup_SetMute;
  FcFMOD_ChannelGroup_GetMute : FMOD_ChannelGroup_GetMute;
  FcFMOD_ChannelGroup_Stop : FMOD_ChannelGroup_Stop;
  FcFMOD_ChannelGroup_OverrideVolume : FMOD_ChannelGroup_OverrideVolume;
  FcFMOD_ChannelGroup_OverrideFrequency : FMOD_ChannelGroup_OverrideFrequency;
  FcFMOD_ChannelGroup_OverridePan : FMOD_ChannelGroup_OverridePan;
  FcFMOD_ChannelGroup_OverrideReverbProperties : FMOD_ChannelGroup_OverrideReverbProperties;
  FcFMOD_ChannelGroup_Override3DAttributes : FMOD_ChannelGroup_Override3DAttributes;
  FcFMOD_ChannelGroup_OverrideSpeakerMix : FMOD_ChannelGroup_OverrideSpeakerMix;
  FcFMOD_ChannelGroup_AddGroup : FMOD_ChannelGroup_AddGroup;
  FcFMOD_ChannelGroup_GetNumGroups : FMOD_ChannelGroup_GetNumGroups;
  FcFMOD_ChannelGroup_GetGroup : FMOD_ChannelGroup_GetGroup;
  FcFMOD_ChannelGroup_GetParentGroup : FMOD_ChannelGroup_GetParentGroup;
  FcFMOD_ChannelGroup_GetDSPHead : FMOD_ChannelGroup_GetDSPHead;
  FcFMOD_ChannelGroup_AddDSP : FMOD_ChannelGroup_AddDSP;
  FcFMOD_ChannelGroup_GetName : FMOD_ChannelGroup_GetName;
  FcFMOD_ChannelGroup_GetNumChannels : FMOD_ChannelGroup_GetNumChannels;
  FcFMOD_ChannelGroup_GetChannel : FMOD_ChannelGroup_GetChannel;
  FcFMOD_ChannelGroup_GetSpectrum : FMOD_ChannelGroup_GetSpectrum;
  FcFMOD_ChannelGroup_GetWaveData : FMOD_ChannelGroup_GetWaveData;
  FcFMOD_ChannelGroup_SetUserData : FMOD_ChannelGroup_SetUserData;
  FcFMOD_ChannelGroup_GetUserData : FMOD_ChannelGroup_GetUserData;
  FcFMOD_ChannelGroup_GetMemoryInfo : FMOD_ChannelGroup_GetMemoryInfo;
  FcFMOD_SoundGroup_Release : FMOD_SoundGroup_Release;
  FcFMOD_SoundGroup_GetSystemObject : FMOD_SoundGroup_GetSystemObject;
  FcFMOD_SoundGroup_SetMaxAudible : FMOD_SoundGroup_SetMaxAudible;
  FcFMOD_SoundGroup_GetMaxAudible : FMOD_SoundGroup_GetMaxAudible;
  FcFMOD_SoundGroup_SetMaxAudibleBehavior : FMOD_SoundGroup_SetMaxAudibleBehavior;
  FcFMOD_SoundGroup_GetMaxAudibleBehavior : FMOD_SoundGroup_GetMaxAudibleBehavior;
  FcFMOD_SoundGroup_SetMuteFadeSpeed : FMOD_SoundGroup_SetMuteFadeSpeed;
  FcFMOD_SoundGroup_GetMuteFadeSpeed : FMOD_SoundGroup_GetMuteFadeSpeed;
  FcFMOD_SoundGroup_SetVolume : FMOD_SoundGroup_SetVolume;
  FcFMOD_SoundGroup_GetVolume : FMOD_SoundGroup_GetVolume;
  FcFMOD_SoundGroup_Stop : FMOD_SoundGroup_Stop;
  FcFMOD_SoundGroup_GetName : FMOD_SoundGroup_GetName;
  FcFMOD_SoundGroup_GetNumSounds : FMOD_SoundGroup_GetNumSounds;
  FcFMOD_SoundGroup_GetSound : FMOD_SoundGroup_GetSound;
  FcFMOD_SoundGroup_GetNumPlaying : FMOD_SoundGroup_GetNumPlaying;
  FcFMOD_SoundGroup_SetUserData : FMOD_SoundGroup_SetUserData;
  FcFMOD_SoundGroup_GetUserData : FMOD_SoundGroup_GetUserData;
  FcFMOD_SoundGroup_GetMemoryInfo : FMOD_SoundGroup_GetMemoryInfo;
  FcFMOD_DSP_Release : FMOD_DSP_Release;
  FcFMOD_DSP_GetSystemObject : FMOD_DSP_GetSystemObject;
  FcFMOD_DSP_AddInput : FMOD_DSP_AddInput;
  FcFMOD_DSP_DisconnectFrom : FMOD_DSP_DisconnectFrom;
  FcFMOD_DSP_DisconnectAll : FMOD_DSP_DisconnectAll;
  FcFMOD_DSP_Remove : FMOD_DSP_Remove;
  FcFMOD_DSP_GetNumInputs : FMOD_DSP_GetNumInputs;
  FcFMOD_DSP_GetNumOutputs : FMOD_DSP_GetNumOutputs;
  FcFMOD_DSP_GetInput : FMOD_DSP_GetInput;
  FcFMOD_DSP_GetOutput : FMOD_DSP_GetOutput;
  FcFMOD_DSP_SetActive : FMOD_DSP_SetActive;
  FcFMOD_DSP_GetActive : FMOD_DSP_GetActive;
  FcFMOD_DSP_SetBypass : FMOD_DSP_SetBypass;
  FcFMOD_DSP_GetBypass : FMOD_DSP_GetBypass;
  FcFMOD_DSP_SetSpeakerActive : FMOD_DSP_SetSpeakerActive;
  FcFMOD_DSP_GetSpeakerActive : FMOD_DSP_GetSpeakerActive;
  FcFMOD_DSP_Reset : FMOD_DSP_Reset;
  FcFMOD_DSP_SetParameter : FMOD_DSP_SetParameter;
  FcFMOD_DSP_GetParameter : FMOD_DSP_GetParameter;
  FcFMOD_DSP_GetNumParameters : FMOD_DSP_GetNumParameters;
  FcFMOD_DSP_GetParameterInfo : FMOD_DSP_GetParameterInfo;
  FcFMOD_DSP_ShowConfigDialog : FMOD_DSP_ShowConfigDialog;
  FcFMOD_DSP_GetInfo : FMOD_DSP_GetInfo;
  FcFMOD_DSP_GetType : FMOD_DSP_GetType;
  FcFMOD_DSP_SetDefaults : FMOD_DSP_SetDefaults;
  FcFMOD_DSP_GetDefaults : FMOD_DSP_GetDefaults;
  FcFMOD_DSP_SetUserData : FMOD_DSP_SetUserData;
  FcFMOD_DSP_GetUserData : FMOD_DSP_GetUserData;
  FcFMOD_DSP_GetMemoryInfo : FMOD_DSP_GetMemoryInfo;
  FcFMOD_DSPConnection_GetInput : FMOD_DSPConnection_GetInput;
  FcFMOD_DSPConnection_GetOutput : FMOD_DSPConnection_GetOutput;
  FcFMOD_DSPConnection_SetMix : FMOD_DSPConnection_SetMix;
  FcFMOD_DSPConnection_GetMix : FMOD_DSPConnection_GetMix;
  FcFMOD_DSPConnection_SetLevels : FMOD_DSPConnection_SetLevels;
  FcFMOD_DSPConnection_GetLevels : FMOD_DSPConnection_GetLevels;
  FcFMOD_DSPConnection_SetUserData : FMOD_DSPConnection_SetUserData;
  FcFMOD_DSPConnection_GetUserData : FMOD_DSPConnection_GetUserData;
  FcFMOD_DSPConnection_GetMemoryInfo : FMOD_DSPConnection_GetMemoryInfo;
  FcFMOD_Geometry_Release : FMOD_Geometry_Release;
  FcFMOD_Geometry_AddPolygon : FMOD_Geometry_AddPolygon;
  FcFMOD_Geometry_GetNumPolygons : FMOD_Geometry_GetNumPolygons;
  FcFMOD_Geometry_GetMaxPolygons : FMOD_Geometry_GetMaxPolygons;
  FcFMOD_Geometry_GetPolygonNumVertices : FMOD_Geometry_GetPolygonNumVertices;
  FcFMOD_Geometry_SetPolygonVertex : FMOD_Geometry_SetPolygonVertex;
  FcFMOD_Geometry_GetPolygonVertex : FMOD_Geometry_GetPolygonVertex;
  FcFMOD_Geometry_SetPolygonAttributes : FMOD_Geometry_SetPolygonAttributes;
  FcFMOD_Geometry_GetPolygonAttributes : FMOD_Geometry_GetPolygonAttributes;
  FcFMOD_Geometry_SetActive : FMOD_Geometry_SetActive;
  FcFMOD_Geometry_GetActive : FMOD_Geometry_GetActive;
  FcFMOD_Geometry_SetRotation : FMOD_Geometry_SetRotation;
  FcFMOD_Geometry_GetRotation : FMOD_Geometry_GetRotation;
  FcFMOD_Geometry_SetPosition : FMOD_Geometry_SetPosition;
  FcFMOD_Geometry_GetPosition : FMOD_Geometry_GetPosition;
  FcFMOD_Geometry_SetScale : FMOD_Geometry_SetScale;
  FcFMOD_Geometry_GetScale : FMOD_Geometry_GetScale;
  FcFMOD_Geometry_Save : FMOD_Geometry_Save;
  FcFMOD_Geometry_SetUserData : FMOD_Geometry_SetUserData;
  FcFMOD_Geometry_GetUserData : FMOD_Geometry_GetUserData;
  FcFMOD_Geometry_GetMemoryInfo : FMOD_Geometry_GetMemoryInfo;
  FcFMOD_Reverb_Release : FMOD_Reverb_Release;
  FcFMOD_Reverb_Set3DAttributes : FMOD_Reverb_Set3DAttributes;
  FcFMOD_Reverb_Get3DAttributes : FMOD_Reverb_Get3DAttributes;
  FcFMOD_Reverb_SetProperties : FMOD_Reverb_SetProperties;
  FcFMOD_Reverb_GetProperties : FMOD_Reverb_GetProperties;
  FcFMOD_Reverb_SetActive : FMOD_Reverb_SetActive;
  FcFMOD_Reverb_GetActive : FMOD_Reverb_GetActive;
  FcFMOD_Reverb_SetUserData : FMOD_Reverb_SetUserData;
  FcFMOD_Reverb_GetUserData : FMOD_Reverb_GetUserData;
  FcFMOD_Reverb_GetMemoryInfo : FMOD_Reverb_GetMemoryInfo;
  PFMODSystem : Pointer;

implementation

uses
  Windows, SysUtils, Dialogs;

var
  Handle : THandle;

  function LocalGetProcAddress(const AFunc:String):FARPROC;
  begin
    Result := GetProcAddress(Handle, PChar(AFunc));
    if Result=nil then
      raise Exception.Create('Function <'+AFunc+'> not found. Aborting');
  end;

var
  Res   : FMOD_RESULT;
  sound : FMOD_SOUND;
  PFMODSystemToRelease : Boolean;
  poolmem              : Pointer;
initialization
  PFMODSystemToRelease := False;
  Handle := LoadLibrary('C:\Olivier\Sources\Sources olivier\FMod\fmodex.dll');
  if Handle <> 0 then
  begin
    @FcFMOD_Memory_Initialize := LocalGetProcAddress('FMOD_Memory_Initialize');
    @FcFMOD_Memory_GetStats := LocalGetProcAddress('FMOD_Memory_GetStats');
    @FcFMOD_Debug_SetLevel := LocalGetProcAddress('FMOD_Debug_SetLevel');
    @FcFMOD_Debug_GetLevel := LocalGetProcAddress('FMOD_Debug_GetLevel');
    @FcFMOD_File_SetDiskBusy := LocalGetProcAddress('FMOD_File_SetDiskBusy');
    @FcFMOD_File_GetDiskBusy := LocalGetProcAddress('FMOD_File_GetDiskBusy');
    @FcFMOD_System_Create := LocalGetProcAddress('FMOD_System_Create');
    @FcFMOD_System_Release := LocalGetProcAddress('FMOD_System_Release');
    @FcFMOD_System_SetOutput := LocalGetProcAddress('FMOD_System_SetOutput');
    @FcFMOD_System_GetOutput := LocalGetProcAddress('FMOD_System_GetOutput');
    @FcFMOD_System_GetNumDrivers := LocalGetProcAddress('FMOD_System_GetNumDrivers');
    @FcFMOD_System_GetDriverInfo := LocalGetProcAddress('FMOD_System_GetDriverInfo');
    @FcFMOD_System_GetDriverInfoW := LocalGetProcAddress('FMOD_System_GetDriverInfoW');
    @FcFMOD_System_GetDriverCaps := LocalGetProcAddress('FMOD_System_GetDriverCaps');
    @FcFMOD_System_SetDriver := LocalGetProcAddress('FMOD_System_SetDriver');
    @FcFMOD_System_GetDriver := LocalGetProcAddress('FMOD_System_GetDriver');
    @FcFMOD_System_SetHardwareChannels := LocalGetProcAddress('FMOD_System_SetHardwareChannels');
    @FcFMOD_System_SetSoftwareChannels := LocalGetProcAddress('FMOD_System_SetSoftwareChannels');
    @FcFMOD_System_GetSoftwareChannels := LocalGetProcAddress('FMOD_System_GetSoftwareChannels');
    @FcFMOD_System_SetSoftwareFormat := LocalGetProcAddress('FMOD_System_SetSoftwareFormat');
    @FcFMOD_System_GetSoftwareFormat := LocalGetProcAddress('FMOD_System_GetSoftwareFormat');
    @FcFMOD_System_SetDSPBufferSize := LocalGetProcAddress('FMOD_System_SetDSPBufferSize');
    @FcFMOD_System_GetDSPBufferSize := LocalGetProcAddress('FMOD_System_GetDSPBufferSize');
    @FcFMOD_System_SetFileSystem := LocalGetProcAddress('FMOD_System_SetFileSystem');
    @FcFMOD_System_AttachFileSystem := LocalGetProcAddress('FMOD_System_AttachFileSystem');
    @FcFMOD_System_SetAdvancedSettings := LocalGetProcAddress('FMOD_System_SetAdvancedSettings');
    @FcFMOD_System_GetAdvancedSettings := LocalGetProcAddress('FMOD_System_GetAdvancedSettings');
    @FcFMOD_System_SetSpeakerMode := LocalGetProcAddress('FMOD_System_SetSpeakerMode');
    @FcFMOD_System_GetSpeakerMode := LocalGetProcAddress('FMOD_System_GetSpeakerMode');
    @FcFMOD_System_SetCallback := LocalGetProcAddress('FMOD_System_SetCallback');
    @FcFMOD_System_SetPluginPath := LocalGetProcAddress('FMOD_System_SetPluginPath');
    @FcFMOD_System_LoadPlugin := LocalGetProcAddress('FMOD_System_LoadPlugin');
    @FcFMOD_System_UnloadPlugin := LocalGetProcAddress('FMOD_System_UnloadPlugin');
    @FcFMOD_System_GetNumPlugins := LocalGetProcAddress('FMOD_System_GetNumPlugins');
    @FcFMOD_System_GetPluginHandle := LocalGetProcAddress('FMOD_System_GetPluginHandle');
    @FcFMOD_System_GetPluginInfo := LocalGetProcAddress('FMOD_System_GetPluginInfo');
    @FcFMOD_System_SetOutputByPlugin := LocalGetProcAddress('FMOD_System_SetOutputByPlugin');
    @FcFMOD_System_GetOutputByPlugin := LocalGetProcAddress('FMOD_System_GetOutputByPlugin');
    @FcFMOD_System_CreateDSPByPlugin := LocalGetProcAddress('FMOD_System_CreateDSPByPlugin');
    @FcFMOD_System_CreateCodec := LocalGetProcAddress('FMOD_System_CreateCodec');
    @FcFMOD_System_Init := LocalGetProcAddress('FMOD_System_Init');
    @FcFMOD_System_Close := LocalGetProcAddress('FMOD_System_Close');
    @FcFMOD_System_Update := LocalGetProcAddress('FMOD_System_Update');
    @FcFMOD_System_Set3DSettings := LocalGetProcAddress('FMOD_System_Set3DSettings');
    @FcFMOD_System_Get3DSettings := LocalGetProcAddress('FMOD_System_Get3DSettings');
    @FcFMOD_System_Set3DNumListeners := LocalGetProcAddress('FMOD_System_Set3DNumListeners');
    @FcFMOD_System_Get3DNumListeners := LocalGetProcAddress('FMOD_System_Get3DNumListeners');
    @FcFMOD_System_Set3DListenerAttributes := LocalGetProcAddress('FMOD_System_Set3DListenerAttributes');
    @FcFMOD_System_Get3DListenerAttributes := LocalGetProcAddress('FMOD_System_Get3DListenerAttributes');
    @FcFMOD_System_Set3DRolloffCallback := LocalGetProcAddress('FMOD_System_Set3DRolloffCallback');
    @FcFMOD_System_Set3DSpeakerPosition := LocalGetProcAddress('FMOD_System_Set3DSpeakerPosition');
    @FcFMOD_System_Get3DSpeakerPosition := LocalGetProcAddress('FMOD_System_Get3DSpeakerPosition');
    @FcFMOD_System_SetStreamBufferSize := LocalGetProcAddress('FMOD_System_SetStreamBufferSize');
    @FcFMOD_System_GetStreamBufferSize := LocalGetProcAddress('FMOD_System_GetStreamBufferSize');
    @FcFMOD_System_GetVersion := LocalGetProcAddress('FMOD_System_GetVersion');
    @FcFMOD_System_GetOutputHandle := LocalGetProcAddress('FMOD_System_GetOutputHandle');
    @FcFMOD_System_GetChannelsPlaying := LocalGetProcAddress('FMOD_System_GetChannelsPlaying');
    @FcFMOD_System_GetHardwareChannels := LocalGetProcAddress('FMOD_System_GetHardwareChannels');
    @FcFMOD_System_GetCPUUsage := LocalGetProcAddress('FMOD_System_GetCPUUsage');
    @FcFMOD_System_GetSoundRAM := LocalGetProcAddress('FMOD_System_GetSoundRAM');
    @FcFMOD_System_GetNumCDROMDrives := LocalGetProcAddress('FMOD_System_GetNumCDROMDrives');
    @FcFMOD_System_GetCDROMDriveName := LocalGetProcAddress('FMOD_System_GetCDROMDriveName');
    @FcFMOD_System_GetSpectrum := LocalGetProcAddress('FMOD_System_GetSpectrum');
    @FcFMOD_System_GetWaveData := LocalGetProcAddress('FMOD_System_GetWaveData');
    @FcFMOD_System_CreateSound := LocalGetProcAddress('FMOD_System_CreateSound');
    @FcFMOD_System_CreateStream := LocalGetProcAddress('FMOD_System_CreateStream');
    @FcFMOD_System_CreateDSP := LocalGetProcAddress('FMOD_System_CreateDSP');
    @FcFMOD_System_CreateDSPByType := LocalGetProcAddress('FMOD_System_CreateDSPByType');
    @FcFMOD_System_CreateChannelGroup := LocalGetProcAddress('FMOD_System_CreateChannelGroup');
    @FcFMOD_System_CreateSoundGroup := LocalGetProcAddress('FMOD_System_CreateSoundGroup');
    @FcFMOD_System_CreateReverb := LocalGetProcAddress('FMOD_System_CreateReverb');
    @FcFMOD_System_PlaySound := LocalGetProcAddress('FMOD_System_PlaySound');
    @FcFMOD_System_PlayDSP := LocalGetProcAddress('FMOD_System_PlayDSP');
    @FcFMOD_System_GetChannel := LocalGetProcAddress('FMOD_System_GetChannel');
    @FcFMOD_System_GetMasterChannelGroup := LocalGetProcAddress('FMOD_System_GetMasterChannelGroup');
    @FcFMOD_System_GetMasterSoundGroup := LocalGetProcAddress('FMOD_System_GetMasterSoundGroup');
    @FcFMOD_System_SetReverbProperties := LocalGetProcAddress('FMOD_System_SetReverbProperties');
    @FcFMOD_System_GetReverbProperties := LocalGetProcAddress('FMOD_System_GetReverbProperties');
    @FcFMOD_System_SetReverbAmbientProperties := LocalGetProcAddress('FMOD_System_SetReverbAmbientProperties');
    @FcFMOD_System_GetReverbAmbientProperties := LocalGetProcAddress('FMOD_System_GetReverbAmbientProperties');
    @FcFMOD_System_GetDSPHead := LocalGetProcAddress('FMOD_System_GetDSPHead');
    @FcFMOD_System_AddDSP := LocalGetProcAddress('FMOD_System_AddDSP');
    @FcFMOD_System_LockDSP := LocalGetProcAddress('FMOD_System_LockDSP');
    @FcFMOD_System_UnlockDSP := LocalGetProcAddress('FMOD_System_UnlockDSP');
    @FcFMOD_System_GetDSPClock := LocalGetProcAddress('FMOD_System_GetDSPClock');
    @FcFMOD_System_GetRecordNumDrivers := LocalGetProcAddress('FMOD_System_GetRecordNumDrivers');
    @FcFMOD_System_GetRecordDriverInfo := LocalGetProcAddress('FMOD_System_GetRecordDriverInfo');
    @FcFMOD_System_GetRecordDriverInfoW := LocalGetProcAddress('FMOD_System_GetRecordDriverInfoW');
    @FcFMOD_System_GetRecordDriverCaps := LocalGetProcAddress('FMOD_System_GetRecordDriverCaps');
    @FcFMOD_System_GetRecordPosition := LocalGetProcAddress('FMOD_System_GetRecordPosition');
    @FcFMOD_System_RecordStart := LocalGetProcAddress('FMOD_System_RecordStart');
    @FcFMOD_System_RecordStop := LocalGetProcAddress('FMOD_System_RecordStop');
    @FcFMOD_System_IsRecording := LocalGetProcAddress('FMOD_System_IsRecording');
    @FcFMOD_System_CreateGeometry := LocalGetProcAddress('FMOD_System_CreateGeometry');
    @FcFMOD_System_SetGeometrySettings := LocalGetProcAddress('FMOD_System_SetGeometrySettings');
    @FcFMOD_System_GetGeometrySettings := LocalGetProcAddress('FMOD_System_GetGeometrySettings');
    @FcFMOD_System_LoadGeometry := LocalGetProcAddress('FMOD_System_LoadGeometry');
    @FcFMOD_System_GetGeometryOcclusion := LocalGetProcAddress('FMOD_System_GetGeometryOcclusion');
    @FcFMOD_System_SetNetworkProxy := LocalGetProcAddress('FMOD_System_SetNetworkProxy');
    @FcFMOD_System_GetNetworkProxy := LocalGetProcAddress('FMOD_System_GetNetworkProxy');
    @FcFMOD_System_SetNetworkTimeout := LocalGetProcAddress('FMOD_System_SetNetworkTimeout');
    @FcFMOD_System_GetNetworkTimeout := LocalGetProcAddress('FMOD_System_GetNetworkTimeout');
    @FcFMOD_System_SetUserData := LocalGetProcAddress('FMOD_System_SetUserData');
    @FcFMOD_System_GetUserData := LocalGetProcAddress('FMOD_System_GetUserData');
    @FcFMOD_System_GetMemoryInfo := LocalGetProcAddress('FMOD_System_GetMemoryInfo');
    @FcFMOD_Sound_Release := LocalGetProcAddress('FMOD_Sound_Release');
    @FcFMOD_Sound_GetSystemObject := LocalGetProcAddress('FMOD_Sound_GetSystemObject');
    @FcFMOD_Sound_Lock := LocalGetProcAddress('FMOD_Sound_Lock');
    @FcFMOD_Sound_Unlock := LocalGetProcAddress('FMOD_Sound_Unlock');
    @FcFMOD_Sound_SetDefaults := LocalGetProcAddress('FMOD_Sound_SetDefaults');
    @FcFMOD_Sound_GetDefaults := LocalGetProcAddress('FMOD_Sound_GetDefaults');
    @FcFMOD_Sound_SetVariations := LocalGetProcAddress('FMOD_Sound_SetVariations');
    @FcFMOD_Sound_GetVariations := LocalGetProcAddress('FMOD_Sound_GetVariations');
    @FcFMOD_Sound_Set3DMinMaxDistance := LocalGetProcAddress('FMOD_Sound_Set3DMinMaxDistance');
    @FcFMOD_Sound_Get3DMinMaxDistance := LocalGetProcAddress('FMOD_Sound_Get3DMinMaxDistance');
    @FcFMOD_Sound_Set3DConeSettings := LocalGetProcAddress('FMOD_Sound_Set3DConeSettings');
    @FcFMOD_Sound_Get3DConeSettings := LocalGetProcAddress('FMOD_Sound_Get3DConeSettings');
    @FcFMOD_Sound_Set3DCustomRolloff := LocalGetProcAddress('FMOD_Sound_Set3DCustomRolloff');
    @FcFMOD_Sound_Get3DCustomRolloff := LocalGetProcAddress('FMOD_Sound_Get3DCustomRolloff');
    @FcFMOD_Sound_SetSubSound := LocalGetProcAddress('FMOD_Sound_SetSubSound');
    @FcFMOD_Sound_GetSubSound := LocalGetProcAddress('FMOD_Sound_GetSubSound');
    @FcFMOD_Sound_SetSubSoundSentence := LocalGetProcAddress('FMOD_Sound_SetSubSoundSentence');
    @FcFMOD_Sound_GetName := LocalGetProcAddress('FMOD_Sound_GetName');
    @FcFMOD_Sound_GetLength := LocalGetProcAddress('FMOD_Sound_GetLength');
    @FcFMOD_Sound_GetFormat := LocalGetProcAddress('FMOD_Sound_GetFormat');
    @FcFMOD_Sound_GetNumSubSounds := LocalGetProcAddress('FMOD_Sound_GetNumSubSounds');
    @FcFMOD_Sound_GetNumTags := LocalGetProcAddress('FMOD_Sound_GetNumTags');
    @FcFMOD_Sound_GetTag := LocalGetProcAddress('FMOD_Sound_GetTag');
    @FcFMOD_Sound_GetOpenState := LocalGetProcAddress('FMOD_Sound_GetOpenState');
    @FcFMOD_Sound_ReadData := LocalGetProcAddress('FMOD_Sound_ReadData');
    @FcFMOD_Sound_SeekData := LocalGetProcAddress('FMOD_Sound_SeekData');
    @FcFMOD_Sound_SetSoundGroup := LocalGetProcAddress('FMOD_Sound_SetSoundGroup');
    @FcFMOD_Sound_GetSoundGroup := LocalGetProcAddress('FMOD_Sound_GetSoundGroup');
    @FcFMOD_Sound_GetNumSyncPoints := LocalGetProcAddress('FMOD_Sound_GetNumSyncPoints');
    @FcFMOD_Sound_GetSyncPoint := LocalGetProcAddress('FMOD_Sound_GetSyncPoint');
    @FcFMOD_Sound_GetSyncPointInfo := LocalGetProcAddress('FMOD_Sound_GetSyncPointInfo');
    @FcFMOD_Sound_AddSyncPoint := LocalGetProcAddress('FMOD_Sound_AddSyncPoint');
    @FcFMOD_Sound_DeleteSyncPoint := LocalGetProcAddress('FMOD_Sound_DeleteSyncPoint');
    @FcFMOD_Sound_SetMode := LocalGetProcAddress('FMOD_Sound_SetMode');
    @FcFMOD_Sound_GetMode := LocalGetProcAddress('FMOD_Sound_GetMode');
    @FcFMOD_Sound_SetLoopCount := LocalGetProcAddress('FMOD_Sound_SetLoopCount');
    @FcFMOD_Sound_GetLoopCount := LocalGetProcAddress('FMOD_Sound_GetLoopCount');
    @FcFMOD_Sound_SetLoopPoints := LocalGetProcAddress('FMOD_Sound_SetLoopPoints');
    @FcFMOD_Sound_GetLoopPoints := LocalGetProcAddress('FMOD_Sound_GetLoopPoints');
    @FcFMOD_Sound_GetMusicNumChannels := LocalGetProcAddress('FMOD_Sound_GetMusicNumChannels');
    @FcFMOD_Sound_SetMusicChannelVolume := LocalGetProcAddress('FMOD_Sound_SetMusicChannelVolume');
    @FcFMOD_Sound_GetMusicChannelVolume := LocalGetProcAddress('FMOD_Sound_GetMusicChannelVolume');
    @FcFMOD_Sound_SetMusicSpeed := LocalGetProcAddress('FMOD_Sound_SetMusicSpeed');
    @FcFMOD_Sound_GetMusicSpeed := LocalGetProcAddress('FMOD_Sound_GetMusicSpeed');
    @FcFMOD_Sound_SetUserData := LocalGetProcAddress('FMOD_Sound_SetUserData');
    @FcFMOD_Sound_GetUserData := LocalGetProcAddress('FMOD_Sound_GetUserData');
    @FcFMOD_Sound_GetMemoryInfo := LocalGetProcAddress('FMOD_Sound_GetMemoryInfo');
    @FcFMOD_Channel_GetSystemObject := LocalGetProcAddress('FMOD_Channel_GetSystemObject');
    @FcFMOD_Channel_Stop := LocalGetProcAddress('FMOD_Channel_Stop');
    @FcFMOD_Channel_SetPaused := LocalGetProcAddress('FMOD_Channel_SetPaused');
    @FcFMOD_Channel_GetPaused := LocalGetProcAddress('FMOD_Channel_GetPaused');
    @FcFMOD_Channel_SetVolume := LocalGetProcAddress('FMOD_Channel_SetVolume');
    @FcFMOD_Channel_GetVolume := LocalGetProcAddress('FMOD_Channel_GetVolume');
    @FcFMOD_Channel_SetFrequency := LocalGetProcAddress('FMOD_Channel_SetFrequency');
    @FcFMOD_Channel_GetFrequency := LocalGetProcAddress('FMOD_Channel_GetFrequency');
    @FcFMOD_Channel_SetPan := LocalGetProcAddress('FMOD_Channel_SetPan');
    @FcFMOD_Channel_GetPan := LocalGetProcAddress('FMOD_Channel_GetPan');
    @FcFMOD_Channel_SetDelay := LocalGetProcAddress('FMOD_Channel_SetDelay');
    @FcFMOD_Channel_GetDelay := LocalGetProcAddress('FMOD_Channel_GetDelay');
    @FcFMOD_Channel_SetSpeakerMix := LocalGetProcAddress('FMOD_Channel_SetSpeakerMix');
    @FcFMOD_Channel_GetSpeakerMix := LocalGetProcAddress('FMOD_Channel_GetSpeakerMix');
    @FcFMOD_Channel_SetSpeakerLevels := LocalGetProcAddress('FMOD_Channel_SetSpeakerLevels');
    @FcFMOD_Channel_GetSpeakerLevels := LocalGetProcAddress('FMOD_Channel_GetSpeakerLevels');
    @FcFMOD_Channel_SetInputChannelMix := LocalGetProcAddress('FMOD_Channel_SetInputChannelMix');
    @FcFMOD_Channel_GetInputChannelMix := LocalGetProcAddress('FMOD_Channel_GetInputChannelMix');
    @FcFMOD_Channel_SetMute := LocalGetProcAddress('FMOD_Channel_SetMute');
    @FcFMOD_Channel_GetMute := LocalGetProcAddress('FMOD_Channel_GetMute');
    @FcFMOD_Channel_SetPriority := LocalGetProcAddress('FMOD_Channel_SetPriority');
    @FcFMOD_Channel_GetPriority := LocalGetProcAddress('FMOD_Channel_GetPriority');
    @FcFMOD_Channel_SetPosition := LocalGetProcAddress('FMOD_Channel_SetPosition');
    @FcFMOD_Channel_GetPosition := LocalGetProcAddress('FMOD_Channel_GetPosition');
    @FcFMOD_Channel_SetReverbProperties := LocalGetProcAddress('FMOD_Channel_SetReverbProperties');
    @FcFMOD_Channel_GetReverbProperties := LocalGetProcAddress('FMOD_Channel_GetReverbProperties');
    @FcFMOD_Channel_SetLowPassGain := LocalGetProcAddress('FMOD_Channel_SetLowPassGain');
    @FcFMOD_Channel_GetLowPassGain := LocalGetProcAddress('FMOD_Channel_GetLowPassGain');
    @FcFMOD_Channel_SetChannelGroup := LocalGetProcAddress('FMOD_Channel_SetChannelGroup');
    @FcFMOD_Channel_GetChannelGroup := LocalGetProcAddress('FMOD_Channel_GetChannelGroup');
    @FcFMOD_Channel_SetCallback := LocalGetProcAddress('FMOD_Channel_SetCallback');
    @FcFMOD_Channel_Set3DAttributes := LocalGetProcAddress('FMOD_Channel_Set3DAttributes');
    @FcFMOD_Channel_Get3DAttributes := LocalGetProcAddress('FMOD_Channel_Get3DAttributes');
    @FcFMOD_Channel_Set3DMinMaxDistance := LocalGetProcAddress('FMOD_Channel_Set3DMinMaxDistance');
    @FcFMOD_Channel_Get3DMinMaxDistance := LocalGetProcAddress('FMOD_Channel_Get3DMinMaxDistance');
    @FcFMOD_Channel_Set3DConeSettings := LocalGetProcAddress('FMOD_Channel_Set3DConeSettings');
    @FcFMOD_Channel_Get3DConeSettings := LocalGetProcAddress('FMOD_Channel_Get3DConeSettings');
    @FcFMOD_Channel_Set3DConeOrientation := LocalGetProcAddress('FMOD_Channel_Set3DConeOrientation');
    @FcFMOD_Channel_Get3DConeOrientation := LocalGetProcAddress('FMOD_Channel_Get3DConeOrientation');
    @FcFMOD_Channel_Set3DCustomRolloff := LocalGetProcAddress('FMOD_Channel_Set3DCustomRolloff');
    @FcFMOD_Channel_Get3DCustomRolloff := LocalGetProcAddress('FMOD_Channel_Get3DCustomRolloff');
    @FcFMOD_Channel_Set3DOcclusion := LocalGetProcAddress('FMOD_Channel_Set3DOcclusion');
    @FcFMOD_Channel_Get3DOcclusion := LocalGetProcAddress('FMOD_Channel_Get3DOcclusion');
    @FcFMOD_Channel_Set3DSpread := LocalGetProcAddress('FMOD_Channel_Set3DSpread');
    @FcFMOD_Channel_Get3DSpread := LocalGetProcAddress('FMOD_Channel_Get3DSpread');
    @FcFMOD_Channel_Set3DPanLevel := LocalGetProcAddress('FMOD_Channel_Set3DPanLevel');
    @FcFMOD_Channel_Get3DPanLevel := LocalGetProcAddress('FMOD_Channel_Get3DPanLevel');
    @FcFMOD_Channel_Set3DDopplerLevel := LocalGetProcAddress('FMOD_Channel_Set3DDopplerLevel');
    @FcFMOD_Channel_Get3DDopplerLevel := LocalGetProcAddress('FMOD_Channel_Get3DDopplerLevel');
    @FcFMOD_Channel_GetDSPHead := LocalGetProcAddress('FMOD_Channel_GetDSPHead');
    @FcFMOD_Channel_AddDSP := LocalGetProcAddress('FMOD_Channel_AddDSP');
    @FcFMOD_Channel_IsPlaying := LocalGetProcAddress('FMOD_Channel_IsPlaying');
    @FcFMOD_Channel_IsVirtual := LocalGetProcAddress('FMOD_Channel_IsVirtual');
    @FcFMOD_Channel_GetAudibility := LocalGetProcAddress('FMOD_Channel_GetAudibility');
    @FcFMOD_Channel_GetCurrentSound := LocalGetProcAddress('FMOD_Channel_GetCurrentSound');
    @FcFMOD_Channel_GetSpectrum := LocalGetProcAddress('FMOD_Channel_GetSpectrum');
    @FcFMOD_Channel_GetWaveData := LocalGetProcAddress('FMOD_Channel_GetWaveData');
    @FcFMOD_Channel_GetIndex := LocalGetProcAddress('FMOD_Channel_GetIndex');
    @FcFMOD_Channel_SetMode := LocalGetProcAddress('FMOD_Channel_SetMode');
    @FcFMOD_Channel_GetMode := LocalGetProcAddress('FMOD_Channel_GetMode');
    @FcFMOD_Channel_SetLoopCount := LocalGetProcAddress('FMOD_Channel_SetLoopCount');
    @FcFMOD_Channel_GetLoopCount := LocalGetProcAddress('FMOD_Channel_GetLoopCount');
    @FcFMOD_Channel_SetLoopPoints := LocalGetProcAddress('FMOD_Channel_SetLoopPoints');
    @FcFMOD_Channel_GetLoopPoints := LocalGetProcAddress('FMOD_Channel_GetLoopPoints');
    @FcFMOD_Channel_SetUserData := LocalGetProcAddress('FMOD_Channel_SetUserData');
    @FcFMOD_Channel_GetUserData := LocalGetProcAddress('FMOD_Channel_GetUserData');
    @FcFMOD_Channel_GetMemoryInfo := LocalGetProcAddress('FMOD_Channel_GetMemoryInfo');
    @FcFMOD_ChannelGroup_Release := LocalGetProcAddress('FMOD_ChannelGroup_Release');
    @FcFMOD_ChannelGroup_GetSystemObject := LocalGetProcAddress('FMOD_ChannelGroup_GetSystemObject');
    @FcFMOD_ChannelGroup_SetVolume := LocalGetProcAddress('FMOD_ChannelGroup_SetVolume');
    @FcFMOD_ChannelGroup_GetVolume := LocalGetProcAddress('FMOD_ChannelGroup_GetVolume');
    @FcFMOD_ChannelGroup_SetPitch := LocalGetProcAddress('FMOD_ChannelGroup_SetPitch');
    @FcFMOD_ChannelGroup_GetPitch := LocalGetProcAddress('FMOD_ChannelGroup_GetPitch');
    @FcFMOD_ChannelGroup_Set3DOcclusion := LocalGetProcAddress('FMOD_ChannelGroup_Set3DOcclusion');
    @FcFMOD_ChannelGroup_Get3DOcclusion := LocalGetProcAddress('FMOD_ChannelGroup_Get3DOcclusion');
    @FcFMOD_ChannelGroup_SetPaused := LocalGetProcAddress('FMOD_ChannelGroup_SetPaused');
    @FcFMOD_ChannelGroup_GetPaused := LocalGetProcAddress('FMOD_ChannelGroup_GetPaused');
    @FcFMOD_ChannelGroup_SetMute := LocalGetProcAddress('FMOD_ChannelGroup_SetMute');
    @FcFMOD_ChannelGroup_GetMute := LocalGetProcAddress('FMOD_ChannelGroup_GetMute');
    @FcFMOD_ChannelGroup_Stop := LocalGetProcAddress('FMOD_ChannelGroup_Stop');
    @FcFMOD_ChannelGroup_OverrideVolume := LocalGetProcAddress('FMOD_ChannelGroup_OverrideVolume');
    @FcFMOD_ChannelGroup_OverrideFrequency := LocalGetProcAddress('FMOD_ChannelGroup_OverrideFrequency');
    @FcFMOD_ChannelGroup_OverridePan := LocalGetProcAddress('FMOD_ChannelGroup_OverridePan');
    @FcFMOD_ChannelGroup_OverrideReverbProperties := LocalGetProcAddress('FMOD_ChannelGroup_OverrideReverbProperties');
    @FcFMOD_ChannelGroup_Override3DAttributes := LocalGetProcAddress('FMOD_ChannelGroup_Override3DAttributes');
    @FcFMOD_ChannelGroup_OverrideSpeakerMix := LocalGetProcAddress('FMOD_ChannelGroup_OverrideSpeakerMix');
    @FcFMOD_ChannelGroup_AddGroup := LocalGetProcAddress('FMOD_ChannelGroup_AddGroup');
    @FcFMOD_ChannelGroup_GetNumGroups := LocalGetProcAddress('FMOD_ChannelGroup_GetNumGroups');
    @FcFMOD_ChannelGroup_GetGroup := LocalGetProcAddress('FMOD_ChannelGroup_GetGroup');
    @FcFMOD_ChannelGroup_GetParentGroup := LocalGetProcAddress('FMOD_ChannelGroup_GetParentGroup');
    @FcFMOD_ChannelGroup_GetDSPHead := LocalGetProcAddress('FMOD_ChannelGroup_GetDSPHead');
    @FcFMOD_ChannelGroup_AddDSP := LocalGetProcAddress('FMOD_ChannelGroup_AddDSP');
    @FcFMOD_ChannelGroup_GetName := LocalGetProcAddress('FMOD_ChannelGroup_GetName');
    @FcFMOD_ChannelGroup_GetNumChannels := LocalGetProcAddress('FMOD_ChannelGroup_GetNumChannels');
    @FcFMOD_ChannelGroup_GetChannel := LocalGetProcAddress('FMOD_ChannelGroup_GetChannel');
    @FcFMOD_ChannelGroup_GetSpectrum := LocalGetProcAddress('FMOD_ChannelGroup_GetSpectrum');
    @FcFMOD_ChannelGroup_GetWaveData := LocalGetProcAddress('FMOD_ChannelGroup_GetWaveData');
    @FcFMOD_ChannelGroup_SetUserData := LocalGetProcAddress('FMOD_ChannelGroup_SetUserData');
    @FcFMOD_ChannelGroup_GetUserData := LocalGetProcAddress('FMOD_ChannelGroup_GetUserData');
    @FcFMOD_ChannelGroup_GetMemoryInfo := LocalGetProcAddress('FMOD_ChannelGroup_GetMemoryInfo');
    @FcFMOD_SoundGroup_Release := LocalGetProcAddress('FMOD_SoundGroup_Release');
    @FcFMOD_SoundGroup_GetSystemObject := LocalGetProcAddress('FMOD_SoundGroup_GetSystemObject');
    @FcFMOD_SoundGroup_SetMaxAudible := LocalGetProcAddress('FMOD_SoundGroup_SetMaxAudible');
    @FcFMOD_SoundGroup_GetMaxAudible := LocalGetProcAddress('FMOD_SoundGroup_GetMaxAudible');
    @FcFMOD_SoundGroup_SetMaxAudibleBehavior := LocalGetProcAddress('FMOD_SoundGroup_SetMaxAudibleBehavior');
    @FcFMOD_SoundGroup_GetMaxAudibleBehavior := LocalGetProcAddress('FMOD_SoundGroup_GetMaxAudibleBehavior');
    @FcFMOD_SoundGroup_SetMuteFadeSpeed := LocalGetProcAddress('FMOD_SoundGroup_SetMuteFadeSpeed');
    @FcFMOD_SoundGroup_GetMuteFadeSpeed := LocalGetProcAddress('FMOD_SoundGroup_GetMuteFadeSpeed');
    @FcFMOD_SoundGroup_SetVolume := LocalGetProcAddress('FMOD_SoundGroup_SetVolume');
    @FcFMOD_SoundGroup_GetVolume := LocalGetProcAddress('FMOD_SoundGroup_GetVolume');
    @FcFMOD_SoundGroup_Stop := LocalGetProcAddress('FMOD_SoundGroup_Stop');
    @FcFMOD_SoundGroup_GetName := LocalGetProcAddress('FMOD_SoundGroup_GetName');
    @FcFMOD_SoundGroup_GetNumSounds := LocalGetProcAddress('FMOD_SoundGroup_GetNumSounds');
    @FcFMOD_SoundGroup_GetSound := LocalGetProcAddress('FMOD_SoundGroup_GetSound');
    @FcFMOD_SoundGroup_GetNumPlaying := LocalGetProcAddress('FMOD_SoundGroup_GetNumPlaying');
    @FcFMOD_SoundGroup_SetUserData := LocalGetProcAddress('FMOD_SoundGroup_SetUserData');
    @FcFMOD_SoundGroup_GetUserData := LocalGetProcAddress('FMOD_SoundGroup_GetUserData');
    @FcFMOD_SoundGroup_GetMemoryInfo := LocalGetProcAddress('FMOD_SoundGroup_GetMemoryInfo');
    @FcFMOD_DSP_Release := LocalGetProcAddress('FMOD_DSP_Release');
    @FcFMOD_DSP_GetSystemObject := LocalGetProcAddress('FMOD_DSP_GetSystemObject');
    @FcFMOD_DSP_AddInput := LocalGetProcAddress('FMOD_DSP_AddInput');
    @FcFMOD_DSP_DisconnectFrom := LocalGetProcAddress('FMOD_DSP_DisconnectFrom');
    @FcFMOD_DSP_DisconnectAll := LocalGetProcAddress('FMOD_DSP_DisconnectAll');
    @FcFMOD_DSP_Remove := LocalGetProcAddress('FMOD_DSP_Remove');
    @FcFMOD_DSP_GetNumInputs := LocalGetProcAddress('FMOD_DSP_GetNumInputs');
    @FcFMOD_DSP_GetNumOutputs := LocalGetProcAddress('FMOD_DSP_GetNumOutputs');
    @FcFMOD_DSP_GetInput := LocalGetProcAddress('FMOD_DSP_GetInput');
    @FcFMOD_DSP_GetOutput := LocalGetProcAddress('FMOD_DSP_GetOutput');
    @FcFMOD_DSP_SetActive := LocalGetProcAddress('FMOD_DSP_SetActive');
    @FcFMOD_DSP_GetActive := LocalGetProcAddress('FMOD_DSP_GetActive');
    @FcFMOD_DSP_SetBypass := LocalGetProcAddress('FMOD_DSP_SetBypass');
    @FcFMOD_DSP_GetBypass := LocalGetProcAddress('FMOD_DSP_GetBypass');
    @FcFMOD_DSP_SetSpeakerActive := LocalGetProcAddress('FMOD_DSP_SetSpeakerActive');
    @FcFMOD_DSP_GetSpeakerActive := LocalGetProcAddress('FMOD_DSP_GetSpeakerActive');
    @FcFMOD_DSP_Reset := LocalGetProcAddress('FMOD_DSP_Reset');
    @FcFMOD_DSP_SetParameter := LocalGetProcAddress('FMOD_DSP_SetParameter');
    @FcFMOD_DSP_GetParameter := LocalGetProcAddress('FMOD_DSP_GetParameter');
    @FcFMOD_DSP_GetNumParameters := LocalGetProcAddress('FMOD_DSP_GetNumParameters');
    @FcFMOD_DSP_GetParameterInfo := LocalGetProcAddress('FMOD_DSP_GetParameterInfo');
    @FcFMOD_DSP_ShowConfigDialog := LocalGetProcAddress('FMOD_DSP_ShowConfigDialog');
    @FcFMOD_DSP_GetInfo := LocalGetProcAddress('FMOD_DSP_GetInfo');
    @FcFMOD_DSP_GetType := LocalGetProcAddress('FMOD_DSP_GetType');
    @FcFMOD_DSP_SetDefaults := LocalGetProcAddress('FMOD_DSP_SetDefaults');
    @FcFMOD_DSP_GetDefaults := LocalGetProcAddress('FMOD_DSP_GetDefaults');
    @FcFMOD_DSP_SetUserData := LocalGetProcAddress('FMOD_DSP_SetUserData');
    @FcFMOD_DSP_GetUserData := LocalGetProcAddress('FMOD_DSP_GetUserData');
    @FcFMOD_DSP_GetMemoryInfo := LocalGetProcAddress('FMOD_DSP_GetMemoryInfo');
    @FcFMOD_DSPConnection_GetInput := LocalGetProcAddress('FMOD_DSPConnection_GetInput');
    @FcFMOD_DSPConnection_GetOutput := LocalGetProcAddress('FMOD_DSPConnection_GetOutput');
    @FcFMOD_DSPConnection_SetMix := LocalGetProcAddress('FMOD_DSPConnection_SetMix');
    @FcFMOD_DSPConnection_GetMix := LocalGetProcAddress('FMOD_DSPConnection_GetMix');
    @FcFMOD_DSPConnection_SetLevels := LocalGetProcAddress('FMOD_DSPConnection_SetLevels');
    @FcFMOD_DSPConnection_GetLevels := LocalGetProcAddress('FMOD_DSPConnection_GetLevels');
    @FcFMOD_DSPConnection_SetUserData := LocalGetProcAddress('FMOD_DSPConnection_SetUserData');
    @FcFMOD_DSPConnection_GetUserData := LocalGetProcAddress('FMOD_DSPConnection_GetUserData');
    @FcFMOD_DSPConnection_GetMemoryInfo := LocalGetProcAddress('FMOD_DSPConnection_GetMemoryInfo');
    @FcFMOD_Geometry_Release := LocalGetProcAddress('FMOD_Geometry_Release');
    @FcFMOD_Geometry_AddPolygon := LocalGetProcAddress('FMOD_Geometry_AddPolygon');
    @FcFMOD_Geometry_GetNumPolygons := LocalGetProcAddress('FMOD_Geometry_GetNumPolygons');
    @FcFMOD_Geometry_GetMaxPolygons := LocalGetProcAddress('FMOD_Geometry_GetMaxPolygons');
    @FcFMOD_Geometry_GetPolygonNumVertices := LocalGetProcAddress('FMOD_Geometry_GetPolygonNumVertices');
    @FcFMOD_Geometry_SetPolygonVertex := LocalGetProcAddress('FMOD_Geometry_SetPolygonVertex');
    @FcFMOD_Geometry_GetPolygonVertex := LocalGetProcAddress('FMOD_Geometry_GetPolygonVertex');
    @FcFMOD_Geometry_SetPolygonAttributes := LocalGetProcAddress('FMOD_Geometry_SetPolygonAttributes');
    @FcFMOD_Geometry_GetPolygonAttributes := LocalGetProcAddress('FMOD_Geometry_GetPolygonAttributes');
    @FcFMOD_Geometry_SetActive := LocalGetProcAddress('FMOD_Geometry_SetActive');
    @FcFMOD_Geometry_GetActive := LocalGetProcAddress('FMOD_Geometry_GetActive');
    @FcFMOD_Geometry_SetRotation := LocalGetProcAddress('FMOD_Geometry_SetRotation');
    @FcFMOD_Geometry_GetRotation := LocalGetProcAddress('FMOD_Geometry_GetRotation');
    @FcFMOD_Geometry_SetPosition := LocalGetProcAddress('FMOD_Geometry_SetPosition');
    @FcFMOD_Geometry_GetPosition := LocalGetProcAddress('FMOD_Geometry_GetPosition');
    @FcFMOD_Geometry_SetScale := LocalGetProcAddress('FMOD_Geometry_SetScale');
    @FcFMOD_Geometry_GetScale := LocalGetProcAddress('FMOD_Geometry_GetScale');
    @FcFMOD_Geometry_Save := LocalGetProcAddress('FMOD_Geometry_Save');
    @FcFMOD_Geometry_SetUserData := LocalGetProcAddress('FMOD_Geometry_SetUserData');
    @FcFMOD_Geometry_GetUserData := LocalGetProcAddress('FMOD_Geometry_GetUserData');
    @FcFMOD_Geometry_GetMemoryInfo := LocalGetProcAddress('FMOD_Geometry_GetMemoryInfo');
    @FcFMOD_Reverb_Release := LocalGetProcAddress('FMOD_Reverb_Release');
    @FcFMOD_Reverb_Set3DAttributes := LocalGetProcAddress('FMOD_Reverb_Set3DAttributes');
    @FcFMOD_Reverb_Get3DAttributes := LocalGetProcAddress('FMOD_Reverb_Get3DAttributes');
    @FcFMOD_Reverb_SetProperties := LocalGetProcAddress('FMOD_Reverb_SetProperties');
    @FcFMOD_Reverb_GetProperties := LocalGetProcAddress('FMOD_Reverb_GetProperties');
    @FcFMOD_Reverb_SetActive := LocalGetProcAddress('FMOD_Reverb_SetActive');
    @FcFMOD_Reverb_GetActive := LocalGetProcAddress('FMOD_Reverb_GetActive');
    @FcFMOD_Reverb_SetUserData := LocalGetProcAddress('FMOD_Reverb_SetUserData');
    @FcFMOD_Reverb_GetUserData := LocalGetProcAddress('FMOD_Reverb_GetUserData');
    @FcFMOD_Reverb_GetMemoryInfo := LocalGetProcAddress('FMOD_Reverb_GetMemoryInfo');
    { Initialisation systme }
    { (1) Crer le systme }
    Res := fmod.FcFMOD_System_Create(PFMODSystem);
    if Res <> FMOD_OK then
      raise Exception.Create('Erreur de cration du systme');
    { (2) Initialiser le systme (2 = deux voies a suffit) }
    Res := fmod.FcFMOD_System_Init(PFMODSystem, 32, FMOD_INIT_NORMAL, nil);
    if Res <> FMOD_OK then
      raise Exception.Create('Erreur initialisation du systme');
    { (3) Si tout est ok, se souvenir qui faut le librer : }
    PFMODSystemToRelease := True;
  end;

finalization
  if Handle<>0 then
  begin
    if PFMODSystemToRelease then
    begin
      Res := fmod.FcFMOD_System_Release(PFMODSystem);
      if Res <> FMOD_OK then
        ShowMessage('Erreur libration FMOD');
    end;
    FreeLibrary(Handle);
  end;
end.
